// Bongo.h: interface for the CBongo class.
//
//////////////////////////////////////////////////////////////////////


//General Notes - Bongo 1.0 SDK and Bonus Tools source.
/*

  Use #include "bongo.h" to add support for Bongo in a Rhino 3.0 SDK project

  Bongo cannot be started by a Bongo plug-in from the plug-in manager.  It is therefore important
  that you don't rely on instantiating any CBongo... object during your plug-in's OnLoadPlugin 
  function (or any constructors on globals or similar).

  You can safely instantiate a CBongo... object once you have called "CBongo::Start()" and received
  a positive response.

  The CBongoCommand class, which inherits from CRhinoScriptCommand does this checking for you so you.
  Instead of overriding the RunCommand function, override RunBongoCommand to gain this functionality.

  Constructors of CBongo... objects will throw CBongoException objects in the case that Bongo is not
  correctly loaded.  However, if you always call "Start" before instantiating objects, the SDK
  guarantees that this exception will not be thrown.

  It is strongly recommended that you install the BongoSDK.dll file into the same folder as your
  RHP file.  You are free to redistribute the BongoSDK.dll file in any way you need.

  This SDK is currently in beta.  Please use the Bongo newsgroup at news://news.mcneel.com/bongo for
  technical support.

  Please see the source code for the Bongo 1.0 Bonus Tools for more information on how to use this SDK.

  */






#if !defined(AFX_BONGO_H__DB1645D3_BD3C_4A82_89A6_823FDD39C9E3__INCLUDED_)
#define AFX_BONGO_H__DB1645D3_BD3C_4A82_89A6_823FDD39C9E3__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define BO_SDK_API_EXP	__declspec(dllexport)

#include "BongoDoc.h"
#include "BongoVariant.h"
#include "BongoEventWatcher.h"
#include "BongoView.h"
#include "BongoObject.h"
#include "BongoKeyframePtr.h"
#include "BongoKeyframe.h"
#include "BongoObjectKeyframe.h"
#include "BongoViewKeyframe.h"
#include "BongoConstraintParameterKeyframe.h"
#include "BongoObjectIterator.h"
#include "BongoException.h"
#include "BongoCommand.h"

class CBongoEventWatcher;

struct Bongo
{
	enum ErrorCode
	{
		eSuccess = 0,
		eInvalidParameter =1,
		eRhinoObjectLookupFailure = 2,	// object uuid doesnt exist in active document
		eNoAnimationDataOnObject = 3,
		eErrorCreatingKeyframeIterator = 4,
		eInvalidKeyframeType = 5,
		eNoKeyframeAtPosition = 6,
		eErrorCastingObjectKeyframe = 7,
		eErrorCastingConstrainedParameterKeyframe = 8,
		eTargetUuidEqualsSourceUuid = 9,
		eTargetIsNotAValidPath = 10,
		eInvalidConstraintType = 11,
		eObjectAlreadyConstrainedToTarget = 12,
		eInvalidOrNoParent = 13,
		eParentUuidEqualsSourceUuid = 14,
		eMutualParentChildRelation = 15,
		eViewportNameDoesNotExist = 16,
		eNoAnimationDataOnView = 17,
		eViewNotAnimated = 18,
		eObjectAlreadyExisting = 19,
		eRhinoMainMenuNotFound = 20,
		eBongoMenuNotFound = 21,
	};
};

class BO_SDK_API_EXP CBongo  
{
public:
	CBongo();
	virtual ~CBongo();

public:
	static bool StartBongo();

public:
///////////////////////////////////////////////
// Bongo rhp info
	bool		IsRelease() const;
	bool		IsEvaluation() const;
	bool		IsBeta() const;
	bool		IsNetworkNode() const;

	int			EvalSaveRemaining() const;

	DWORD		ExpiryDate() const;
	DWORD		BuildDate() const;
	DWORD		LanguageID() const;
	
	bool		Version( int& iMajor, int& iMinor, int& iSR, int& iBeta ) const;

	ON_wString	InstallPath() const;
	ON_wString	LicenseType() const;
	ON_wString	UserName() const;
	ON_wString	UserOrganization() const;

	HMENU		BongoMenu();


///////////////////////////////////////////////
// Localized strings

	ON_wString	String( DWORD dwCode ) const;


///////////////////////////////////////////////
// Timeline stuff

	void	StartPreview() const;
	void	StopPreview() const;
	void	PausePreview() const;

	bool	ShowTickNumber() const;
	void	SetShowTickNumbers(bool b);

	bool	RedrawOnlyActiveView() const;
	void	SetRedrawOnlyActiveView(bool b);

	bool	ShowSplashScreen() const;
	void	SetShowSplashScreen(bool b);

	bool	UseSliderBitmaps() const;
	void	SetUseSliderBitmaps(bool b);

	bool	SmoothScrubbing() const;
	void	SetSmoothScrubbing(bool b);

	bool	IsCompactMode() const;
	void	SetCompactMode(bool b);

///////////////////////////////////////////////
// Others

	bool	IsLoggingEnabled() const;
	void	EnableLogging(bool b);	// restart Bongo to activate changes

	int		EditBoxDigits() const;
	bool	SetEditBoxDigits(int iDigits);

	bool	RegisterCommand( const wchar_t* wszLocalCommandName );
	bool	RegisterCommand( const CRhinoCommand* pCommand );

	bool	RegisterBongoEventWatcher( CBongoEventWatcher* pEvent );
	bool	UnRegisterBongoEventWatcher( CBongoEventWatcher* pEvent );

	Bongo::ErrorCode	LastError();

	CBongoVariant::BOVARTYPE ParameterType( GUID param );

};

#endif // !defined(AFX_BONGO_H__DB1645D3_BD3C_4A82_89A6_823FDD39C9E3__INCLUDED_)
