// BBTDeleteTimelineTicksCMD.cpp: implementation of the CBBTDeleteTimelineTicksCMD class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BBTApp.h"
#include "BBTDeleteTimelineTicksCMD.h"
#include "BBTMoveViewKeyframeRangeCmd.h"
#include "BBTMoveObjectKeyframeRangeCmd.h"
#include "BBTObjectWrapper.h"
#include "bbtmain.h"

extern CBBTMain bbt;

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

static class CBBTDeleteTimelineTicksCMD theBBTDeleteTimelineTicks;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBBTDeleteTimelineTicksCMD::CBBTDeleteTimelineTicksCMD()
{
	wcsncpy( m_wszLocalCommandName, ThePlugIn().LocalizedString(10), BBT_STRING_BUFFER);
}

CBBTDeleteTimelineTicksCMD::~CBBTDeleteTimelineTicksCMD()
{

}

UUID CBBTDeleteTimelineTicksCMD::CommandUUID()
{
	// {6B5029F1-C66B-4b0a-9777-76DED7EE5583}
	static const GUID uuidCommand = 
	{ 0x6b5029f1, 0xc66b, 0x4b0a, { 0x97, 0x77, 0x76, 0xde, 0xd7, 0xee, 0x55, 0x83 } };


	return uuidCommand;
}
	
void CBBTDeleteTimelineTicksCMD::DoHelp()
{
	
}

const wchar_t* CBBTDeleteTimelineTicksCMD::EnglishCommandName()
{
	return L"BongoBonusDeleteTimelineTicks"; 
}

const wchar_t* CBBTDeleteTimelineTicksCMD::LocalCommandName() const 
{ 
	return m_wszLocalCommandName;
}

CRhinoCommand::result CBBTDeleteTimelineTicksCMD::RunBongoCommand( const CRhinoCommandContext& context )
{
	bbt.WakeUp();

	int iFrom, iNumberOfTicks;

	if(!CommandLineInput(iFrom, iNumberOfTicks))
		return CRhinoCommand::cancel;

	CArray<const CRhinoObject*, const CRhinoObject*> aObjects;
	CArray<CRhinoViewport*, CRhinoViewport*> aViews;

	CBBTObjectWrapper ow;
	ow.CollectAnimatedObjects(aObjects);
	CollectViews(aViews);

	DeleteViewKeyframes(iFrom, iNumberOfTicks, aViews);
	DeleteObjectKeyframes(iFrom, iNumberOfTicks, aObjects);

	CBongoDoc* pBongoDoc = 0;
	if(!CBongoDoc::DocumentObject(pBongoDoc) || !pBongoDoc)
	{
		return CRhinoCommand::failure;
	}

	int iTimelineTicks = pBongoDoc->TimelineTicks();

	delete pBongoDoc;
	pBongoDoc = 0;

	MoveViewKeyframeRangeCmd().Flush();
	MoveObjectKeyframeRangeCmd().FlushObjects();

	int iViewData = MoveViewKeyframeRangeCmd().CreateViewData(aViews);
	int iObjectData = MoveObjectKeyframeRangeCmd().CreateObjectData(aObjects);

	if(iViewData > 0)
	{
		if(!MoveViewKeyframeRangeCmd().MoveKeyframes(iFrom+iNumberOfTicks,iTimelineTicks, -iNumberOfTicks))
		{
			delete pBongoDoc;
			return CRhinoCommand::cancel;
		}
	}

	if(iObjectData > 0)
	{
		if(!MoveObjectKeyframeRangeCmd().MoveKeyframes(iFrom+iNumberOfTicks,iTimelineTicks, -iNumberOfTicks))
		{
			delete pBongoDoc;
			return CRhinoCommand::cancel;
		}
	}

	if(iObjectData > 0)
	{
		MoveObjectKeyframeRangeCmd().SetKeyframes();
	}

	if(iViewData > 0)
	{
		MoveViewKeyframeRangeCmd().SetKeyframes();
	}

	if(!CBongoDoc::DocumentObject(pBongoDoc) || !pBongoDoc)
	{
		return CRhinoCommand::failure;
	}

	pBongoDoc->SetTimelineTicks(iTimelineTicks - iNumberOfTicks+1);
	pBongoDoc->RefreshScene();
	pBongoDoc->RefreshTimeline();
	
	delete pBongoDoc;

	return CRhinoCommand::success;
}

bool CBBTDeleteTimelineTicksCMD::CommandLineInput(int& iFrom, int& iTo)
{
	if(!GetCommandLineInteger( 2101, iFrom,0,BONGO_MAX_TIMELINE_RANGE-2,0,true,true))
		return false;

	if(!GetCommandLineInteger( 2105, iTo,1,BONGO_MAX_TIMELINE_RANGE-1,1,true,false))
		return false;

	return true;
}

bool  CBBTDeleteTimelineTicksCMD::CollectViews(CArray<CRhinoViewport*, CRhinoViewport*>& aViews)
{
	CRhinoDoc* pDoc = RhinoApp().ActiveDoc();
	if(!pDoc)	return false;

	ON_SimpleArray <CRhinoView*> aRhinoView;
	pDoc->GetViewList(aRhinoView);
	
	for(int i=0; i<aRhinoView.Count(); i++)
	{
		aViews.Add(&aRhinoView[i]->Viewport());
	}

	return true;
}

void CBBTDeleteTimelineTicksCMD::DeleteObjectKeyframes(int iFrom, int iTo, CArray<const CRhinoObject*,
													   const CRhinoObject*>& aObjects)
{
	for(int i=0; i<aObjects.GetSize(); i++)
	{
		CBongoObject Object(aObjects[i]);

		for(int j=iFrom; j<=iTo; j++)
		{
			Object.DeleteKeyframe((double)j, BONGO_WHOLE_KEYFRAME,false);
		}
	}
}

void CBBTDeleteTimelineTicksCMD::DeleteViewKeyframes(int iFrom, int iTo,
													 CArray<CRhinoViewport*, CRhinoViewport*>& aViews)
{
	CBongoView* pBongoView = 0;
	if(!CBongoView::ViewObject(pBongoView) || !pBongoView)
	{
		delete pBongoView;
		pBongoView = 0;
		return;
	}	

	for(int i=0; i<aViews.GetSize(); i++)
	{
		pBongoView->SetViewName(aViews[i]->Name());
		if(pBongoView->IsValid() && pBongoView->IsAnimated() && pBongoView->FirstKeyframe())
		{
			for(int j=iFrom; j<=iTo; j++)
			{
				pBongoView->DeleteKeyframe((double)j,false);
			}
		}
	}

	delete pBongoView;
}