// BBTEventWatcher.cpp: implementation of the CBBTEventWatcher class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BBTApp.h"
#include "BBTEventWatcher.h"
#include "BBTTimelineExtension.h"
#include "bbtmain.h"
#include "BBTEmailNotifierMgr.h"
#include "BBTRenderAnimationFileCollectionMgr.h"
#include "BBTGraphicDisplayControlBar.h"

extern CBBTMain bbt;


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBBTEventWatcher::CBBTEventWatcher()
{
	m_pEmailNotifierMgr = new CBBTEmailNotifierMgr;
	m_pRenderFileCollection = new CBBTRenderAnimationFileCollectionMgr;
	m_bEnableRenderAnimationFileCollectionEvents = false;
	m_bTimerEnabled = false;
}

CBBTEventWatcher::~CBBTEventWatcher()
{
	delete m_pEmailNotifierMgr;
	m_pEmailNotifierMgr = 0;

	delete m_pRenderFileCollection;
	m_pRenderFileCollection = 0;
}	

void CBBTEventWatcher::OnBongoSettingsChanged()
{

}

void CBBTEventWatcher::OnBongoObjectAnimationChanged( UUID uuidObject )
{

}

void CBBTEventWatcher::OnBongoViewAnimationChanged( const WCHAR* wszName )
{

}

void CBBTEventWatcher::OnSliderPositionWillChange( double dOldPos, double dNewPos )
{
	bbt.GraphicDisplayControlBar().SetSkipNextSelectionChangedNotification(true);
}

void CBBTEventWatcher::OnSliderPositionChanged( double dOldPos, double dNewPos )
{
	bbt.GraphicDisplayControlBar().SetSkipNextSelectionChangedNotification(false);
	bbt.GraphicDisplayControlBar().UpdateSliderPos(dNewPos);
}

void CBBTEventWatcher::OnRenderAnimationWillStart(int iCommandStyle)
{
	m_pEmailNotifierMgr->Reset();

	bool bSilent = true;
	if(iCommandStyle == CRhinoCommandContext::interactive_ui)
	{
		bSilent = false;
	}

	m_pEmailNotifierMgr->Initialize(bSilent);
}

void CBBTEventWatcher::OnRenderAnimationCanceled(const WCHAR* wszOutputFile)
{
	USES_CONVERSION;

	m_pEmailNotifierMgr->AnimationCanceled(W2A(wszOutputFile));

	if(m_bEnableRenderAnimationFileCollectionEvents)
	{
		m_bEnableRenderAnimationFileCollectionEvents = false;	
	}
}

void CBBTEventWatcher::OnAnimationFrameWritten( const WCHAR* wszFileName, int iFrameNumber,
											   double dTimelinePosition )
{
	USES_CONVERSION;

	m_pEmailNotifierMgr->FrameRendered(W2A(wszFileName), iFrameNumber);
}

void CBBTEventWatcher::OnEncodingWillStart( const WCHAR* wszFirstFileInSequence, const WCHAR* wszOutputFile,
	int iSpeedInFPS, int iStartFrame, int iStopFrame )
{

}

void CBBTEventWatcher::OnRenderAnimationFinished(const WCHAR* wszOutputFile)
{
	USES_CONVERSION;
	
	m_pEmailNotifierMgr->AnimationFinished(W2A(wszOutputFile));

	if(m_bEnableRenderAnimationFileCollectionEvents)
	{
		if(!m_pRenderFileCollection->Render())
		{
			m_bEnableRenderAnimationFileCollectionEvents = false;
		}
	}
}

void CBBTEventWatcher::OnAnimationModeChanged( BOOL bAnimationMode )
{

}

void CBBTEventWatcher::OnPreviewStarted()
{
	bbt.GraphicDisplayControlBar().SetSkipNextSelectionChangedNotification(true);
}

void CBBTEventWatcher::OnPreviewStopped()
{
	bbt.GraphicDisplayControlBar().SetSkipNextSelectionChangedNotification(false);
}	

void CBBTEventWatcher::OnPreviewPaused()
{
	bbt.GraphicDisplayControlBar().SetSkipNextSelectionChangedNotification(false);
}

void CBBTEventWatcher::OnTimelineTicksChanged()
{

}

void CBBTEventWatcher::OnContextMenuDisplay(HMENU hMenu,
											const CONTEXT_MENU_TYPE type,
											const CONTEXT_MENU_VARIANT* pVar,
											const int iCmdFirst,
											const int iCmdLast)
{
	switch(type)
	{
	case timeline:
		bbt.TimelineExtension().OnContextMenuDisplay(hMenu, iCmdFirst, iCmdLast);
		break;

	}
}

void CBBTEventWatcher::OnContextMenuItemInvoke(const CONTEXT_MENU_TYPE type,
												const CONTEXT_MENU_VARIANT* pVar,
												const int iCmdCode,
												const int iCmdFirst)
{
	switch(type)
	{
	case timeline:
		bbt.TimelineExtension().OnContextMenuItemInvoked(iCmdCode, iCmdFirst, pVar->dVar);
		break;

	}
}

void CBBTEventWatcher::EnableRenderAnimationFileCollectionEvents(bool bEnable)
{
	KillTimer(0, BBT_RENDER_FILE_COLLECTION_TIMER);

	if(bEnable)
	{
		if(!m_bEnableRenderAnimationFileCollectionEvents)
		{
			if(m_pRenderFileCollection->Initialize())
			{
				m_bEnableRenderAnimationFileCollectionEvents = bEnable;

				m_pRenderFileCollection->Render();
			}
		}
	}
	else
	{
		m_bEnableRenderAnimationFileCollectionEvents = false;	
	}
}

void CALLBACK RenderFileCollectionTimer(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime )
{
	bbt.BongoEventWatcher().EnableRenderAnimationFileCollectionEvents(true);
	KillTimer(0, idEvent);
}