// BBTInsertTimelineTicksCMD.cpp: implementation of the CBBTInsertTimelineTicksCMD class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BBTApp.h"
#include "BBTInsertTimelineTicksCMD.h"
#include "BBTMoveViewKeyframeRangeCmd.h"
#include "BBTMoveObjectKeyframeRangeCmd.h"
#include "BBTObjectWrapper.h"
#include "bbtmain.h"

extern CBBTMain bbt;

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


static class CBBTInsertTimelineTicksCMD theBBTInsertTimelineTicks;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBBTInsertTimelineTicksCMD::CBBTInsertTimelineTicksCMD()
{
	wcsncpy( m_wszLocalCommandName, ThePlugIn().LocalizedString(7), BBT_STRING_BUFFER);
}

CBBTInsertTimelineTicksCMD::~CBBTInsertTimelineTicksCMD()
{

}

UUID CBBTInsertTimelineTicksCMD::CommandUUID()
{
	// {79FE63C5-11D8-4cfa-8C92-F37A6A139E76}
	static const GUID uuidCommand = 
	{ 0x79fe63c5, 0x11d8, 0x4cfa, { 0x8c, 0x92, 0xf3, 0x7a, 0x6a, 0x13, 0x9e, 0x76 } };

	return uuidCommand;
}
	
void CBBTInsertTimelineTicksCMD::DoHelp()
{
	
}

const wchar_t* CBBTInsertTimelineTicksCMD::EnglishCommandName()
{
	return L"BongoBonusInsertTimelineTicks"; 
}

const wchar_t* CBBTInsertTimelineTicksCMD::LocalCommandName() const 
{ 
	return m_wszLocalCommandName;
}

CRhinoCommand::result CBBTInsertTimelineTicksCMD::RunBongoCommand( const CRhinoCommandContext& context )
{
	bbt.WakeUp();

	CBongoDoc* pBongoDoc = 0;
	if(!CBongoDoc::DocumentObject(pBongoDoc) || !pBongoDoc)
	{
		return CRhinoCommand::failure;
	}

	int iTimelineTicks = pBongoDoc->TimelineTicks();


	delete pBongoDoc;
	pBongoDoc = 0;

	int iFrom, iRange;

	if(!CommandLineInput(iFrom, iRange))
		return CRhinoCommand::cancel;

	CArray<const CRhinoObject*, const CRhinoObject*> aObjects;
	CArray<CRhinoViewport*, CRhinoViewport*> aViews;

	CBBTObjectWrapper ow;
	ow.CollectAnimatedObjects(aObjects);
	CollectViews(aViews);

	MoveViewKeyframeRangeCmd().Flush();
	MoveObjectKeyframeRangeCmd().FlushObjects();

	int iViewData = MoveViewKeyframeRangeCmd().CreateViewData(aViews);
	int iObjectData = MoveObjectKeyframeRangeCmd().CreateObjectData(aObjects);

	if(iViewData > 0)
	{
		if(!MoveViewKeyframeRangeCmd().MoveKeyframes(iFrom, iFrom+iTimelineTicks, iRange))
		{
			delete pBongoDoc;
			return CRhinoCommand::cancel;
		}
	}

	if(iObjectData > 0)
	{
		if(!MoveObjectKeyframeRangeCmd().MoveKeyframes(iFrom, iFrom+iTimelineTicks, iRange))
		{
			delete pBongoDoc;
			return CRhinoCommand::cancel;
		}
	}

	if(iObjectData > 0)
	{
		MoveObjectKeyframeRangeCmd().SetKeyframes();
	}

	if(iViewData > 0)
	{
		MoveViewKeyframeRangeCmd().SetKeyframes();
	}


	if(CBongoDoc::DocumentObject(pBongoDoc) && pBongoDoc)
	{
		pBongoDoc->SetTimelineTicks(iTimelineTicks +iRange);
		pBongoDoc->RefreshScene();
		pBongoDoc->RefreshTimeline();
	}

	delete pBongoDoc;
	pBongoDoc = 0;

	return CRhinoCommand::success;
}

bool CBBTInsertTimelineTicksCMD::CommandLineInput(int& iFrom, int& iRange)
{
	if(!GetCommandLineInteger( 2100, iFrom,0,BONGO_MAX_TIMELINE_RANGE-2,0,true,true))
		return false;

	if(!GetCommandLineInteger( 2105, iRange,1,BONGO_MAX_TIMELINE_RANGE-iFrom-1,1,true,false))
		return false;

	return true;
}

bool  CBBTInsertTimelineTicksCMD::CollectViews(CArray<CRhinoViewport*, CRhinoViewport*>& aViews)
{
	CRhinoDoc* pDoc = RhinoApp().ActiveDoc();
	if(!pDoc)	return false;

	ON_SimpleArray <CRhinoView*> aRhinoView;
	pDoc->GetViewList(aRhinoView);
	
	for(int i=0; i<aRhinoView.Count(); i++)
	{
		aViews.Add(&aRhinoView[i]->Viewport());
	}

	return true;
}