// BBTMoveObjectKeyframeRangeCmd.cpp: implementation of the CBBTMoveObjectKeyframeRangeCmd class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BBTApp.h"
#include "BBTMoveObjectKeyframeRangeCmd.h"
#include "BBTObjectWrapper.h"
#include "bbtmain.h"

extern CBBTMain bbt;


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

static class CBBTMoveObjectKeyframeRangeCmd theBBTMoveObjectKeyframeRangeCmd;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBBTMoveObjectKeyframeRangeCmd::CBBTMoveObjectKeyframeRangeCmd()
{
	wcsncpy( m_wszLocalCommandName, ThePlugIn().LocalizedString(8), BBT_STRING_BUFFER);
}

CBBTMoveObjectKeyframeRangeCmd::~CBBTMoveObjectKeyframeRangeCmd()
{
	FlushObjects();
}

UUID CBBTMoveObjectKeyframeRangeCmd::CommandUUID()
{
	// {86038135-DD09-4fb0-AA49-B04641E77AD8}
	static const GUID uuidCommand = 
	{ 0x86038135, 0xdd09, 0x4fb0, { 0xaa, 0x49, 0xb0, 0x46, 0x41, 0xe7, 0x7a, 0xd8 } };

	return uuidCommand;
}
	
void CBBTMoveObjectKeyframeRangeCmd::DoHelp()
{
	
}

const wchar_t* CBBTMoveObjectKeyframeRangeCmd::EnglishCommandName()
{
	return L"BongoBonusMoveObjectKeyframeRange"; 
}

const wchar_t* CBBTMoveObjectKeyframeRangeCmd::LocalCommandName() const 
{ 
	return m_wszLocalCommandName;
}

CRhinoCommand::result CBBTMoveObjectKeyframeRangeCmd::RunBongoCommand( const CRhinoCommandContext& context )
{
	bbt.WakeUp();

	FlushObjects();

	CBongo bo;
	WCHAR wsz_Prompt[BBT_STRING_BUFFER];
	wcsncpy(wsz_Prompt, bo.String(8001),BBT_STRING_BUFFER);

	CRhinoGetObject go;
	go.AcceptNothing();
	go.EnablePreSelect(true);
	go.SetCommandPrompt(wsz_Prompt);
	go.GetObjects(1,0);
	if( go.Result() != CRhinoGet::object )
			return CRhinoCommand::cancel;

	if(CreateObjectData(&go) <= 0)
	{
		BBTMsgBox(ThePlugIn().LocalizedString(2032));
		return CRhinoCommand::failure;
	}
	
	int iFrom, iTo, iRange;

	if(!CommandLineInput(iFrom, iTo, iRange))
	{
		FlushObjects();
		return CRhinoCommand::cancel;
	}	

	if(!MoveKeyframes(iFrom, iTo, iRange))
	{
		FlushObjects();
		return CRhinoCommand::cancel;
	}

	SetKeyframes();

	FlushObjects();

	CBongoDoc* pBongoDoc = 0;
	if(CBongoDoc::DocumentObject(pBongoDoc) && pBongoDoc)
	{
		pBongoDoc->RefreshTimeline();
		pBongoDoc->RefreshScene();
	}
	
	delete pBongoDoc;

	return CRhinoCommand::success;
}

int CBBTMoveObjectKeyframeRangeCmd::CreateObjectData(CArray<const CRhinoObject*,const CRhinoObject*>& aObjects)
{
	CBongoKeyframePtr BongoPtrKf;
	CBongoObjectKeyframe* pBongoKf;

	for(int i=0; i<aObjects.GetSize(); i++)
	{
		CBongoObject* pBongoObject = new CBongoObject(aObjects[i]);
		
		if(!pBongoObject || !pBongoObject->IsValid() || !pBongoObject->IsAnimated()
			|| !pBongoObject->FirstKeyframe())
		{
			delete pBongoObject;
			pBongoObject = 0;
		}
		else
		{
			OBJECT_DATA* pData = new OBJECT_DATA;
			if(pData)
			{
				pData->pBongoObject = pBongoObject;

				for(BongoPtrKf = pBongoObject->FirstKeyframe(); BongoPtrKf;
					BongoPtrKf = pBongoObject->NextKeyframe())
				{
					pBongoKf = (CBongoObjectKeyframe*)BongoPtrKf.Pointee();
					if(pBongoKf)
					{
						pData->aKFs.Add(new CBongoObjectKeyframe(*pBongoKf));
					}
				}	

				m_aObjects.Add(pData);
			}
		}
	}

	return m_aObjects.GetSize();
}

int CBBTMoveObjectKeyframeRangeCmd::CreateObjectData(CRhinoGetObject* pObjectSelection)
{
	if(!pObjectSelection)
		return 0;

	CArray<const CRhinoObject*,const CRhinoObject*> aObjects;	

	for(int i=0; i<pObjectSelection->ObjectCount(); i++)
	{
		aObjects.Add(pObjectSelection->Object(i).Object());
	}

	return CreateObjectData(aObjects);
}

void CBBTMoveObjectKeyframeRangeCmd::FlushObjects()
{
	for(int i=m_aObjects.GetSize()-1; i>=0; i--)
	{
		delete m_aObjects[i]->pBongoObject;
		m_aObjects[i]->pBongoObject = 0;

		for(int j=m_aObjects[i]->aKFs.GetSize()-1; j>=0; j--)
		{
			delete m_aObjects[i]->aKFs[j];
			m_aObjects[i]->aKFs[j] = 0;
		}
		
		m_aObjects[i]->aKFs.RemoveAll();
	}

	m_aObjects.RemoveAll();
}

bool CBBTMoveObjectKeyframeRangeCmd::CommandLineInput(int& iFrom, int& iTo, int& iRange)
{
	if(!GetCommandLineInteger( 2150, iFrom,0,BONGO_MAX_TIMELINE_RANGE-1,0,true,true))
		return false;

	if(!GetCommandLineInteger( 2151, iTo,iFrom,BONGO_MAX_TIMELINE_RANGE-1,0,true,true))
		return false;

	if(!GetCommandLineInteger( 2152, iRange,-1*iFrom, BONGO_MAX_TIMELINE_RANGE-1-iTo, true, false))
		return false;

	return true;
}

bool CBBTMoveObjectKeyframeRangeCmd::MoveKeyframes(int iFrom, int iTo, int iRange)
{
	double dPos;
	bool bPopupWarningDelete = true;
	bool bPopupWarningOverwrite = true;
	CBBTObjectWrapper ow;

	for(int i=0; i<m_aObjects.GetSize();i++)
	{
		for(int j=m_aObjects[i]->aKFs.GetSize()-1; j>=0; j--)
		{
			dPos = 	m_aObjects[i]->aKFs[j]->Position();

			if(dPos >=iFrom && dPos <= iTo)
			{
				dPos += (double)iRange;
	
				m_aObjects[i]->aKFs[j]->SetPosition(dPos);

				
				if(dPos <= 0.0 || dPos > BONGO_MAX_TIMELINE_RANGE-1)
				{
					delete m_aObjects[i]->aKFs[j];
					m_aObjects[i]->aKFs[j] = 0;
					m_aObjects[i]->aKFs.RemoveAt(j);

					if(bPopupWarningDelete)
					{
						bPopupWarningDelete = false;
						
						if(IDNO == BBTMsgBox(ThePlugIn().LocalizedString(2033),
							MB_YESNO|MB_ICONQUESTION|MB_DEFBUTTON1))
						{
							return false;
						}
					}
				}
				else if(bPopupWarningOverwrite && (dPos < (double)iFrom || dPos > (double)iTo) && 
					ow.IsKeyframeAtPos(m_aObjects[i]->pBongoObject, dPos))
				{
					bPopupWarningOverwrite = false;
					
					if(IDNO == BBTMsgBox(ThePlugIn().LocalizedString(2039),
						MB_YESNO|MB_ICONQUESTION|MB_DEFBUTTON1))
					{
						return false;
					}
				}
			}
		}
	}

	return true;
}


bool CBBTMoveObjectKeyframeRangeCmd::SetKeyframes()
{
	CBBTObjectWrapper ow;

	for(int i=0; i<m_aObjects.GetSize();i++)
	{
		ow.DeleteAllKeyframes(m_aObjects[i]->pBongoObject);

		for(int j=0; j<m_aObjects[i]->aKFs.GetSize(); j++)
		{
			m_aObjects[i]->pBongoObject->SetKeyframe(m_aObjects[i]->aKFs[j], false);
		}
	}

	return true;
}

CBBTMoveObjectKeyframeRangeCmd& MoveObjectKeyframeRangeCmd()
{
	return theBBTMoveObjectKeyframeRangeCmd;
}