// BBTMoveViewKeyframeRangeCmd.cpp: implementation of the CBBTMoveViewKeyframeRangeCmd class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BBTApp.h"
#include "BBTMoveViewKeyframeRangeCmd.h"
#include "BBTViewWrapper.h"
#include "bbtmain.h"

extern CBBTMain bbt;

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

static class CBBTMoveViewKeyframeRangeCmd theBBTMoveViewKeyframeRangeCmd;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBBTMoveViewKeyframeRangeCmd::CBBTMoveViewKeyframeRangeCmd()
{
	wcsncpy( m_wszLocalCommandName, ThePlugIn().LocalizedString(9), BBT_STRING_BUFFER);
}

CBBTMoveViewKeyframeRangeCmd::~CBBTMoveViewKeyframeRangeCmd()
{
	Flush();
}

UUID CBBTMoveViewKeyframeRangeCmd::CommandUUID()
{
	// {02225F87-BBB1-42c7-A717-253261415979}
	static const GUID uuidCommand = 
	{ 0x2225f87, 0xbbb1, 0x42c7, { 0xa7, 0x17, 0x25, 0x32, 0x61, 0x41, 0x59, 0x79 } };

	return uuidCommand;
}
	
void CBBTMoveViewKeyframeRangeCmd::DoHelp()
{
	
}

const wchar_t* CBBTMoveViewKeyframeRangeCmd::EnglishCommandName()
{
	return L"BongoBonusMoveViewKeyframeRange"; 
}

const wchar_t* CBBTMoveViewKeyframeRangeCmd::LocalCommandName() const 
{ 
	return m_wszLocalCommandName;
}

CRhinoCommand::result CBBTMoveViewKeyframeRangeCmd::RunBongoCommand( const CRhinoCommandContext& context )
{
	bbt.WakeUp();

	Flush();

	CRhinoViewport* pViewport = &(RhinoApp().ActiveView()->Viewport());
	if(!pViewport)	return CRhinoCommand::failure;

	CArray<CRhinoViewport*, CRhinoViewport*> aViews;
	aViews.Add(pViewport);

	if(CreateViewData(aViews) <= 0)
	{
		BBTMsgBox(ThePlugIn().LocalizedString(2050));
		return CRhinoCommand::failure;
	}
	
	int iFrom, iTo, iRange;
	if(!CommandLineInput(iFrom, iTo, iRange))
	{
		Flush();
		return CRhinoCommand::cancel;
	}	

	if(!MoveKeyframes(iFrom, iTo, iRange))
	{
		Flush();
		return CRhinoCommand::cancel;
	}

	SetKeyframes();

	Flush();

	CBongoDoc* pBongoDoc = 0;
	if(CBongoDoc::DocumentObject(pBongoDoc) && pBongoDoc)
	{
		pBongoDoc->RefreshTimeline();
		pBongoDoc->RefreshScene();
	}
	
	delete pBongoDoc;

	return CRhinoCommand::success;
}

int CBBTMoveViewKeyframeRangeCmd::CreateViewData(CArray<CRhinoViewport*, CRhinoViewport*>& aViews)
{
	CBongoKeyframePtr BongoPtrKf;
	CBongoViewKeyframe* pBongoKf;

	CBongoView* pBongoView = 0;
	if(!CBongoView::ViewObject(pBongoView) || !pBongoView)
	{
		delete pBongoView;
		pBongoView = 0;
		return 0;
	}

	for(int i=0; i<aViews.GetSize(); i++)
	{
		pBongoView->SetViewName(aViews[i]->Name());
		
		if(pBongoView->IsValid() && pBongoView->IsAnimated() && pBongoView->FirstKeyframe())
		{
			VIEW_DATA* pData = new VIEW_DATA;
			if(pData)
			{
				pData->pViewport = aViews[i];

				for(BongoPtrKf = pBongoView->FirstKeyframe(); BongoPtrKf;
					BongoPtrKf = pBongoView->NextKeyframe())
				{
					pBongoKf = (CBongoViewKeyframe*)BongoPtrKf.Pointee();
					if(pBongoKf)
					{
						pData->aKFs.Add(new CBongoViewKeyframe(*pBongoKf));
					}
				}	

				m_aViews.Add(pData);
			}
		}
	}

	delete pBongoView;

	return m_aViews.GetSize();
}

void CBBTMoveViewKeyframeRangeCmd::Flush()
{
	for(int i=m_aViews.GetSize()-1; i>=0; i--)
	{
		m_aViews[i]->pViewport = 0;

		for(int j=m_aViews[i]->aKFs.GetSize()-1; j>=0; j--)
		{
			delete m_aViews[i]->aKFs[j];
			m_aViews[i]->aKFs[j] = 0;
		}
		
		m_aViews[i]->aKFs.RemoveAll();
	}

	m_aViews.RemoveAll();
}

bool CBBTMoveViewKeyframeRangeCmd::CommandLineInput(int& iFrom, int& iTo, int& iRange)
{
	if(!GetCommandLineInteger( 2150, iFrom,0,BONGO_MAX_TIMELINE_RANGE-1,0,true,true))
		return false;

	if(!GetCommandLineInteger( 2151, iTo,iFrom,BONGO_MAX_TIMELINE_RANGE-1,0,true,true))
		return false;

	if(!GetCommandLineInteger( 2152, iRange,-1*iFrom, BONGO_MAX_TIMELINE_RANGE-1-iTo, true, false))
		return false;

	return true;
}

bool CBBTMoveViewKeyframeRangeCmd::MoveKeyframes(int iFrom, int iTo, int iRange)
{
	double dPos;
	bool bPopupWarningDelete = true;
	bool bPopupWarningOverwrite = true;
	CBBTViewWrapper ow;

	CBongoView* pBongoView = 0;
	if(!CBongoView::ViewObject(pBongoView) || !pBongoView)
	{
		delete pBongoView;
		pBongoView = 0;
		return false;
	}


	for(int i=0; i<m_aViews.GetSize();i++)
	{
		pBongoView->SetViewName(m_aViews[i]->pViewport->Name());

		for(int j=m_aViews[i]->aKFs.GetSize()-1; j>=0; j--)
		{
			dPos = 	m_aViews[i]->aKFs[j]->Position();

			if(dPos >=iFrom && dPos <= iTo)
			{
				dPos += (double)iRange;
	
				m_aViews[i]->aKFs[j]->SetPosition(dPos);

				
				if(dPos <= 0.0 || dPos > BONGO_MAX_TIMELINE_RANGE-1)
				{
					delete m_aViews[i]->aKFs[j];
					m_aViews[i]->aKFs[j] = 0;
					m_aViews[i]->aKFs.RemoveAt(j);

					if(bPopupWarningDelete)
					{
						bPopupWarningDelete = false;
						
						if(IDNO == BBTMsgBox(ThePlugIn().LocalizedString(2033),
							MB_YESNO|MB_ICONQUESTION|MB_DEFBUTTON1))
						{
							delete pBongoView;
							return false;
						}
					}
				}
				else if(bPopupWarningOverwrite && (dPos < (double)iFrom || dPos > (double)iTo) && 
					pBongoView->Keyframe(dPos, view_keyframe, BONGO_VIEW_KEYFRAME))
				{
					bPopupWarningOverwrite = false;
					
					if(IDNO == BBTMsgBox(ThePlugIn().LocalizedString(2039),
						MB_YESNO|MB_ICONQUESTION|MB_DEFBUTTON1))
					{
						delete pBongoView;
						return false;
					}
				}
			}
		}
	}

	delete pBongoView;

	return true;
}

bool CBBTMoveViewKeyframeRangeCmd::SetKeyframes()
{
	CBongoView* pBongoView = 0;
	if(!CBongoView::ViewObject(pBongoView) || !pBongoView)
	{
		delete pBongoView;
		pBongoView = 0;
		return false;
	}

	CBBTViewWrapper ow;

	for(int i=0; i<m_aViews.GetSize();i++)
	{
		pBongoView->SetViewName(m_aViews[i]->pViewport->Name());

		ow.DeleteAllKeyframes(pBongoView);

		for(int j=0; j<m_aViews[i]->aKFs.GetSize(); j++)
		{
			pBongoView->SetKeyframe(m_aViews[i]->aKFs[j], false);
		}
	}

	delete pBongoView;

	return true;
}

CBBTMoveViewKeyframeRangeCmd& MoveViewKeyframeRangeCmd()
{
	return theBBTMoveViewKeyframeRangeCmd;
}