/////////////////////////////////////////////////////////////////////////////
// BongoBonusToolsPlugIn.cpp
//

#include "StdAfx.h"
#include "Resource.h"
#include "BBTMain.h"
#include "LBP_XMLStringTable.h"
#include "LBPRegistryWrapper.h"
#include "LBPShellFunctions.h"
#include "LBPLib.h"
#include "BBTRhinoEventWatcher.h"

extern CBBTMain bbt;

/////////////////////////////////////////////////////////////////////////////
// The plug-in object must be constructed before any plug-in
// classes derived from CRhinoCommand. The #pragma init_seg(lib)
// ensures that this happens.

#pragma warning( push )
#pragma warning( disable : 4073 )
#pragma init_seg( lib )
#pragma warning( pop )

/////////////////////////////////////////////////////////////////////////////
// The one and only CBBTPlugIn object

static CBBTPlugIn thePlugIn;

/////////////////////////////////////////////////////////////////////////////
// Returns reference to the plug-in object

CBBTPlugIn& ThePlugIn() { return thePlugIn; }

/////////////////////////////////////////////////////////////////////////////
// Rhino plug-in declaration

RHINO_PLUG_IN_DECLARE

/////////////////////////////////////////////////////////////////////////////
// Rhino plug-in developer declarations

RHINO_PLUG_IN_DEVELOPER_ORGANIZATION( L"Robert McNeel & Associates" );
RHINO_PLUG_IN_DEVELOPER_ADDRESS( L"3670 Woodland Park Avenue North\r\nSeattle WA 98103" );
RHINO_PLUG_IN_DEVELOPER_COUNTRY( L"United States" );
RHINO_PLUG_IN_DEVELOPER_PHONE( L"206-545-7000" );
RHINO_PLUG_IN_DEVELOPER_FAX( L"206-545-7321" );
RHINO_PLUG_IN_DEVELOPER_EMAIL( L"tech@mcneel.com" );
RHINO_PLUG_IN_DEVELOPER_WEBSITE( L"http://www.Bongo3d.com" );
RHINO_PLUG_IN_UPDATE_URL( L"http://www.Bongo3d.com" );

/////////////////////////////////////////////////////////////////////////////
// CBBTPlugIn definition

CBBTPlugIn::CBBTPlugIn()
{
	m_sPlugInVersion = __DATE__"  "__TIME__;
	m_sProfileSection = _T("Plug-Ins\\659DC54A-2BE4-4FC3-B7C6-E4B68FA61C66\\Settings");
	
	m_pLocalizedStrings = 0;
	m_pEnglishStrings	= 0;

	m_bInitialized = false;

	m_bStringTablesCreated = CreateStringTables();

	m_pRhinoEventWatcher = new CBBTRhinoEventWatcher;
}

CBBTPlugIn::~CBBTPlugIn()
{
	delete m_pRhinoEventWatcher;
	m_pRhinoEventWatcher = 0;
}

/////////////////////////////////////////////////////////////////////////////
// General plug-in overrides

const wchar_t* CBBTPlugIn::PlugInName() const
{
	return L"BongoBonusTools";
}

const wchar_t* CBBTPlugIn::PlugInVersion() const
{
	return m_sPlugInVersion;
}

GUID CBBTPlugIn::PlugInID() const
{
	// {659DC54A-2BE4-4FC3-B7C6-E4B68FA61C66}
	static const GUID BongoBonusToolsPlugIn_GUID =
	{ 0x659DC54A, 0x2BE4, 0x4FC3, { 0xB7, 0xC6, 0xE4, 0xB6, 0x8F, 0xA6, 0x1C, 0x66 } };
	return BongoBonusToolsPlugIn_GUID;
}

BOOL CBBTPlugIn::OnLoadPlugIn()
{
	/////////////////////////////////////////////
	// Localized string table loaded ?

	m_pRhinoEventWatcher->Register();
	m_pRhinoEventWatcher->Enable();
	
	if(!m_bStringTablesCreated)
	{
		BBTMsgBox( "Couldn't load string table", MB_OK|MB_ICONERROR );
		return FALSE;
	}

	AddMenu();

	bbt.Initialize();

	return CRhinoUtilityPlugIn::OnLoadPlugIn();
}

void CBBTPlugIn::OnUnloadPlugIn()
{
	bbt.CloseGraphicDisplayControlBar();

	RemovePlugInMenuFromRhino(m_hMenu);
	
	if(m_pEnglishStrings && m_pEnglishStrings != m_pLocalizedStrings)	
	{
		delete m_pEnglishStrings;
		m_pEnglishStrings = 0;
	}

	delete m_pLocalizedStrings;
	m_pLocalizedStrings = 0;
	
	CRhinoUtilityPlugIn::OnUnloadPlugIn();
}


bool CBBTPlugIn::CreateStringTables()
{
	DWORD dwLanguage;
	CString sPath, sFullPath;
	CLBPRegistryWrapper key;

	if( !key.GetStringValue(HKEY_LOCAL_MACHINE, BONGO_REGPATH, "InstallPath", sPath) )
		return false;

	dwLanguage = RhinoApp().AppSettings().AppearanceSettings().m_language_identifier;
	
	sFullPath.Format("%s\\Languages\\BongoBonusTools_%d.xml",sPath,dwLanguage);
	
	if(LBP_PathFileExists(sFullPath))
	{
		LBPLibSetLocale(dwLanguage);
		m_pLocalizedStrings = new CLBP_XMLStringTable( sFullPath );
	}
	else if( dwLanguage != 1033 )
	{
		sFullPath.Format( "%s\\Languages\\BongoBonusTools_%d.xml",sPath, 1033 );
		if(! LBP_PathFileExists(sFullPath) )
			return false;

		dwLanguage = 1033;

		LBPLibSetLocale(dwLanguage);
		m_pLocalizedStrings = new CLBP_XMLStringTable(sFullPath );
	}
	else
	{
		return false;
	}
	
	if( dwLanguage == 1033 )
	{
		m_pEnglishStrings = m_pLocalizedStrings;
	}
	else
	{
		sFullPath.Format( "%s\\Languages\\BongoBonusTools_%d.xml",sPath, 1033 );
		if( !LBP_PathFileExists(sFullPath) )
			return false;

		m_pEnglishStrings = new CLBP_XMLStringTable(sFullPath );
	}
	
	return true;
}

CLBPString& CBBTPlugIn::LocalizedString(DWORD dwID, bool bEnglishVersion)
{
	if(!bEnglishVersion && m_pLocalizedStrings)
	{
		if(m_pLocalizedStrings->String(dwID).CompareNoCase(L"???") != 0)
		{
			return m_pLocalizedStrings->String(dwID);
		}
	}
	
	if(m_pEnglishStrings)
	{
		return m_pEnglishStrings->String(dwID);
	}

	::MessageBox(NULL, "Bongo Bonus Tools String tables missing!", "Bongo Bonus 1.0", MB_OK | MB_ICONEXCLAMATION);

	static CLBPString sError("???");
	return sError;
}


bool CBBTPlugIn::AddMenu()
{
	AFX_MANAGE_STATE( AfxGetStaticModuleState() );

	m_hMenu = ::LoadMenu( ::AfxFindResourceHandle( MAKEINTRESOURCE(IDR_BONUS_TOOLS_MENU), RT_MENU),
			MAKEINTRESOURCE(IDR_BONUS_TOOLS_MENU) );
	
	if(!m_hMenu)
		return false;

	m_pLocalizedStrings->ProcessMenu(m_hMenu);

	if(!InsertPlugInMenuToRhinoMenu( m_hMenu, "Bongo Bonus Tools"))
		return false;


	return true;

/*
	AFX_MANAGE_STATE( ::RhinoApp().RhinoModuleState() );
	CMenu* pMenuBar = AfxGetMainWnd()->GetMenu();
	if(!pMenuBar)	return false;

	CString sMenu;
	CMenu* pSubMenu;

	for(int i=0; i<(int)pMenuBar->GetMenuItemCount(); i++)
	{
		pMenuBar->GetMenuString(i,sMenu,MF_BYPOSITION);
	
		if(sMenu.CompareNoCase("Bongo")==0)
		{
			pSubMenu = pMenuBar->GetSubMenu(i);
			if(!pSubMenu)	return false;

			pSubMenu->AppendMenu(MF_SEPARATOR);//(UINT)(m_pMenu->GetSafeHmenu()));
		
			AFX_MANAGE_STATE( AfxGetStaticModuleState() );
			m_hMenu = ::LoadMenu( ::AfxFindResourceHandle( MAKEINTRESOURCE(IDR_BONUS_TOOLS_MENU), RT_MENU),
					MAKEINTRESOURCE(IDR_BONUS_TOOLS_MENU) );
			if(!m_hMenu)
				return false;
			
			pSubMenu->AppendMenu(MF_STRING|MF_POPUP|MF_BYPOSITION,(UINT)(m_hMenu),
					"Bonus Tools");
		
			return true;
		}
	}

	return false;
*/
}

BOOL CBBTPlugIn::OnPlugInMenuCommand(WPARAM wParam)
{
	switch(wParam)
	{
	case ID_GRAPHICAL_DISPLAY:
		{
			RhinoApp().RunMenuScript(L"!_BongoBonusGraphicalDisplay");
			break;
		}
	case ID_OBJECT_ADDTOTREE:
		{
			RhinoApp().RunMenuScript(L"!_BongoBonusAnimateObject");
			break;
		}
	case ID_OBJECT_DELETEKEYFRAMERANGE:
		{
			RhinoApp().RunMenuScript(L"!_BongoBonusDeleteObjectKeyframeRange");
			break;
		}
	case ID_OBJECT_SCALEKEYFRAMERANGE:
		{
			RhinoApp().RunMenuScript(L"!_BongoBonusScaleObjectKeyframes");
			break;
		}
	case ID_OBJECT_MOVEKEYFRAMERANGE:
		{
			RhinoApp().RunMenuScript(L"!_BongoBonusMoveObjectKeyframeRange");
			break;
		}
	case ID_VIEW_DELETEKEYFRAMERANGE:
		{
			RhinoApp().RunMenuScript(L"!_BongoBonusDeleteViewKeyframeRange");
			break;
		}
	case ID_VIEW_SCALEKEYFRAMERANGE:
		{
			RhinoApp().RunMenuScript(L"!_BongoBonusScaleViewKeyframes");
			break;
		}
	case ID_VIEW_MOVEKEYFRAMERANGE:
		{
			RhinoApp().RunMenuScript(L"!_BongoBonusMoveViewKeyframeRange");
			break;
		}
	case ID_TIMELINE_INSERTTICKS:
		{
			RhinoApp().RunMenuScript(L"!_BongoBonusInsertTimelineTicks");
			break;
		}
	case ID_TIMELINE_DELETETICKS:
		{
			RhinoApp().RunMenuScript(L"!_BongoBonusDeleteTimelineTicks");
			break;
		}
	case ID_EMAIL_NOTIFICATION:
		{
			RhinoApp().RunMenuScript(L"!_BongoBonusEmailNotification");
			break;
		}
	case ID_RENDER_FILE_COLLECTION:
		{
			RhinoApp().RunMenuScript(L"!_BongoBonusRenderAnimationFileCollection");
			break;
		}
	case ID_CAMASCVSFILE:
		{
			RhinoApp().RunMenuScript(L"!_BongoBonusSaveCameraCSVFile");
			break;
		}
	}

	return TRUE;
}