// BBTSaveCameraCSVFileCmd.cpp: implementation of the CBBTSaveCameraCSVFileCmd class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BBTApp.h"
#include "BBTSaveCameraCSVFileCmd.h"
#include "bbtmain.h"
#include "LBPShellFunctions.h"
#include "LBPFileMgr.h"

extern CBBTMain bbt;

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


static class CBBTSaveCameraCSVFileCmd theBBTSaveCameraCSVFileCmd;


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBBTSaveCameraCSVFileCmd::CBBTSaveCameraCSVFileCmd()
{
	wcsncpy( m_wszLocalCommandName, ThePlugIn().LocalizedString(14), BBT_STRING_BUFFER);
}

CBBTSaveCameraCSVFileCmd::~CBBTSaveCameraCSVFileCmd()
{

}


UUID CBBTSaveCameraCSVFileCmd::CommandUUID()
{
	// {543D9F16-EA69-4161-A57B-7D262BC9FDD2}
	static const GUID uuidCommand = 
	{ 0x543d9f16, 0xea69, 0x4161, { 0xa5, 0x7b, 0x7d, 0x26, 0x2b, 0xc9, 0xfd, 0xd2 } };

	return uuidCommand;
}
	
void CBBTSaveCameraCSVFileCmd::DoHelp()
{
	
}

const wchar_t* CBBTSaveCameraCSVFileCmd::EnglishCommandName()
{
	return L"BongoBonusSaveCameraCSVFile"; 
}

const wchar_t* CBBTSaveCameraCSVFileCmd::LocalCommandName() const 
{ 
	return m_wszLocalCommandName;
}

CRhinoCommand::result CBBTSaveCameraCSVFileCmd::RunBongoCommand( const CRhinoCommandContext& context )
{
	bbt.WakeUp();

	// Get Keyframes and write data to string
	CString sData;
	CBongoKeyframePtr BongoPtrKf;
	CBongoViewKeyframe* pBongoKf;

	CBongoView* pBongoView = 0;
	if(!CBongoView::ViewObject(pBongoView) || !pBongoView)
	{
		delete pBongoView;
		pBongoView = 0;
		RhinoApp().Print("%s\n",ThePlugIn().LocalizedString(1017).AsMBCSString());
		return CRhinoCommand::failure;
	}

	CRhinoDoc* pDoc = RhinoApp().ActiveDoc();
	if(!pDoc)	return CRhinoCommand::failure;

	ON_SimpleArray <CRhinoView*> aRhinoView;
	pDoc->GetViewList(aRhinoView);
	
	for(int i=0; i<aRhinoView.Count(); i++)
	{
		pBongoView->SetViewName(aRhinoView[i]->Viewport().Name());
		
		if(pBongoView->IsValid() && pBongoView->IsAnimated() && pBongoView->FirstKeyframe() && 
			pBongoView->NumberOfConstraints() <= 0)
		{
	
			WriteHeading(&aRhinoView[i]->Viewport(), sData);

			for(BongoPtrKf = pBongoView->FirstKeyframe(); BongoPtrKf;
				BongoPtrKf = pBongoView->NextKeyframe())
			{
				if(view_keyframe == BongoPtrKf->Type())
				{
					pBongoKf = (CBongoViewKeyframe*)BongoPtrKf.Pointee();
					if(pBongoKf)
					{
						WriteData(pBongoKf, sData);
					}
				}
			}	

			WriteEnd(&aRhinoView[i]->Viewport(), sData);
		}
	}

	delete pBongoView;
	pBongoView = 0;


	if(sData.IsEmpty())
	{
		RhinoApp().Print("%s\n",ThePlugIn().LocalizedString(1018).AsMBCSString());
		return CRhinoCommand::failure;
	}


	CString sFilePath;


	if(context.IsInteractive())
	{
		if(!BrowseForFile(sFilePath))
		{
			return CRhinoCommand::cancel;
		}
	}
	else
	{
		CRhinoGetString GS;
		GS.SetCommandPrompt(ThePlugIn().LocalizedString(2950));
		int iBrowse = GS.AddCommandOption(RHCMDOPTNAME(L"Browse"),RHCMDOPTVALUE(L""));

		GS.GetString();

		CRhinoGet::result Res = GS.Result();
		switch (GS.Result())
		{
		case CRhinoGet::cancel:
			return CRhinoCommand::cancel;
			
		case CRhinoGet::string:
			sFilePath = GS.String();
			break;

		case CRhinoGet::option:
			if(!BrowseForFile(sFilePath))
			{
				return CRhinoCommand::cancel;
			}
			break;
		}	
	}


	if(!IsValidFilePath(sFilePath))
	{
		return CRhinoCommand::failure;
	}

	if(CLBPFileMgr::IsExistingFile(sFilePath))
	{
		DeleteFile(sFilePath);
	}

	// Create CSV file
	CFile File;
	BOOL bRet = File.Open(sFilePath, CFile::modeCreate | CFile::modeWrite | CFile::shareExclusive );
	if(!bRet)
	{
		CString sMsg;
		sMsg.Format(ThePlugIn().LocalizedString(1019), sFilePath);
		RhinoApp().Print("%s\n",sMsg);
		return CRhinoCommand::failure;	
	}

	FitDelimterToLocaleId(sData);

	File.Write(sData, sData.GetLength());

	File.Close();

	return CRhinoCommand::success;
}

bool CBBTSaveCameraCSVFileCmd::IsValidFilePath(const char* szFile)
{
	CString sFullFilePath(szFile);
	
	CString sPathOnly = CLBPFileMgr::GetPathOnly(szFile);
	CString sExtension = CLBPFileMgr::GetExtension(szFile);
	CString	sFileName = CLBPFileMgr::GetFilenameOnly(szFile);

	if(!CLBPFileMgr::IsExistingFolder(sPathOnly))
	{
		RhinoApp().Print("%s\n",ThePlugIn().LocalizedString(1020).AsMBCSString());
		return false;
	}
	
	if(sFileName.IsEmpty())
	{
		RhinoApp().Print("%s\n",ThePlugIn().LocalizedString(1021).AsMBCSString());
		return false;
	}

	return true;
}

bool CBBTSaveCameraCSVFileCmd::BrowseForFile(CString& sFile)
{
	CFileDialog dlg(FALSE, "CSV", "", NULL, "CSV Files (*.csv)|*.csv||");
	if(dlg.DoModal() != IDOK)
	{
		return false;
	}	

	sFile = dlg.GetPathName();

	return true;
}

double CBBTSaveCameraCSVFileCmd::LensLengthToFOV(double dLensLength)
{
	double r;
	
	if(dLensLength == 0.0)	{r = 0.0;}
	else					{r = 2 * atan ( 18.0 / dLensLength);}
	
	return r;
}


bool CBBTSaveCameraCSVFileCmd::WriteHeading(CRhinoViewport* pView, CString& sFile)
{
	if(!pView)
	{
		return false;
	}

	sFile = pView->Name() + "\r\n";
	sFile += ThePlugIn().LocalizedString(2951).AsMBCSString();

	FitDelimterToLocaleId(sFile);

	return true;
}

bool CBBTSaveCameraCSVFileCmd::WriteData(CBongoViewKeyframe* pKF, CString& sFile)
{
	if(!pKF)
	{
		return false;
	}

	double dX(0.0), dY(0.0);

	ON_3dPoint ptTarget(pKF->CameraTarget());
	ON_3dPoint ptLoc(pKF->Location());
	
	ON_3dVector vDirection(ptTarget-ptLoc);
	vDirection.Unitize();
	
	GetCameraYawAndPitchAngles(vDirection, dY, dX);

	CString sData;
	sData.Format("%d;%.3f;%.3f;%.3f;%.3f;%.3f;%.3f;%.3f;%.3f;%.3f;%.3f;\r\n",	(int)pKF->Position(),
																				pKF->Location().x,
																				pKF->Location().y,
																				pKF->Location().z,
																				pKF->CameraTarget().x,
																				pKF->CameraTarget().y,
																				pKF->CameraTarget().z,
																				dY,
																				dX,
																				pKF->RotationAngle(),
																				pKF->LensLength());

	sFile += sData;

	return true;
}

bool CBBTSaveCameraCSVFileCmd::WriteEnd(CRhinoViewport* pView, CString& sFile)
{
	if(!pView)
	{
		return false;
	}

	sFile += "\r\n";

	return true;
}

#include "locale.h"

bool CBBTSaveCameraCSVFileCmd::FitDelimterToLocaleId(CString& sData)
{

	char szBuffer[MAX_PATH];
	if(0 ==GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_SDECIMAL, szBuffer, MAX_PATH))
	{
		return false;
	}

	CString sDecimal(szBuffer);
	if(0 != sDecimal.Compare("."))
	{
		sData.Replace(".", sDecimal);
	}


	if(0 ==GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_SLIST, szBuffer, MAX_PATH))
	{
		return false;
	}

	sDecimal = szBuffer;
	if(0 != sDecimal.Compare(";"))
	{
		sData.Replace(";", sDecimal);
	}


	return true;
}