// BBTScaleObjectKeyframesCmd.cpp: implementation of the CBBTScaleObjectKeyframesCmd class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BBTApp.h"
#include "BBTScaleObjectKeyframesCmd.h"
#include "BBTObjectWrapper.h"
#include "bbtmain.h"

extern CBBTMain bbt;

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


static class CBBTScaleObjectKeyframesCmd theBBTScaleObjectKeyframesCmd;

CBBTScaleObjectKeyframesCmd* GetScaleObjectKeyframesCmd()
{
	return &theBBTScaleObjectKeyframesCmd;
}


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBBTScaleObjectKeyframesCmd::CBBTScaleObjectKeyframesCmd()
{
	wcsncpy( m_wszLocalCommandName, ThePlugIn().LocalizedString(5), BBT_STRING_BUFFER);
}

CBBTScaleObjectKeyframesCmd::~CBBTScaleObjectKeyframesCmd()
{
	FlushObjects();
}

UUID CBBTScaleObjectKeyframesCmd::CommandUUID()
{
	// {D70A686F-12C5-4afb-9BFD-A94C8DA4CF73}
	static const GUID uuidScaleObjectKeyframes = 
	{ 0xd70a686f, 0x12c5, 0x4afb, { 0x9b, 0xfd, 0xa9, 0x4c, 0x8d, 0xa4, 0xcf, 0x73 } };

	return uuidScaleObjectKeyframes;
}
	
void CBBTScaleObjectKeyframesCmd::DoHelp()
{
	
}

const wchar_t* CBBTScaleObjectKeyframesCmd::EnglishCommandName()
{
	return L"BongoBonusScaleObjectKeyframes"; 
}

const wchar_t* CBBTScaleObjectKeyframesCmd::LocalCommandName() const 
{ 
	return m_wszLocalCommandName;
}

CRhinoCommand::result CBBTScaleObjectKeyframesCmd::RunBongoCommand( const CRhinoCommandContext& context )
{
	bbt.WakeUp();


	// empty object list
	FlushObjects();

	CBongo bo;
	WCHAR wsz_Prompt[BBT_STRING_BUFFER];
	wcsncpy(wsz_Prompt, bo.String(8001),BBT_STRING_BUFFER);

	// Ask user to select objects
	CRhinoGetObject go;
	go.AcceptNothing();
	go.EnablePreSelect(true);
	go.SetCommandPrompt(wsz_Prompt);
	go.GetObjects(1,0);
	if( go.Result() != CRhinoGet::object )
			return CRhinoCommand::cancel;

	// Create bongo object data
	if(CreateObjectData(&go) <= 0)
	{
		BBTMsgBox(ThePlugIn().LocalizedString(2032));
		return CRhinoCommand::failure;
	}
	
	int iOrigin;
	double dScalingFactor;
	SCALING_DIRECTION sd;

	// Ask user for scaling options
	if(!CommandLineInput(iOrigin, dScalingFactor, sd))
	{
		FlushObjects();
		return CRhinoCommand::cancel;
	}	

	// scale keyframes
	if(!ScaleKeyframes(iOrigin, dScalingFactor, sd))
	{
		FlushObjects();
		return CRhinoCommand::cancel;
	}

	// change keyframe positions
	SetKeyframes();

	// clean up
	FlushObjects();

	// refesh scene
	CBongoDoc* pBongoDoc = 0;
	if(CBongoDoc::DocumentObject(pBongoDoc) && pBongoDoc)
	{
		pBongoDoc->RefreshTimeline();
		pBongoDoc->RefreshScene();
	}
	
	delete pBongoDoc;

	return CRhinoCommand::success;
}

int CBBTScaleObjectKeyframesCmd::CreateObjectData(CRhinoGetObject* pObjectSelection)
{
	if(!pObjectSelection)
		return 0;

	CBongoKeyframePtr BongoPtrKf;
	CBongoObjectKeyframe* pBongoKf;

	for(int i=0; i<pObjectSelection->ObjectCount(); i++)
	{
		CBongoObject* pBongoObject = new CBongoObject(pObjectSelection->Object(i).Object());
		
		if(!pBongoObject || !pBongoObject->IsValid() || !pBongoObject->IsAnimated()
			|| !pBongoObject->FirstKeyframe())
		{
			delete pBongoObject;
			pBongoObject = 0;
		}
		else
		{
			OBJECT_DATA* pData = new OBJECT_DATA;
			if(pData)
			{
				pData->pBongoObject = pBongoObject;

				for(BongoPtrKf = pBongoObject->FirstKeyframe(); BongoPtrKf;
					BongoPtrKf = pBongoObject->NextKeyframe())
				{
					pBongoKf = (CBongoObjectKeyframe*)BongoPtrKf.Pointee();
					if(pBongoKf)
					{
						pData->aKFs.Add(new CBongoObjectKeyframe(*pBongoKf));
					}
				}	

				m_aObjects.Add(pData);
			}
		}
	}

	return m_aObjects.GetSize();
}

void CBBTScaleObjectKeyframesCmd::FlushObjects()
{
	for(int i=m_aObjects.GetSize()-1; i>=0; i--)
	{
		delete m_aObjects[i]->pBongoObject;
		m_aObjects[i]->pBongoObject = 0;

		for(int j=m_aObjects[i]->aKFs.GetSize()-1; j>=0; j--)
		{
			delete m_aObjects[i]->aKFs[j];
			m_aObjects[i]->aKFs[j] = 0;
		}
		
		m_aObjects[i]->aKFs.RemoveAll();
	}

	m_aObjects.RemoveAll();
}

bool CBBTScaleObjectKeyframesCmd::CommandLineInput(int& iOrigin, double& dScalingFactor, SCALING_DIRECTION& sd)
{
	if(!GetCommandLineInteger(2030, iOrigin, 0, BONGO_MAX_TIMELINE_RANGE-1,0, true, true))
		return false;


	WCHAR wsz_Prompt[BBT_STRING_BUFFER];
	wcsncpy(wsz_Prompt, ThePlugIn().LocalizedString(2031),BBT_STRING_BUFFER);

	CRhinoGetNumber gn;
	gn.SetCommandPrompt(wsz_Prompt);
	gn.SetDefaultInteger(1);
	gn.AcceptNothing();
	gn.GetNumber();

	switch (gn.Result())
	{
	case CRhinoGet::cancel:
		return false;

	case CRhinoGet::nothing:
		dScalingFactor = gn.Number();
		break;
		
	case CRhinoGet::number:
		dScalingFactor = gn.Number();
		break;
	}

	if(!GetScalingDirection(sd))
		return false;

	return true;
}

bool CBBTScaleObjectKeyframesCmd::GetScalingDirection(SCALING_DIRECTION& sd)
{
	WCHAR wsz_Prompt[BBT_STRING_BUFFER];
	wcsncpy(wsz_Prompt, ThePlugIn().LocalizedString(2034),BBT_STRING_BUFFER);

	for(;;)
	{
		CRhinoGetOption go;
		go.AcceptNothing();
		go.SetCommandPrompt(wsz_Prompt);
		int iOption_index = go.AddCommandOptionList(
			CBBTCommandOptionName(2035),
			2,
			CBBTCommandOptionValue(2036),
			CBBTCommandOptionValue(2037),
			CBBTCommandOptionValue(2038)
			);

		go.GetOption();
		switch (go.Result())
		{
		case CRhinoGet::cancel:
			return false;
		
		case CRhinoGet::nothing:
			sd = scale_left_and_right;
			return true;

		case CRhinoGet::option:
			const CRhinoCommandOption* opt = go.Option();
			if ( opt->m_option_index == iOption_index)
			{	
				int iIndex = opt->m_list_option_current;


				if(0 == iIndex)			{sd = scale_left;}
				else if(1 == iIndex)	{sd = scale_right;}
				else					{sd = scale_left_and_right;}
					
				return true;
			}
		}
	}

	return false;
}

bool CBBTScaleObjectKeyframesCmd::ScaleKeyframes(int iOrigin, double dScalingFactor, SCALING_DIRECTION sd)
{
	double dPos;
	bool bPopupWarning = true;

	for(int i=0; i<m_aObjects.GetSize();i++)
	{
		for(int j=m_aObjects[i]->aKFs.GetSize()-1; j>=0; j--)
		{
			dPos = 	m_aObjects[i]->aKFs[j]->Position();
		
			if(dPos > 0.0)
			{
				if(dPos > (double)iOrigin && 
					(sd == scale_right || sd == scale_left_and_right))
				{
					dPos = (dScalingFactor * (dPos - (double)iOrigin)) + (double)iOrigin;
				}
				else if(dPos < (double)iOrigin && 
					(sd == scale_left || sd == scale_left_and_right))
				{
					dPos = (double)iOrigin - (dScalingFactor * ((double)iOrigin - dPos));
				}

				m_aObjects[i]->aKFs[j]->SetPosition(dPos);

				if(dPos <= 0.0 || dPos > BONGO_MAX_TIMELINE_RANGE-1)
				{
					bPopupWarning = false;

					delete m_aObjects[i]->aKFs[j];
					m_aObjects[i]->aKFs[j] = 0;
					m_aObjects[i]->aKFs.RemoveAt(j);

					if(bPopupWarning)
					{
						bPopupWarning = false;
						
						if(IDNO == BBTMsgBox(ThePlugIn().LocalizedString(2033),
							MB_YESNO|MB_ICONQUESTION|MB_DEFBUTTON1))
						{
							return false;
						}
					}
				}
			}		
		}
	}

	return true;
}

bool CBBTScaleObjectKeyframesCmd::SetKeyframes()
{
	CBBTObjectWrapper ow;

	for(int i=0; i<m_aObjects.GetSize();i++)
	{
		ow.DeleteAllKeyframes(m_aObjects[i]->pBongoObject);

		for(int j=0; j<m_aObjects[i]->aKFs.GetSize(); j++)
		{
			m_aObjects[i]->pBongoObject->SetKeyframe(m_aObjects[i]->aKFs[j], false);
		}
	}

	return true;
}
