// BBTScaleViewKeyframesCmd.cpp: implementation of the CBBTScaleViewKeyframesCmd class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BBTApp.h"
#include "BBTScaleViewKeyframesCmd.h"
#include "BBTViewWrapper.h"
#include "bbtmain.h"

extern CBBTMain bbt;

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

static class CBBTScaleViewKeyframesCmd theBBTScaleViewKeyframesCmd;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBBTScaleViewKeyframesCmd::CBBTScaleViewKeyframesCmd()
{
	wcsncpy( m_wszLocalCommandName, ThePlugIn().LocalizedString(6), BBT_STRING_BUFFER);
}

CBBTScaleViewKeyframesCmd::~CBBTScaleViewKeyframesCmd()
{

}

UUID CBBTScaleViewKeyframesCmd::CommandUUID()
{
	// {68BE288F-E927-4f55-BC44-119B72F2FC29}
	static const GUID uuidScaleViewKeyframes = 
	{ 0x68be288f, 0xe927, 0x4f55, { 0xbc, 0x44, 0x11, 0x9b, 0x72, 0xf2, 0xfc, 0x29 } };

	return uuidScaleViewKeyframes;
}
	
void CBBTScaleViewKeyframesCmd::DoHelp()
{
	
}

const wchar_t* CBBTScaleViewKeyframesCmd::EnglishCommandName()
{
	return L"BongoBonusScaleViewKeyframes"; 
}

const wchar_t* CBBTScaleViewKeyframesCmd::LocalCommandName() const 
{ 
	return m_wszLocalCommandName;
}

CRhinoCommand::result CBBTScaleViewKeyframesCmd::RunBongoCommand( const CRhinoCommandContext& context )
{
	bbt.WakeUp();

	CRhinoViewport* pViewport = &(RhinoApp().ActiveView()->Viewport());
	if(!pViewport)	return CRhinoCommand::failure;

	CBongoView* pBongoView = 0;
	if(!CBongoView::ViewObject(pBongoView, pViewport) || !pBongoView)
	{
		delete pBongoView;
		pBongoView = 0;
		return CRhinoCommand::failure;
	}

	if(!ValidateView(pBongoView))
	{
		delete pBongoView;
		pBongoView = 0;
		BBTMsgBox(ThePlugIn().LocalizedString(2050));
		return CRhinoCommand::failure;
	}

	int iOrigin;
	double dScalingFactor;
	SCALING_DIRECTION sd;

	// Ask user for scaling options
	if(!CommandLineInput(iOrigin, dScalingFactor, sd))
	{
		delete pBongoView;
		pBongoView = 0;
		return CRhinoCommand::cancel;
	}	

	CArray<CBongoKeyframe*, CBongoKeyframe*> aKFs;

	// scale keyframes
	if(ScaleKeyframes(pBongoView, iOrigin, dScalingFactor, sd, aKFs))
	{
		// change keyframe positions
		SetKeyframes(pBongoView, aKFs);
	}

	// clean up
	for(int i=aKFs.GetSize()-1; i>=0; i--)
	{
		delete aKFs[i];
		aKFs[i] = 0;
	}

	delete pBongoView;
	pBongoView = 0;

	// refesh scene
	CBongoDoc* pBongoDoc = 0;
	if(CBongoDoc::DocumentObject(pBongoDoc) && pBongoDoc)
	{
		pBongoDoc->RefreshTimeline();
		pBongoDoc->RefreshScene();
	}
	
	delete pBongoDoc;

	return CRhinoCommand::success;
}

bool CBBTScaleViewKeyframesCmd::ValidateView(CBongoView* pView)
{
	if(!pView || !pView->IsValid() || !pView->IsAnimated() || !pView->FirstKeyframe())
		return false;

	return true;
}

bool CBBTScaleViewKeyframesCmd::CommandLineInput(int& iOrigin, double& dScalingFactor, SCALING_DIRECTION& sd)
{
	if(!GetCommandLineInteger(2030, iOrigin, 0, BONGO_MAX_TIMELINE_RANGE-1,0, true, true))
		return false;

	WCHAR wsz_Prompt[BBT_STRING_BUFFER];
	wcsncpy(wsz_Prompt, ThePlugIn().LocalizedString(2031),BBT_STRING_BUFFER);

	CRhinoGetNumber gn;
	gn.SetCommandPrompt(wsz_Prompt);
	gn.SetDefaultInteger(1);
	gn.AcceptNothing();
	gn.GetNumber();

	switch (gn.Result())
	{
	case CRhinoGet::cancel:
		return false;

	case CRhinoGet::nothing:
		dScalingFactor = gn.Number();
		break;
		
	case CRhinoGet::number:
		dScalingFactor = gn.Number();
		break;
	}

	if(!GetScalingDirection(sd))
		return false;

	return true;
}

bool CBBTScaleViewKeyframesCmd::GetScalingDirection(SCALING_DIRECTION& sd)
{
	WCHAR wsz_Prompt[BBT_STRING_BUFFER];
	wcsncpy(wsz_Prompt, ThePlugIn().LocalizedString(2034),BBT_STRING_BUFFER);

	for(;;)
	{
		CRhinoGetOption go;
		go.AcceptNothing();
		go.SetCommandPrompt(wsz_Prompt);
		int iOption_index = go.AddCommandOptionList(
			CBBTCommandOptionName(2035),
			2,
			CBBTCommandOptionValue(2036),
			CBBTCommandOptionValue(2037),
			CBBTCommandOptionValue(2038)
			);

		go.GetOption();
		switch (go.Result())
		{
		case CRhinoGet::cancel:
			return false;
		
		case CRhinoGet::nothing:
			sd = scale_left_and_right;
			return true;

		case CRhinoGet::option:
			const CRhinoCommandOption* opt = go.Option();
			if ( opt->m_option_index == iOption_index)
			{	
				int iIndex = opt->m_list_option_current;


				if(0 == iIndex)			{sd = scale_left;}
				else if(1 == iIndex)	{sd = scale_right;}
				else					{sd = scale_left_and_right;}
					
				return true;
			}
		}
	}

	return false;
}

bool CBBTScaleViewKeyframesCmd::ScaleKeyframes(CBongoView* pBongoView,int iOrigin, double dScalingFactor, SCALING_DIRECTION sd,
		CArray<CBongoKeyframe*, CBongoKeyframe*>& aKFs)
{
	if(!pBongoView)
		return false;

	double dPos;
	bool bPopupWarning = true;
	CBongoKeyframePtr BongoPtrKf;	
	CBongoViewKeyframe* pBongoKf, *pDupBongoKf;
	
	for(BongoPtrKf = pBongoView->FirstKeyframe(); BongoPtrKf;
		BongoPtrKf = pBongoView->NextKeyframe())
	{
		pBongoKf = (CBongoViewKeyframe*)BongoPtrKf.Pointee();
		if(pBongoKf)
		{
			dPos = 	pBongoKf->Position();
		
			if(dPos > 0.0)
			{
				if(dPos > (double)iOrigin && 
					(sd == scale_right || sd == scale_left_and_right))
				{
					dPos = (dScalingFactor* (dPos - (double)iOrigin)) + (double)iOrigin;
				}
				else if(dPos < (double)iOrigin && 
					(sd == scale_left || sd == scale_left_and_right))
				{
					dPos = (double)iOrigin - (dScalingFactor* ((double)iOrigin - dPos));
				}

				if(dPos <= 0.0 || dPos > BONGO_MAX_TIMELINE_RANGE-1)
				{
					bPopupWarning = false;

					if(bPopupWarning)
					{
						bPopupWarning = false;
						
						if(IDNO == BBTMsgBox(ThePlugIn().LocalizedString(2033),
							MB_YESNO|MB_ICONQUESTION|MB_DEFBUTTON1))
						{
							return false;
						}
					}
				}
				else
				{
					pDupBongoKf = new CBongoViewKeyframe(*pBongoKf);
					pDupBongoKf->SetPosition(dPos);
					aKFs.Add(pDupBongoKf);
				}
			}			
		}
	}

	return true;
}

bool CBBTScaleViewKeyframesCmd::SetKeyframes(CBongoView* pView, CArray<CBongoKeyframe*, CBongoKeyframe*>& aKFs)
{
	if(!pView)
		return false;

	CBBTViewWrapper ow;

	ow.DeleteAllKeyframes(pView);

	for(int i=0; i<aKFs.GetSize(); i++)
	{
		pView->SetKeyframe(aKFs[i], false);
	}

	return true;
}
