// TreeListCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "TreeListDC.h"
#include "TreeListColumnInfo.h"
#include "TreeListHeaderCtrl.h"
#include "TreeListTipCtrl.h"
#include "TreeListStaticCtrl.h"
#include "TreeListEditCtrl.h"
#include "TreeListComboCtrl.h"
#include "TLCDragWnd.h"
#include "TLCDropWnd.h"
#include "TreeListCtrl.h"
#include "TreeListResource.h"
#include "TreeListItem.h"
#include "TreeListDC.h"
#include "TLHDragWnd.h"
#include "TLHDropWnd.h"
#include "mfxWhlPan.h"

#include "resource.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


static lpfnUpdateLayeredWindow g_lpfnUpdateLayeredWindow = NULL;
static lpfnSetLayeredWindowAttributes g_lpfnSetLayeredWindowAttributes = NULL;


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTreeListColumnInfo::CTreeListColumnInfo() :
	m_dwFormat( TLF_DEFAULT_LEFT ),
	m_dwModify( TLM_STATIC ),
	m_nMin( 0 ),				// С
	m_nMax( 0 ),				// 
	m_nWidth( 30 ),				// ǰ
	m_strCaption( _T("") ),		// ʾ
	m_iImage( -1 ),				// ͼ
	m_nOldWidth( 0 ),
	m_nTrackPos( 0 )
// 	m_fnCompare( NULL )
{
	IMPLEMENT_LOGGING("	m_fnCompare", __DATE__,__FILE__, __LINE__);

}

CTreeListColumnInfo::~CTreeListColumnInfo()
{
	IMPLEMENT_LOGGING("CTreeListColumnInfo::~CTreeListColumnInfo", __DATE__,__FILE__, __LINE__);

}

void CTreeListColumnInfo::BeginTrack( int nTrackPos )
{
	IMPLEMENT_LOGGING("CTreeListColumnInfo::BeginTrack", __DATE__,__FILE__, __LINE__);
	ASSERT( m_nOldWidth == 0 );
	ASSERT( m_nTrackPos == 0 );

	m_nOldWidth = m_nWidth;
	m_nTrackPos = nTrackPos;
}

void CTreeListColumnInfo::EndTrack( int nTrackPos )
{
	IMPLEMENT_LOGGING("CTreeListColumnInfo::EndTrack", __DATE__,__FILE__, __LINE__);
	int nWidth = m_nOldWidth + ( nTrackPos - m_nTrackPos );

	if( nWidth < m_nMin )
		nWidth = m_nMin;

	if( m_nMax != 0 && nWidth > m_nMax )
		nWidth = m_nMax;

	m_nWidth	= nWidth;
	m_nOldWidth = 0;
	m_nTrackPos = 0;
}

void CTreeListColumnInfo::DoTrack( int nTrackPos )
{
	IMPLEMENT_LOGGING("CTreeListColumnInfo::DoTrack", __DATE__,__FILE__, __LINE__);
	int nWidth = m_nOldWidth + ( nTrackPos - m_nTrackPos );

	if( nWidth < m_nMin )
		nWidth = m_nMin;

	if( m_nMax != 0 && nWidth > m_nMax )
		nWidth = m_nMax;

	m_nWidth = nWidth;
}

UINT CTreeListColumnInfo::GetTextFormat()
{
	IMPLEMENT_LOGGING("CTreeListColumnInfo::GetTextFormat", __DATE__,__FILE__, __LINE__);
	UINT dwFormat = DT_SINGLELINE;

	if( m_dwFormat&TLF_TEXTALIGN_LEFT )
	{
		dwFormat |= DT_LEFT;
		dwFormat |= DT_WORD_ELLIPSIS;
	}
	else if( m_dwFormat&TLF_TEXTALIGN_RIGHT )
	{
		dwFormat |= DT_RIGHT;
	}
	else if( m_dwFormat&TLF_TEXTALIGN_CENTER )
	{
		dwFormat |= DT_CENTER;
	}
	else
	{
		dwFormat |= DT_LEFT;
		dwFormat |= DT_WORD_ELLIPSIS;
	}

	if( m_dwFormat&TLF_TEXTVALIGN_TOP )
		dwFormat |= DT_TOP;
	else if( m_dwFormat&TLF_TEXTVALIGN_BOTTOM )
		dwFormat |= DT_BOTTOM;
	else if( m_dwFormat&TLF_TEXTVALIGN_CENTER )
		dwFormat |= DT_VCENTER;
	else
		dwFormat |= DT_VCENTER;

	return dwFormat;
}

UINT CTreeListColumnInfo::GetCaptionFormat()
{
	IMPLEMENT_LOGGING("CTreeListColumnInfo::GetCaptionFormat", __DATE__,__FILE__, __LINE__);
	UINT dwFormat = DT_SINGLELINE;

	if( m_dwFormat&TLF_ALIGN_LEFT )
	{
		dwFormat |= DT_LEFT;
		dwFormat |= DT_WORD_ELLIPSIS;
	}
	else if( m_dwFormat&TLF_ALIGN_RIGHT )
	{
		dwFormat |= DT_RIGHT;
	}
	else if( m_dwFormat&TLF_TEXTALIGN_CENTER )
	{
		dwFormat |= DT_CENTER;
	}
	else
	{
		dwFormat |= DT_CENTER;
	}

	if( m_dwFormat&TLF_VALIGN_TOP )
		dwFormat |= DT_TOP;
	else if( m_dwFormat&TLF_VALIGN_BOTTOM )
		dwFormat |= DT_BOTTOM;
	else if( m_dwFormat&TLF_VALIGN_CENTER )
		dwFormat |= DT_VCENTER;
	else
		dwFormat |= DT_VCENTER;

	return dwFormat;
}

DWORD CTreeListColumnInfo::SetFormat( DWORD dwAdd, DWORD dwRemove )
{
	IMPLEMENT_LOGGING("CTreeListColumnInfo::SetFormat", __DATE__,__FILE__, __LINE__);
	DWORD dwOld = m_dwFormat;
	m_dwFormat |= dwAdd;
	m_dwFormat &= ~dwRemove;
	return dwOld;
}

DWORD CTreeListColumnInfo::GetFormat( DWORD dwMask )
{
	IMPLEMENT_LOGGING("CTreeListColumnInfo::GetFormat", __DATE__,__FILE__, __LINE__);
	return m_dwFormat&dwMask;
}

DWORD CTreeListColumnInfo::SetModify( DWORD dwModify )
{
	IMPLEMENT_LOGGING("CTreeListColumnInfo::SetModify", __DATE__,__FILE__, __LINE__);
	DWORD dwOld = m_dwModify;
	m_dwModify = dwModify;
	return dwOld;
}

DWORD CTreeListColumnInfo::GetModify()
{
	IMPLEMENT_LOGGING("CTreeListColumnInfo::GetModify", __DATE__,__FILE__, __LINE__);
	return m_dwModify;
}

int CTreeListColumnInfo::SetWidth( int nWidth, int nMin, int nMax )
{
	IMPLEMENT_LOGGING("CTreeListColumnInfo::SetWidth", __DATE__,__FILE__, __LINE__);
	if( 0 > nMin )
		nMin = 0;

	if( 0 > nMax )
		nMax = 0;

	if( nMin > nWidth )
		nWidth = nMin;

	if( nMax != 0 )
	{
		if( nMin > nMax )
			nMax = nMin;

		if( nWidth > nMax )
			nWidth = nMax;
	}

	m_nMin = nMin;
	m_nMax = nMax;
	m_nWidth = nWidth;

	return 0;
}

int CTreeListColumnInfo::GetWidth()
{
	IMPLEMENT_LOGGING("CTreeListColumnInfo::GetWidth", __DATE__,__FILE__, __LINE__);
	return m_nWidth;
}

int CTreeListColumnInfo::SetImage( int iImage )
{
	IMPLEMENT_LOGGING("CTreeListColumnInfo::SetImage", __DATE__,__FILE__, __LINE__);
	int iOld = m_iImage;
	m_iImage = iImage;
	return iOld;
}

int CTreeListColumnInfo::GetImage()
{
	IMPLEMENT_LOGGING("CTreeListColumnInfo::GetImage", __DATE__,__FILE__, __LINE__);
	return m_iImage;
}

BOOL CTreeListColumnInfo::SetText( LPCTSTR lpszText )
{
	IMPLEMENT_LOGGING("CTreeListColumnInfo::SetText", __DATE__,__FILE__, __LINE__);
	m_strCaption = lpszText;
	return TRUE;
}

LPCTSTR CTreeListColumnInfo::GetText( )
{
	IMPLEMENT_LOGGING("CTreeListColumnInfo::GetText", __DATE__,__FILE__, __LINE__);
	return m_strCaption;
}

BOOL CTreeListColumnInfo::SetDefaultText( LPCTSTR lpszText )
{
	IMPLEMENT_LOGGING("CTreeListColumnInfo::SetDefaultText", __DATE__,__FILE__, __LINE__);
	m_strDefault = lpszText;
	return TRUE;
}

LPCTSTR CTreeListColumnInfo::GetDefaultText()
{
	IMPLEMENT_LOGGING("CTreeListColumnInfo::GetDefaultText", __DATE__,__FILE__, __LINE__);
	return m_strDefault;
}

int CTreeListColumnInfo::GetMinWidth()
{
	IMPLEMENT_LOGGING("CTreeListColumnInfo::GetMinWidth", __DATE__,__FILE__, __LINE__);
	return m_nMin;
}

int CTreeListColumnInfo::GetMaxWidth()
{
	IMPLEMENT_LOGGING("CTreeListColumnInfo::GetMaxWidth", __DATE__,__FILE__, __LINE__);
	return m_nMax;
}


/////////////////////////////////////////////////////////////////////////////
// CTreeListComboCtrl

CTreeListComboCtrl::CTreeListComboCtrl() :
	m_pTreeListCtrl( NULL )
{
	IMPLEMENT_LOGGING("m_pTreeListCtrl", __DATE__,__FILE__, __LINE__);
}

CTreeListComboCtrl::~CTreeListComboCtrl()
{
	IMPLEMENT_LOGGING("CTreeListComboCtrl::~CTreeListComboCtrl", __DATE__,__FILE__, __LINE__);
}


BEGIN_MESSAGE_MAP(CTreeListComboCtrl, CComboBox)
	//{{AFX_MSG_MAP(CTreeListComboCtrl)
	ON_WM_KILLFOCUS()
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTreeListComboCtrl message handlers

int CTreeListComboCtrl::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	IMPLEMENT_LOGGING("CTreeListComboCtrl::OnCreate", __DATE__,__FILE__, __LINE__);
	if (CComboBox::OnCreate(lpCreateStruct) == -1)
		return -1;

	m_pTreeListCtrl = (CTreeListCtrl*)GetParent();

	return 0;
}

void CTreeListComboCtrl::OnKillFocus(CWnd* pNewWnd) 
{
	IMPLEMENT_LOGGING("CTreeListComboCtrl::OnKillFocus", __DATE__,__FILE__, __LINE__);
	CComboBox::OnKillFocus(pNewWnd);
	
	m_pTreeListCtrl->SetCtrlFocus( pNewWnd, FALSE );
}









/////////////////////////////////////////////////////////////////////////////
// CTreeListCtrl
IMPLEMENT_DYNCREATE(CTreeListCtrl, CWnd)

CTreeListCtrl::CTreeListCtrl() : 
	m_nItemHeight( 0 ),
	m_dwStyle( 0 ),
	m_dwState( TLS_OFFSCREENBUFFER ),
	m_pRootItem( NULL ),
	m_iSelCol( -1 ),
	m_pFocusItem( NULL ),
	m_pTargetItem( NULL ),
	m_pHoverItem( NULL ),
	m_iHoverItem( -1 ),
	m_iModifyCol( -1 ),
	m_pModifyItem( NULL ),
	m_pUpdateItem( NULL ),
	m_hButton( NULL ),
	m_hCheck( NULL ),
	m_hLock( NULL ),
	m_hDrop( NULL ),
	m_hStop( NULL ),
	m_pImageList( NULL ),
	m_pDragWnd( NULL ),
	m_pDropWnd( NULL ),
	m_pTargetWnd( NULL )
{
	IMPLEMENT_LOGGING("m_pTargetWnd", __DATE__,__FILE__, __LINE__);
	// construct

	// create windows class
	WNDCLASS wndclass;
	HINSTANCE hInst = AfxGetInstanceHandle();
	//CTreeListResource	TreeListResource;

	if( !::GetClassInfo(hInst, TREELISTCTRL_CLASSNAME, &wndclass) )
	{
        wndclass.style			= CS_DBLCLKS|CS_HREDRAW|CS_VREDRAW;
		wndclass.lpfnWndProc	= ::DefWindowProc;
		wndclass.cbClsExtra		= 0;
		wndclass.cbWndExtra		= 0;
		wndclass.hInstance		= hInst;
		wndclass.hIcon			= NULL;
		wndclass.hCursor		= AfxGetApp()->LoadStandardCursor( IDC_ARROW);
		wndclass.hbrBackground	= (HBRUSH)COLOR_WINDOW;
		wndclass.lpszMenuName	= NULL;
		wndclass.lpszClassName	= TREELISTCTRL_CLASSNAME;

		if (!AfxRegisterClass(&wndclass))
			AfxThrowResourceException();
	}

	// back ground colors of normal status
	m_crBkColor				= 0xFFFFE0;				
	m_crBkColor_0			= 0xFCFCFC;				
	m_crBkColor_1			= 0xF0F0F0;				
	m_crBkColor_2			= 0xF8F8F8;				

	// back ground colors of disable status
	m_crBkDisable			= 0xC0C080;				
	m_crBkDisable_0			= 0xCCCCCC;				
	m_crBkDisable_1			= 0xC0C0C0;				
	m_crBkDisable_2			= 0xC8C8C8;				

	// back ground colors of selected item
	m_crBkSelected			= 0xC08080;				
	m_crBkSelectedRow		= 0xA00000;				
	m_crBkSelectedColumn	= 0xD00000;				
	m_crBkSelectedNoFocus	= 0xD0A0A0;				

	// text colors
	m_crTextSelected		= 0xE8E8E8;				
	m_crTextSelectedRow		= 0xF0F0F0;				
	m_crTextSelectedColumn	= 0xFFFFFF;				
	m_crTextSelectedNoFocus	= 0xE0E0E0;				

	// normal text color
	m_crText				= 0x000000;				
	m_crTextHover			= 0x00FF00;				

	// grid color
	m_crGrid				= 0xD0D0D0;				
	m_crTreeLine			= 0x808080;				

	m_Palette.CreateStockObject( DEFAULT_PALETTE );
	m_Font.CreateStockObject( SYSTEM_FIXED_FONT );

	m_hButton	= AfxGetApp()->LoadCursor( IDC_CURSOR_BUTTON );
	m_hCheck	= AfxGetApp()->LoadCursor( IDC_CURSOR_CHECK );
	m_hLock		= AfxGetApp()->LoadCursor( IDC_CURSOR_LOCK );

	m_hDrop		= AfxGetApp()->LoadCursor( IDC_CURSOR_DROP );
	m_hStop		= AfxGetApp()->LoadCursor( IDC_CURSOR_STOP );
	m_hArrow	= AfxGetApp()->LoadStandardCursor( IDC_ARROW );

	m_pRootItem = new CTreeListItem( 1 );
	m_pRootItem->Expand();
}

CTreeListCtrl::~CTreeListCtrl()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::~CTreeListCtrl", __DATE__,__FILE__, __LINE__);
	// deconstruct
	m_wndHeader.DestroyWindow();
	m_wndTip.DestroyWindow();

	if( m_Palette.m_hObject != NULL )
		m_Palette.DeleteObject();

	if( m_Font.m_hObject != NULL )
		m_Font.DeleteObject();

	if( m_hButton != NULL )
		DestroyCursor( m_hButton );

	if( m_hCheck != NULL )
		DestroyCursor( m_hCheck );

	if( m_hLock != NULL )
		DestroyCursor( m_hLock );

	if( m_hDrop != NULL )
		DestroyCursor( m_hDrop );

	if( m_hStop != NULL )
		DestroyCursor( m_hStop );

	if( m_hArrow != NULL )
		DestroyCursor( m_hArrow );

	while( m_arColumns.GetSize() > 0 )
	{
		CTreeListColumnInfo* pColumnInfo;
		int nIndex;
		
		nIndex = m_arColumns.GetUpperBound();
		pColumnInfo = (CTreeListColumnInfo*) m_arColumns[nIndex];

		m_arColumns.RemoveAt( nIndex );
		delete pColumnInfo;
	}

	m_bmpBkBitmap.DeleteObject();
	m_bmpBkDisable.DeleteObject();

	m_imgCheck.DeleteImageList();
	m_imgLock.DeleteImageList();
	m_imgTree.DeleteImageList();

	delete m_pRootItem;
}

BEGIN_MESSAGE_MAP(CTreeListCtrl, CWnd)
	//{{AFX_MSG_MAP(CTreeListCtrl)
	ON_WM_ERASEBKGND()
	ON_WM_PAINT()
	ON_WM_ENABLE()
	ON_WM_NCCALCSIZE()
	ON_WM_NCPAINT()
	ON_WM_SIZE()
	ON_WM_HSCROLL()
	ON_WM_VSCROLL()
	ON_WM_MOUSEWHEEL()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
	ON_WM_MBUTTONDOWN()
	ON_WM_SETFOCUS()
	ON_WM_KILLFOCUS()
	ON_WM_LBUTTONUP()
	ON_WM_RBUTTONUP()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_DESTROY()
	ON_WM_TIMER()
	ON_WM_CANCELMODE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTreeListCtrl message handler
BOOL CTreeListCtrl::Create( DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::Create", __DATE__,__FILE__, __LINE__);
	// create control
	SetState( TLS_SUBCLASSFROMCREATE );
	DWORD dwExStyle = 0;

	if( dwStyle&WS_BORDER )
	{
		dwStyle &= ~WS_BORDER;
		dwExStyle |= WS_EX_CLIENTEDGE;
	}

	if( !CWnd::CreateEx( dwExStyle, TREELISTCTRL_CLASSNAME, NULL, dwStyle, rect, pParentWnd, nID ) )
		return FALSE;

	if( !Create() )
		return FALSE;

	

	return TRUE;
}

BOOL CTreeListCtrl::Create()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::Create", __DATE__,__FILE__, __LINE__);
	// create control
	CRect Rect( 0, 0, 0, 0);

	if( !m_wndHeader.Create( 0, Rect, this, 0 ) )
		return FALSE;

	if( !m_wndTip.Create( this ) )
		return FALSE;

	DWORD dwStyle;

	dwStyle = WS_BORDER | WS_CHILD | SS_NOTIFY | SS_CENTERIMAGE;
	if( !m_wndStatic.Create( _T("123"), dwStyle, Rect, this, 0 ) )
		return FALSE;

	dwStyle = WS_BORDER | WS_CHILD | ES_CENTER | ES_AUTOHSCROLL;

	if( !m_wndEdit.Create( dwStyle, Rect, this, 0 ) )
		return FALSE;

	dwStyle = WS_CHILD | CBS_DROPDOWNLIST | CBS_AUTOHSCROLL | CBS_NOINTEGRALHEIGHT;

	if( !m_wndCombo.Create( dwStyle, Rect, this, 0 ) )
		return FALSE;

	if( !m_bmpBkBitmap.LoadBitmap( IDB_TREELIST_BITMAP ) )
		return FALSE;

	if( !m_bmpBkDisable.LoadBitmap( IDB_TREELIST_DISABLE ) )
		return FALSE;

	if( !m_imgCheck.Create( IDB_TREELIST_CHECK, 16, 3, 0xFF00FF ) )
		return FALSE;

	if( !m_imgLock.Create( IDB_TREELIST_LOCK, 8, 3, 0xFF00FF ) )
		return FALSE;

	if( !m_imgTree.Create( IDB_TREELIST_TREE, 16, 3, 0xFFFFFF ) )
		return FALSE;

	m_pImageList = &m_imgTree;

	SetFont();
	SetItemHeight();
	
	m_wndHeader.SetHeaderFont();
	m_wndHeader.SetHeaderHeight();
	m_wndTip.SetFont();
	
	m_wndStatic.SetFont( GetFont() );
	m_wndEdit.SetFont( GetFont() );
	m_wndCombo.SetFont( GetFont() );

	Layout();

	return TRUE;
}

void CTreeListCtrl::PreSubclassWindow() 
{
	IMPLEMENT_LOGGING("CTreeListCtrl::PreSubclassWindow", __DATE__,__FILE__, __LINE__);
	// pre subclass window
	CWnd::PreSubclassWindow();

	if( !GetState( TLS_SUBCLASSFROMCREATE ) )
	{
		if( !Create() )
		{
			AfxThrowMemoryException();
			return;
		}
	}

	return;
}

void CTreeListCtrl::OnDestroy() 
{
	IMPLEMENT_LOGGING("CTreeListCtrl::OnDestroy", __DATE__,__FILE__, __LINE__);
	CWnd::OnDestroy();
	
	m_arShows.RemoveAll();
	m_arSorts.RemoveAll();

	DeleteItem( TLI_ROOT );
}

void CTreeListCtrl::SetAllScroll()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetAllScroll", __DATE__,__FILE__, __LINE__);
	// set all scroll bars
	CRect rcPreClient;
	GetWindowRect( &rcPreClient );
	rcPreClient.DeflateRect( 1, 1 );

	int cx = ::GetSystemMetrics( SM_CXVSCROLL );
	int cy = ::GetSystemMetrics( SM_CYHSCROLL );

	SCROLLINFO siv;
	SCROLLINFO sih;

	GetScrollInfo( SB_VERT, &siv, SIF_POS | SIF_RANGE );
	GetScrollInfo( SB_HORZ, &sih, SIF_POS | SIF_RANGE );

	BOOL bHorzScroll = FALSE;
	BOOL bVertScroll = FALSE;

	if( rcPreClient.Width() < GetWidth() )
	{
		bHorzScroll = TRUE;

		if( rcPreClient.Height() - cy < m_wndHeader.GetHeaderHeight() + GetVisibleCount() * GetItemHeight() )
			bVertScroll = TRUE;
	}
	else if( rcPreClient.Height() < m_wndHeader.GetHeaderHeight() + GetVisibleCount() * GetItemHeight() )
	{
		bVertScroll = TRUE;

		if( rcPreClient.Width() - cx < GetWidth() )
			bHorzScroll = TRUE;
	}

	if( sih.nPos != sih.nMin )
		bHorzScroll = TRUE;

	if( siv.nPos != siv.nMin )
		bVertScroll = TRUE;

	if( bHorzScroll )
	{
		if( rcPreClient.Width() > cx )
			rcPreClient.right -= cx;
		else
			rcPreClient.right = rcPreClient.left;
	}

	if( bVertScroll )
	{
		if( rcPreClient.Height() > cy )
			rcPreClient.bottom -= cy;
		else
			rcPreClient.bottom = rcPreClient.top;
	}

	SetHorzScroll( &rcPreClient );
	SetVertScroll( &rcPreClient );
}

void CTreeListCtrl::SetHorzScroll( CRect* pPreClient )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetHorzScroll", __DATE__,__FILE__, __LINE__);
	// set horizonal scroll
	SCROLLINFO si;
	GetScrollInfo( SB_HORZ, &si, SIF_ALL );

	if( pPreClient->Width() < GetWidth() )
	{
		si.cbSize	= sizeof( SCROLLINFO );
		si.nMin		= 0;
		si.nMax		= GetWidth() - 1;
		si.nPage	= pPreClient->Width();
		if (si.nPos > si.nMax)	//*KB* This is a hack - still don't know why this happens
		{
			si.nPos = 0;
		} else {
			si.nPos		= si.nPos;
		}

		si.fMask	=  SIF_PAGE | SIF_RANGE | SIF_POS;
		if( si.nPos > si.nMax - (int)si.nPage )
			si.nPos = si.nMax - (int)si.nPage;
	}
	else
	{
		si.cbSize	= sizeof( SCROLLINFO );
		si.nMin		= 0;
		si.nMax		= 0;
		si.nPage	= 0;
		si.nPos		= 0;
		si.fMask	=  SIF_PAGE | SIF_RANGE | SIF_POS;
	}
	
	SetScrollInfo( SB_HORZ, &si, TRUE );
}

void CTreeListCtrl::SetVertScroll( CRect* pPreClient )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetVertScroll", __DATE__,__FILE__, __LINE__);
	// set vertical scroll
	SCROLLINFO si;
	GetScrollInfo( SB_VERT, &si, SIF_ALL );


//	if( pPreClient->Height() < m_wndHeader.GetHeaderHeight() + GetVisibleCount() * GetItemHeight() || si.nPos != si.nMin )
//	{
		int nShowRows	=	m_rcTreeList.Height() / GetItemHeight();
		int nMax = GetVisibleCount() - 1;

		si.cbSize	= sizeof( SCROLLINFO );
		si.nMin		= 0;
		if(nMax > 0)	{si.nMax = nMax;}
		si.nPos		= si.nPos;
		si.nPage	= max( nShowRows, 1 ); // min( pPreClient->Height()/m_nRowHeight, si.nMax -si.nPos );
		si.fMask	=  SIF_PAGE | SIF_RANGE | SIF_POS;
		if( si.nPos > si.nMax )
			si.nPos = si.nMax;
//	}
//	else
//	{
	/*	si.cbSize	= sizeof( SCROLLINFO );
		si.nMin		= 0;
		si.nMax		= 0;
		si.nPage	= 0;
		si.nPos		= 0;
		si.fMask	=  SIF_PAGE | SIF_RANGE | SIF_POS;*/
	//}

	SetScrollInfo( SB_VERT, &si, TRUE );
}


void CTreeListCtrl::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	IMPLEMENT_LOGGING("CTreeListCtrl::OnHScroll", __DATE__,__FILE__, __LINE__);
	// on horizonal scroll
	SCROLLINFO si;
	GetScrollInfo( SB_HORZ, &si, SIF_ALL );

	switch( nSBCode )
	{
	case SB_LINEUP:
		if( si.nPos > si.nMin )
			si.nPos--;
		break;

	case SB_LINEDOWN:
		if( si.nPos <= si.nMax - (int)si.nPage )
			si.nPos++;
		break;

	case SB_PAGEUP:
		si.nPos -= si.nPage;
		if( si.nPos < si.nMin )
			si.nPos = si.nMin;
		break;

	case SB_PAGEDOWN:
		si.nPos += si.nPage;
		if( si.nPos > si.nMax )
			si.nPos = si.nMax;
		break;

	case SB_TOP:
		si.nPos = si.nMin;
		break;

	case SB_BOTTOM:
		si.nPos = si.nMax;
		break;

	case SB_THUMBPOSITION:
	case SB_THUMBTRACK:
		si.nPos = si.nTrackPos;
		break;

	default:
		break;
	}
	
	SetScrollPos( SB_HORZ, si.nPos );
	m_wndHeader.RedrawWindow();
	RedrawWindow();
}

void CTreeListCtrl::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	IMPLEMENT_LOGGING("CTreeListCtrl::OnVScroll", __DATE__,__FILE__, __LINE__);
	// on vertival scroll
	SCROLLINFO si;
	GetScrollInfo( SB_VERT, &si, SIF_ALL );

	switch( nSBCode )
	{
	case SB_LINEUP:
		if( si.nPos > si.nMin )
			si.nPos--;
		break;

	case SB_LINEDOWN:
		if( si.nPos <= si.nMax - (int)si.nPage )
			si.nPos++;;
		break;

	case SB_PAGEUP:
		si.nPos -= si.nPage;
		if( si.nPos < si.nMin )
			si.nPos = si.nMin;
		break;

	case SB_PAGEDOWN:
		si.nPos += si.nPage;
		if( si.nPos > si.nMax )
			si.nPos = si.nMax;
		break;

	case SB_TOP:
		si.nPos = si.nMin;
		break;

	case SB_BOTTOM:
		si.nPos = si.nMax;
		break;

	case SB_THUMBPOSITION:
	case SB_THUMBTRACK:
		si.nPos = si.nTrackPos;
		break;

	default:
		break;
	}

	if( si.nPos == si.nMin )
	{
		if(  si.nMax-si.nMin+1 <= m_rcClient.Height()/m_nItemHeight )
		{
			Layout();
		}
	}

	SetScrollPos( SB_VERT, si.nPos );

	Invalidate();
}

LRESULT CTreeListCtrl::Notify( DWORD dwMessage, CTreeListItem* pItem, int iCol )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::Notify", __DATE__,__FILE__, __LINE__);
	// notify a message
	NMTREELIST nm;
	nm.hdr.hwndFrom = GetSafeHwnd();
	nm.hdr.idFrom	= GetDlgCtrlID();
	nm.hdr.code		= dwMessage;

	nm.pItem	= pItem;
	nm.iCol		= iCol;

	if( nm.iCol == -1 )
		nm.iCol = 0;

	CWnd* pWnd		= GetParent();
	LRESULT lResult	= 0;

	if( pWnd == NULL )
		lResult = 0;
	else
		lResult = pWnd->SendMessage( WM_NOTIFY, GetDlgCtrlID(), (LPARAM)&nm );

	return lResult;	
}

LRESULT CTreeListCtrl::NotifyDrop( DWORD dwMessage, CTreeListCtrl* pSource , CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::NotifyDrop", __DATE__,__FILE__, __LINE__);
	// notify drop messages
	NMTREELISTDROP nm;
	nm.hdr.hwndFrom	= GetSafeHwnd();
	nm.hdr.idFrom	= GetDlgCtrlID();
	nm.hdr.code		= dwMessage;

	nm.pSource	= pSource;
	nm.pItem	= pItem;

	CWnd* pWnd		= GetParent();
	LRESULT lResult = 0;

	if( pWnd == NULL )
		lResult = 0;
	else
		lResult = pWnd->SendMessage( WM_NOTIFY, GetDlgCtrlID(), (LPARAM)&nm );

	return lResult;	
}

void CTreeListCtrl::Layout()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::Layout", __DATE__,__FILE__, __LINE__);
	// layout size
	if( !IsWindow( m_wndHeader.GetSafeHwnd() ) )
		return;

	//if( !IsWindow( m_wndTip.GetSafeHwnd() ) )		return;

	m_wndTip.Hide();

	SetAllScroll();

	GetClientRect( &m_rcClient );
	m_rcHeader		= m_rcClient;
	m_rcTreeList	= m_rcClient;

	m_wndHeader.SendMessage( HDM_LAYOUT, (WPARAM)0, (LPARAM)&m_rcHeader );

	if( m_rcHeader.Height() == 0 )
	{
		m_rcTreeList = m_rcClient;
	}
	else if( m_rcHeader.Height() == m_rcClient.Height() )
	{
		m_rcTreeList.top	= m_rcHeader.bottom;
		m_rcTreeList.bottom = m_rcHeader.bottom;
	}
	else
	{
		m_rcTreeList.top	= m_rcHeader.bottom;
	}

	SetAllScroll();

	return;
}
int CTreeListCtrl::GetWidth()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetWidth", __DATE__,__FILE__, __LINE__);
	// retrieve total width
	int nWidth = 0;

	for( int iShow=0; iShow<m_arShows.GetSize(); iShow++ )
	{
		CTreeListColumnInfo* pColumnInfo;
		int iCol;

		iCol = m_arShows[iShow];
		pColumnInfo = (CTreeListColumnInfo*)m_arColumns[iCol];

		nWidth += pColumnInfo->m_nWidth;
	}

	return nWidth;
}

int CTreeListCtrl::GetHeight()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetHeight", __DATE__,__FILE__, __LINE__);
	// retrieve total height
	return GetItemHeight() * GetVisibleCount();
}

int CTreeListCtrl::GetColumns()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetColumns", __DATE__,__FILE__, __LINE__);
	// retrieve total number of columns
	return m_arColumns.GetSize();
}

int CTreeListCtrl::GetCount()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetCount", __DATE__,__FILE__, __LINE__);
	// retrieve a count of the items in treelist control
	return m_pRootItem->m_nChild - 1;
}

CImageList* CTreeListCtrl::GetImageList()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetImageList", __DATE__,__FILE__, __LINE__);
	// retrieve global point of image list
	return m_pImageList;
}

void CTreeListCtrl::SetImageList( CImageList* pImageList )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetImageList", __DATE__,__FILE__, __LINE__);
	// set global point of image list
	ASSERT_VALID( pImageList );
	m_pImageList = pImageList;

	Invalidate();
}

CTreeListItem* CTreeListCtrl::GetNextItem( CTreeListItem* pItem, UINT nCode )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetNextItem", __DATE__,__FILE__, __LINE__);
	// retrieve next item specified by pItem
	CTreeListItem* pRetItem = NULL;

	switch( nCode )
	{
	case TLGN_ROOT:				pRetItem = GetRootItem();					break;
	case TLGN_NEXT:				pRetItem = GetNextSiblingItem( pItem );		break;
	case TLGN_PREVIOUS:			pRetItem = GetPrevSiblingItem( pItem );		break;
	case TLGN_PARENT:			pRetItem = GetParentItem( pItem );			break;
	case TLGN_CHILD:			pRetItem = GetChildItem( pItem );			break;
	case TLGN_FIRSTVISIBLE:		pRetItem = GetFirstVisibleItem();			break;
	case TLGN_NEXTVISIBLE:		pRetItem = GetNextVisibleItem( pItem );		break;
	case TLGN_PREVIOUSVISIBLE:	pRetItem = GetPrevVisibleItem( pItem );		break;
	case TLGN_DROPHILITE:		pRetItem = NULL;							break;
	case TLGN_CARET:			pRetItem = GetSelectedItem();				break;
	case TLGN_NEXTSELECT:		pRetItem = GetNextSelectedItem( pItem );	break;
	default:					pRetItem = NULL;							break;
	}

	return pRetItem;
}

BOOL CTreeListCtrl::ItemHasChildren( CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::ItemHasChildren", __DATE__,__FILE__, __LINE__);
	// determine whether the treelist item specified by pItem has a child item
	ASSERT( pItem != NULL );

	if( pItem->m_pChild == NULL )
		return FALSE;

	return TRUE;
}

CTreeListItem* CTreeListCtrl::GetChildItem( CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetChildItem", __DATE__,__FILE__, __LINE__);
	// retrieve first child item specified by pItem
	ASSERT( pItem != NULL );
	return pItem->m_pChild;
}

CTreeListItem* CTreeListCtrl::GetNextSiblingItem( CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetNextSiblingItem", __DATE__,__FILE__, __LINE__);
	// retrieve next sibling item specified by pItem
	ASSERT( pItem != NULL );
	return pItem->m_pNext;
}

CTreeListItem* CTreeListCtrl::GetPrevSiblingItem( CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetPrevSiblingItem", __DATE__,__FILE__, __LINE__);
	// retrieve prev sibling item specified by pItem
	ASSERT( pItem != NULL );
	return pItem->m_pPrev;
}

CTreeListItem* CTreeListCtrl::GetParentItem( CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetParentItem", __DATE__,__FILE__, __LINE__);
	// retrieve parent item specified by pItem
	ASSERT( pItem != NULL );
	if( pItem->m_pParent == m_pRootItem )
		return NULL;
	else
		return pItem->m_pParent;
}

CTreeListItem* CTreeListCtrl::GetFirstVisibleItem()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetFirstVisibleItem", __DATE__,__FILE__, __LINE__);
	// retrieve first visible item of treelist
	if( m_pRootItem->m_pChild != NULL )
		return m_pRootItem->m_pChild;

	return NULL;
}

CTreeListItem* CTreeListCtrl::GetNextVisibleItem( CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetNextVisibleItem", __DATE__,__FILE__, __LINE__);
	// retrieve next visible item of treelist
	ASSERT( pItem != NULL );

	return GetNextVisibleItem( pItem, TRUE );
}

CTreeListItem* CTreeListCtrl::GetNextVisibleItem( CTreeListItem* pItem, BOOL bChild )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetNextVisibleItem", __DATE__,__FILE__, __LINE__);
	// retrieve next visible item of treelist
	ASSERT( pItem != NULL );

	if( pItem->GetState()&TLIS_EXPANDED )
		if( bChild && pItem->m_pChild != NULL )
			return pItem->m_pChild;

	if( pItem->m_pNext != NULL )
		return pItem->m_pNext;

	if( pItem != m_pRootItem )
	{
		pItem = pItem->m_pParent;
		if( pItem != m_pRootItem )
			return GetNextVisibleItem( pItem, FALSE );
	}

	return NULL;
}

CTreeListItem* CTreeListCtrl::GetPrevVisibleItem( CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetPrevVisibleItem", __DATE__,__FILE__, __LINE__);
	// retrieve previous visible item of treelist
	ASSERT( pItem != NULL );

	return GetPrevVisibleItem( pItem, TRUE );
}

CTreeListItem* CTreeListCtrl::GetPrevVisibleItem( CTreeListItem* pItem, BOOL bChild )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetPrevVisibleItem", __DATE__,__FILE__, __LINE__);
	// retrieve previous visible item of treelist
	ASSERT( pItem != NULL );

	if( pItem->m_pPrev != NULL )
	{
		pItem = pItem->m_pPrev;
		if( pItem->GetState()&TLIS_EXPANDED && pItem->m_pChild != NULL )
		{
			pItem = GetLastVisibleItem( pItem );
		}
		return pItem;
	}

	pItem = pItem->m_pParent;
	if( pItem != m_pRootItem )
		return pItem;

	return FALSE;
}

CTreeListItem* CTreeListCtrl::GetLastVisibleItem( CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetLastVisibleItem", __DATE__,__FILE__, __LINE__);
	// retrieve last visible child item of item specified by pItem
	ASSERT( pItem != NULL && pItem->m_pChild != NULL );

	CTreeListItem* pChild;
	pChild = pItem->m_pChild;

	while( pChild->m_pNext != NULL )
	{
		pChild = pChild->m_pNext;
	}

	if( pChild->GetState()&TLIS_EXPANDED && pChild->m_pChild != NULL  )
		return GetLastVisibleItem( pChild );
	else
		return pChild;
}


CTreeListItem* CTreeListCtrl::GetSelectedItem()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetSelectedItem", __DATE__,__FILE__, __LINE__);
	// always retrieve first selected item
	POSITION pos = GetFirstSelectedItemPosition();
	return GetNextSelectedItem( pos );
}

CTreeListItem* CTreeListCtrl::GetNextSelectedItem( CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetNextSelectedItem", __DATE__,__FILE__, __LINE__);
	// retrieve next selected item specified by pItem
	if( pItem == NULL )
		return NULL;

	if( pItem == m_pRootItem && m_arSelects.GetSize() > 0 )
		return (CTreeListItem*)m_arSelects[0];

	for( int iSel = 0; iSel<m_arSelects.GetSize()-1; iSel++ )
	{
		if( pItem == (CTreeListItem*)m_arSelects[iSel] )
			return (CTreeListItem*)m_arSelects[iSel+1];
	}

	return NULL;
}

CTreeListItem* CTreeListCtrl::GetRootItem()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetRootItem", __DATE__,__FILE__, __LINE__);
	// retrieve root item of treelist control
	return m_pRootItem->m_pChild;
}

DWORD CTreeListCtrl::GetItemState( CTreeListItem* pItem, DWORD nStateMask )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetItemState", __DATE__,__FILE__, __LINE__);
	// retrieve status of item specified by pItem
	ASSERT( pItem != NULL );
	return pItem->GetState( nStateMask );
}

void CTreeListCtrl::SetItemState( CTreeListItem* pItem, DWORD dwAddState, DWORD dwRemoveState )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetItemState", __DATE__,__FILE__, __LINE__);
	// set status of item specified by pItem
	ASSERT( pItem != NULL );
	pItem->SetState( dwAddState, dwRemoveState );
	Invalidate();
}

void CTreeListCtrl::GetItemImage( CTreeListItem* pItem, int& nImage, int& nExpandImage, int& nSelectedImage, int& nExpandSelectedImage )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetItemImage", __DATE__,__FILE__, __LINE__);
	// retrieve images of item specified by pItem
	ASSERT( pItem != NULL );
	pItem->GetImage( nImage, nExpandImage, nSelectedImage, nExpandSelectedImage );
}

void CTreeListCtrl::SetItemImage( CTreeListItem* pItem, int nImage, int nExpandImage, int nSelectedImage, int nExpandSelectedImage )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetItemImage", __DATE__,__FILE__, __LINE__);
	// set images of item specified by pItem
	ASSERT( pItem != NULL );
	pItem->SetState( TLIS_SHOWTREEIMAGE );
	pItem->SetImage( nImage, nExpandImage, nSelectedImage, nExpandSelectedImage );
	Invalidate();
}	

LPCTSTR	CTreeListCtrl::GetItemText( CTreeListItem* pItem, int nSubItem )
{
	IMPLEMENT_LOGGING("LPCTSTR	CTreeListCtrl::GetItemText", __DATE__,__FILE__, __LINE__);
	// retrieve text of item specified by pItem
	ASSERT( pItem != NULL );
	return pItem->GetText( nSubItem );
}

BOOL CTreeListCtrl::SetItemText( CTreeListItem* pItem, LPCTSTR lpszText, int nSubItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetItemText", __DATE__,__FILE__, __LINE__);
	// set text of item specified by pItem
	ASSERT( pItem != NULL );
	BOOL bRet = pItem->SetText( lpszText, nSubItem );
	Invalidate();
	return bRet;
}

DWORD CTreeListCtrl::GetItemData( CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetItemData", __DATE__,__FILE__, __LINE__);
	// retrieve 32-bit data of item specified by pItem
	ASSERT( pItem != NULL );
	return pItem->GetData();
}

void CTreeListCtrl::SetItemData( CTreeListItem* pItem, DWORD dwData )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetItemData", __DATE__,__FILE__, __LINE__);
	// set 32-bit data of item specified by pItem
	ASSERT( pItem != NULL );
	pItem->SetData( dwData );
}

int CTreeListCtrl::GetVisibleCount()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetVisibleCount", __DATE__,__FILE__, __LINE__);
	// retrieve the  number of visible items in the treelist control
	return GetVisibleCount( m_pRootItem );
}

int CTreeListCtrl::GetItemHeight()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetItemHeight", __DATE__,__FILE__, __LINE__);
	// retrieve the height of the item
	return m_nItemHeight;
}

int CTreeListCtrl::SetItemHeight( int cyHeight )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetItemHeight", __DATE__,__FILE__, __LINE__);
	// set the height of the item
	int nOldHeight = m_nItemHeight;

	if( cyHeight == -1 )
	{
		ASSERT( m_Font.m_hObject != NULL );

		LOGFONT lf;
		m_Font.GetLogFont( &lf );
	
		if( lf.lfHeight < 0 )
		{
			m_nItemHeight = -lf.lfHeight + 5;
		}
		else if( lf.lfHeight == 0 )
		{
			m_nItemHeight = DEFAULT_HEIGHT;
		}
		else
		{
			m_nItemHeight = lf.lfHeight + 5;
		}
	}
	else
	{
		m_nItemHeight = cyHeight;
	}

	if( m_nItemHeight % 2 == 1 )
		m_nItemHeight ++;

	Invalidate();

	return nOldHeight;
}

COLORREF CTreeListCtrl::GetBkColor( int nColor )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetBkColor", __DATE__,__FILE__, __LINE__);
	// set background color of treelist control
	COLORREF clr;

	switch( nColor )
	{
	case 0:		clr = m_crBkColor_0;		break;
	case 1:		clr = m_crBkColor_1;		break;
	case 2:		clr = m_crBkColor_2;		break;
	default:	clr = m_crBkColor;		break;
	}

	return clr;
}

COLORREF CTreeListCtrl::SetBkColor( COLORREF clr, int nColor )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetBkColor", __DATE__,__FILE__, __LINE__);
	// retrieve background color of treelist control
	COLORREF oldclr;

	switch( nColor )
	{
	case 0:		oldclr = m_crBkColor_0;		m_crBkColor_0 = clr;		break;
	case 1:		oldclr = m_crBkColor_1;		m_crBkColor_1 = clr;		break;
	case 2:		oldclr = m_crBkColor_2;		m_crBkColor_2 = clr;		break;
	default:	oldclr = m_crBkColor;		m_crBkColor = clr;		break;
	}

	Invalidate();
	return oldclr;
}

COLORREF CTreeListCtrl::GetTextColor()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetTextColor", __DATE__,__FILE__, __LINE__);
	// retrieve the color of the text
	return m_crText;
}

COLORREF CTreeListCtrl::SetTextColor( COLORREF clr )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetTextColor", __DATE__,__FILE__, __LINE__);
	// set the color of the text
	COLORREF oldclr;
	oldclr = m_crText;
	m_crText = clr;

	Invalidate();

	return m_crText;
}

int CTreeListCtrl::GetCheck( CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetCheck", __DATE__,__FILE__, __LINE__);
	// retrieve an item checked box state
	ASSERT( pItem != NULL );
	return pItem->GetCheck();
}

void CTreeListCtrl::SetCheck( CTreeListItem* pItem, BOOL bCheck )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetCheck", __DATE__,__FILE__, __LINE__);
	// set an item checked box state
	ASSERT( pItem != NULL );
	pItem->SetCheck( bCheck );
	
	if( pItem->GetCheck() )
	{
		SetItemChildStatus( pItem, TLIS_CHECKED, TLIS_CHECKED_NULL );
		SetItemParentStatus( pItem, TLIS_CHECKEDMASK );
	}
	else
	{
		SetItemChildStatus( pItem, TLIS_CHECKED_NULL, TLIS_CHECKED );
		SetItemParentStatus( pItem, TLIS_CHECKEDMASK );
	}

	Invalidate();
}

int CTreeListCtrl::GetLock( CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetLock", __DATE__,__FILE__, __LINE__);
	// retrieve an item locked box state
	ASSERT( pItem != NULL );
	return pItem->GetLock();
}

void CTreeListCtrl::SetLock( CTreeListItem* pItem, BOOL bLock )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetLock", __DATE__,__FILE__, __LINE__);
	// set an item locked box state
	ASSERT( pItem != NULL );
	pItem->SetLock( bLock );

	if( pItem->GetLock() )
	{
		SetItemChildStatus( pItem, TLIS_LOCKED, TLIS_LOCKED_NULL );
		SetItemParentStatus( pItem, TLIS_LOCKEDMASK );
	}
	else
	{
		SetItemChildStatus( pItem, TLIS_LOCKED_NULL, TLIS_LOCKED );
		SetItemParentStatus( pItem, TLIS_LOCKEDMASK );
	}
	Invalidate();
}

BOOL CTreeListCtrl::GetItemRect( CTreeListItem* pItem, LPRECT lpRect )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetItemRect", __DATE__,__FILE__, __LINE__);
	// retrieve rect of an item
	int nFirstRow	=	GetScrollPos( SB_VERT );
	int	nShowRows	=	m_rcTreeList.Height() / GetItemHeight() + 1;

	CRect rcItem;
	rcItem.SetRect( 0, 0, GetWidth(), GetItemHeight() );
	rcItem.OffsetRect( m_rcTreeList.left, m_rcTreeList.top );
	rcItem.OffsetRect( -GetScrollPos( SB_HORZ ), 0 );

	for( int i = nFirstRow; i<= nFirstRow + nShowRows && i < GetVisibleCount(); i++ )
	{
		CTreeListItem* pShowItem = GetFirstShowItem( i );
		if( pShowItem == pItem )
		{
			lpRect->left	= rcItem.left;
			lpRect->right	= rcItem.right;
			lpRect->top		= rcItem.top;
			lpRect->bottom	= rcItem.bottom;

//			lpRect->top		+= 1;
//			lpRect->bottom	-= 1;

			return TRUE;
		}

		rcItem.OffsetRect( 0, GetItemHeight() );
	}
	return FALSE;
}

BOOL CTreeListCtrl::GetItemRect( CTreeListItem* pItem, int iSubItem, LPRECT lpRect, BOOL bTextOnly )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetItemRect", __DATE__,__FILE__, __LINE__);
	// retrieve rect of a column of an item
	CRect rcItem;
	
	if( GetItemRect( pItem, rcItem ) )
	{
		int nColPos = 0;
		CTreeListColumnInfo* pColumnInfo;

		for( int iShow = 0; iShow < m_arShows.GetSize(); iShow++, nColPos += pColumnInfo->GetWidth() )
		{
			int iCol;

			iCol = m_arShows[iShow];
			pColumnInfo = (CTreeListColumnInfo*)m_arColumns[iCol];

			if( iCol == iSubItem )
			{
				CRect rcColumn;
				rcColumn.SetRect( 0, rcItem.top, pColumnInfo->GetWidth(), rcItem.bottom );
				rcColumn.OffsetRect( nColPos, 0 );
				rcColumn.OffsetRect( rcItem.left, 0 );

				lpRect->left	= rcColumn.left;
				lpRect->right	= rcColumn.right;
				lpRect->top		= rcColumn.top;
				lpRect->bottom	= rcColumn.bottom;

				if( iCol == 0 && bTextOnly )
				{
					if ( m_dwStyle&TLC_TREELIST )
						return GetItemRectTree( pItem, lpRect );
					else
						return GetItemRectMain( pItem, lpRect );

				}

//				lpRect->left	+= 1;
//				lpRect->right	-= 1;

				return TRUE;
			}
		}
	}
	
	return FALSE;
}	

BOOL CTreeListCtrl::GetItemRectTree( CTreeListItem* pItem, LPRECT lpRect )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetItemRectTree", __DATE__,__FILE__, __LINE__);
	int nPerfix = 0;

	// skip perfix and tree line
	nPerfix += TLL_BORDER;
	nPerfix += (pItem->m_nLevel-1) * TLL_WIDTH;

	// skip tree button
	if( m_dwStyle&(TLC_TREELINE|TLC_BUTTON) )
		nPerfix += TLL_WIDTH;

	// skip tree check box
	if( m_dwStyle&TLC_CHECKBOX )
		if( pItem->GetState()&TLIS_SHOWCHECKBOX )
			nPerfix += TLL_WIDTH;

	// skip tree lock box
	if( m_dwStyle&TLC_LOCKBOX )
		if( pItem->GetState()&TLIS_SHOWLOCKBOX )
			nPerfix += TLL_WIDTH/2;

	// skip tree image
	if( m_dwStyle&TLC_IMAGE )
		if( pItem->GetState()&TLIS_SHOWTREEIMAGE )
			nPerfix += TLL_WIDTH;

	// skip gaps between text and image/loc
	if( m_dwStyle&TLC_LOCKBOX || m_dwStyle&TLC_IMAGE )
		nPerfix += 2;

	if( lpRect->left + nPerfix + 2 <= lpRect->right )
	{
		lpRect->left	+= nPerfix;
//		lpRect->left	+= 1;
//		lpRect->right	-= 1;
		return TRUE;
	}
	
	lpRect->left	+= nPerfix;
	return FALSE;
}

BOOL CTreeListCtrl::GetItemRectMain( CTreeListItem* pItem, LPRECT lpRect )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetItemRectMain", __DATE__,__FILE__, __LINE__);
	int nPerfix = 0;

	// skip perfix
	nPerfix += TLL_BORDER;

	// skip tree button
	if( m_dwStyle&(TLC_TREELINE|TLC_BUTTON) )
		nPerfix += TLL_WIDTH;

	// skip tree check box
	if( m_dwStyle&TLC_CHECKBOX )
		if( pItem->GetState()&TLIS_SHOWCHECKBOX )
			nPerfix += TLL_WIDTH;

	// skip tree lock box
	if( m_dwStyle&TLC_LOCKBOX )
		if( pItem->GetState()&TLIS_SHOWLOCKBOX )
			nPerfix += TLL_WIDTH/2;

	// skip tree image
	if( m_dwStyle&TLC_IMAGE )
		if( pItem->GetState()&TLIS_SHOWTREEIMAGE )
			nPerfix += TLL_WIDTH;

	// skip gaps between text and image/loc
	if( m_dwStyle&TLC_LOCKBOX || m_dwStyle&TLC_IMAGE )
		nPerfix += 2;

	if( lpRect->left + nPerfix + 2 <= lpRect->right )
	{
		lpRect->left	+= nPerfix;
//		lpRect->left	+= 1;
//		lpRect->right	-= 1;
		return TRUE;
	}
	
	lpRect->left += nPerfix;
	return FALSE;
}

BOOL CTreeListCtrl::EnsureVisible( CTreeListItem* pItem, int iSubItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::EnsureVisible", __DATE__,__FILE__, __LINE__);
	// ensure an item is visible that specified by pItem
	int nItem		=	GetTreeListItem( pItem );
	int nFirstRow	=	GetScrollPos( SB_VERT );
	int nShowRows	=	m_rcTreeList.Height() / GetItemHeight();

	if( nShowRows > 0 )
		nShowRows = nShowRows - 1;

	if( nItem < nFirstRow )
	{
		SetScrollPos( SB_VERT, nItem );
	}
	else
	{
		if( nItem > nFirstRow + nShowRows )
			SetScrollPos( SB_VERT, nItem - nShowRows );
	}

	CRect rcClient;
	GetClientRect( rcClient );
	int	nCurrent	=	GetScrollPos( SB_HORZ );
	int nLeft		=	0;
	int nRight		=	0;
	for( int iShow = 0; iShow < m_arShows.GetSize(); iShow ++ )
	{
		CTreeListColumnInfo* pColumnInfo;
		pColumnInfo = (CTreeListColumnInfo*)m_arColumns[m_arShows[iShow]];

		if( m_arShows[iShow] == iSubItem )
		{
			nRight = pColumnInfo->GetWidth();
			break;
		}
		nLeft += pColumnInfo->GetWidth();
	}
	nRight = nLeft + nRight;

	if( iSubItem == 0 )
	{
		int iLevel		= pItem->m_nLevel - 1;
		CTreeListColumnInfo* pColumnInfo;
		pColumnInfo = (CTreeListColumnInfo*)m_arColumns[0];
		nLeft += TLL_WIDTH * iLevel;
	}

	if( nLeft <= nCurrent )
	{
		
		SetScrollPos( SB_HORZ, nLeft );
	}
	else
	{
		if( nCurrent + rcClient.Width() < nRight )
		{
			if( nRight-nLeft <= rcClient.Width() )
			{
				SetScrollPos( SB_HORZ, nRight - rcClient.Width() );
			}
			else
			{
				SetScrollPos( SB_HORZ, nLeft );
			}
		}
	}

	Invalidate();
	return TRUE;
}

int	CTreeListCtrl::GetItemCount()
{
	IMPLEMENT_LOGGING("int	CTreeListCtrl::GetItemCount", __DATE__,__FILE__, __LINE__);
	// retrieve count of all items.
	return m_pRootItem->m_nChild - 1;
}

int CTreeListCtrl::GetNextItem( int nItem, int nFlags )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetNextItem", __DATE__,__FILE__, __LINE__);
	// retrieve next item specified by nItem
	CTreeListItem* pItem = NULL;
	pItem = GetTreeListItem( nItem );

	if( pItem == NULL )
		return -1;

	if( nFlags&TLNI_ABOVE )
		return GetTreeListItem( GetPrevSiblingItem( pItem ) );

	if( nFlags&TLNI_BELOW )
		return GetTreeListItem( GetNextSiblingItem( pItem ) );

	return -1;
}

POSITION CTreeListCtrl::GetFirstSelectedItemPosition()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetFirstSelectedItemPosition", __DATE__,__FILE__, __LINE__);
	// retrieve position of selected item
	if( m_arSelects.GetSize() == 0 )
		return NULL;

	POSITION pos = (POSITION)m_pRootItem;
	return pos;
}

CTreeListItem* CTreeListCtrl::GetNextSelectedItem( POSITION& pos )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetNextSelectedItem", __DATE__,__FILE__, __LINE__);
	// retrieve next selected item specified by pItem
	CTreeListItem* pItem;
	
	pItem = GetNextSelectedItem( (CTreeListItem*)pos );
	pos = (POSITION)pItem;
	return pItem;
}

int CTreeListCtrl::GetStringWidth( LPCTSTR lpsz )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetStringWidth", __DATE__,__FILE__, __LINE__);
	// retrieve width of string in pixel
	CPaintDC dc(this);

	CFont* pOldFont = dc.SelectObject( GetFont() );
	CSize sz = dc.GetTextExtent( lpsz, _tcslen(lpsz) );
	dc.SelectObject( pOldFont );

	return sz.cx + 6;
}

DWORD CTreeListCtrl::SetColumnFormat( int nCol, DWORD dwAdd, DWORD dwRemove )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetColumnFormat", __DATE__,__FILE__, __LINE__);
	// set column fromat
	ASSERT( 0 <= nCol && nCol < m_arColumns.GetSize() );

	CTreeListColumnInfo* pColumnInfo;
	pColumnInfo = (CTreeListColumnInfo*)m_arColumns[nCol];

	DWORD dwFormat = pColumnInfo->SetFormat( dwAdd, dwRemove );

	Invalidate();

	return dwFormat;
}

DWORD CTreeListCtrl::GetColumnFormat( int nCol, DWORD dwMask )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetColumnFormat", __DATE__,__FILE__, __LINE__);
	// retrieve column format
	ASSERT( 0 <= nCol && nCol < m_arColumns.GetSize() );

	CTreeListColumnInfo* pColumnInfo;
	pColumnInfo = (CTreeListColumnInfo*)m_arColumns[nCol];

	return pColumnInfo->GetFormat();
}

DWORD CTreeListCtrl::SetColumnModify( int nCol, DWORD dwModify )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetColumnModify", __DATE__,__FILE__, __LINE__);
	// retrieve column modify
	ASSERT( 0 <= nCol && nCol < m_arColumns.GetSize() );

	CTreeListColumnInfo* pColumnInfo;
	pColumnInfo = (CTreeListColumnInfo*)m_arColumns[nCol];

	return pColumnInfo->SetModify( dwModify );
}

DWORD CTreeListCtrl::GetColumnModify( int nCol )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetColumnModify", __DATE__,__FILE__, __LINE__);
	// retrieve column modify
	ASSERT( 0 <= nCol && nCol < m_arColumns.GetSize() );

	CTreeListColumnInfo* pColumnInfo;
	pColumnInfo = (CTreeListColumnInfo*)m_arColumns[nCol];

	return pColumnInfo->GetModify();
}

int CTreeListCtrl::SetColumnWidth( int nCol, int nWidth, int nMin, int nMax )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetColumnWidth", __DATE__,__FILE__, __LINE__);
	// set column width
	if( nWidth == TLSCW_AUTOSIZE || nWidth == TLSCW_AUTOSIZE_USEHEADER )
	{
		// AutoSetWidth();		// all show items
		if( nWidth == TLSCW_AUTOSIZE )
		{
			nWidth = nMin;
		}
		else
		{
			nWidth = GetStringWidth( GetColumnText( nCol ) ) + 4;
		}

		CTreeListItem* pShowItem = GetFirstShowItem( 0 );
		while( pShowItem != NULL )
		{
			ASSERT( pShowItem != NULL );

			int x = 0;
			if( nCol == 0 )
			{
				if( m_dwStyle&TLC_TREELIST )
					x += (pShowItem->m_nLevel-1) * TLL_WIDTH;

				if( m_dwStyle&(TLC_TREELINE|TLC_BUTTON) )
					x += TLL_WIDTH;

				if( m_dwStyle&TLC_CHECKBOX )
					if( pShowItem->GetState()&TLIS_SHOWCHECKBOX )
						x += TLL_WIDTH;

				if( m_dwStyle&TLC_LOCKBOX )
					if( pShowItem->GetState()&TLIS_SHOWLOCKBOX )
						x += TLL_WIDTH/2;

				if( m_dwStyle&TLC_IMAGE )
					if( pShowItem->GetState()&TLIS_SHOWTREEIMAGE )
						x += TLL_WIDTH;

				if( m_dwStyle&TLC_LOCKBOX || m_dwStyle&TLC_IMAGE )
					x += 2;
			}

			x += GetStringWidth( pShowItem->GetText( nCol ) );
			x += 2;

			if( nWidth < x )
				nWidth = x;

			pShowItem = GetNextShowItem( pShowItem );
		}
		return SetColumnWidth( nCol, nWidth, nMin, nMax );
	}
	else if( nWidth == TLSCW_AUTOSIZEV || nWidth == TLSCW_AUTOSIZEV_USEHEADER )
	{
		// AutoSetWidth();		// all visible items
		if( nWidth == TLSCW_AUTOSIZEV )
		{
			nWidth = nMin;
		}
		else
		{
			nWidth = GetStringWidth( GetColumnText( nCol ) ) + 4;
		}

		CTreeListItem* pVisibleItem = GetFirstVisibleItem();
		while( pVisibleItem != NULL )
		{
			ASSERT( pVisibleItem != NULL );
			int x = 0;
			if( nCol == 0 )
			{
				if( m_dwStyle&TLC_TREELIST )
					x += (pVisibleItem->m_nLevel-1) * TLL_WIDTH;

				if( m_dwStyle&(TLC_TREELINE|TLC_BUTTON) )
					x += TLL_WIDTH;

				if( m_dwStyle&TLC_CHECKBOX )
					if( pVisibleItem->GetState()&TLIS_SHOWCHECKBOX )
						x += TLL_WIDTH;

				if( m_dwStyle&TLC_LOCKBOX )
					if( pVisibleItem->GetState()&TLIS_SHOWLOCKBOX )
						x += TLL_WIDTH/2;

				if( m_dwStyle&TLC_IMAGE )
					if( pVisibleItem->GetState()&TLIS_SHOWTREEIMAGE )
						x += TLL_WIDTH;

				if( m_dwStyle&TLC_LOCKBOX || m_dwStyle&TLC_IMAGE )
					x += 2;
			}

			x += GetStringWidth( pVisibleItem->GetText( nCol ) );
			x += 2;

			if( nWidth < x )
				nWidth = x;

			pVisibleItem = GetNextShowItem( pVisibleItem );
		}
		return SetColumnWidth( nCol, nWidth, nMin, nMax );
	}
	else
	{
		// SetWidth()
		ASSERT( 0 <= nCol && nCol < m_arColumns.GetSize() );

		CTreeListColumnInfo* pColumnInfo;
		pColumnInfo = (CTreeListColumnInfo*)m_arColumns[nCol];

		int iRet = pColumnInfo->SetWidth( nWidth, nMin, nMax );
		Layout();
		Invalidate();
		return iRet;
	}
}

int CTreeListCtrl::GetColumnWidth( int nCol )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetColumnWidth", __DATE__,__FILE__, __LINE__);
	// retrieve column width
	ASSERT( 0 <= nCol && nCol < m_arColumns.GetSize() );

	CTreeListColumnInfo* pColumnInfo;
	pColumnInfo = (CTreeListColumnInfo*)m_arColumns[nCol];

	return pColumnInfo->GetWidth();
}

int CTreeListCtrl::SetColumnImage( int nCol, int iImage )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetColumnImage", __DATE__,__FILE__, __LINE__);
	// set index of column image
	ASSERT( 0 <= nCol && nCol < m_arColumns.GetSize() );

	CTreeListColumnInfo* pColumnInfo;
	pColumnInfo = (CTreeListColumnInfo*)m_arColumns[nCol];

	int iRet = pColumnInfo->SetImage( iImage );

	Invalidate();

	return iRet;
}

int CTreeListCtrl::GetColumnImage( int nCol )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetColumnImage", __DATE__,__FILE__, __LINE__);
	// retrieve index of column image
	ASSERT( 0 <= nCol && nCol < m_arColumns.GetSize() );

	CTreeListColumnInfo* pColumnInfo;
	pColumnInfo = (CTreeListColumnInfo*)m_arColumns[nCol];

	return pColumnInfo->GetImage();
}

BOOL CTreeListCtrl::SetColumnText( int nCol, LPCTSTR lpszText )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetColumnText", __DATE__,__FILE__, __LINE__);
	// set text of column
	ASSERT( 0 <= nCol && nCol < m_arColumns.GetSize() );

	CTreeListColumnInfo* pColumnInfo;
	pColumnInfo = (CTreeListColumnInfo*)m_arColumns[nCol];

	BOOL bRet = pColumnInfo->SetText( lpszText );

	Invalidate();

	return bRet;
}

LPCTSTR CTreeListCtrl::GetColumnText( int nCol )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetColumnText", __DATE__,__FILE__, __LINE__);
	// retrieve text of column
	ASSERT( 0 <= nCol && nCol < m_arColumns.GetSize() );

	CTreeListColumnInfo* pColumnInfo;
	pColumnInfo = (CTreeListColumnInfo*)m_arColumns[nCol];

	return pColumnInfo->GetText();
}

BOOL CTreeListCtrl::SetColumnDefaultText( int nCol, LPCTSTR lpszText )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetColumnDefaultText", __DATE__,__FILE__, __LINE__);
	// set default text of column
	ASSERT( 0 <= nCol && nCol < m_arColumns.GetSize() );

	CTreeListColumnInfo* pColumnInfo;
	pColumnInfo = (CTreeListColumnInfo*)m_arColumns[nCol];

	return pColumnInfo->SetDefaultText( lpszText );
}

LPCTSTR CTreeListCtrl::GetColumnDefaultText( int nCol )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetColumnDefaultText", __DATE__,__FILE__, __LINE__);
	// retrieve default text of column
	ASSERT( 0 <= nCol && nCol < m_arColumns.GetSize() );

	CTreeListColumnInfo* pColumnInfo;
	pColumnInfo = (CTreeListColumnInfo*)m_arColumns[nCol];

	return pColumnInfo->GetDefaultText();
}

DWORD CTreeListCtrl::GetItemState( int nItem, DWORD nStateMask )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetItemState", __DATE__,__FILE__, __LINE__);
	// retrieve item state ( list )
	ASSERT( 0 <= nItem && nItem < GetCount() );
	return GetItemState( GetTreeListItem(nItem), nStateMask );
}

void CTreeListCtrl::SetItemState( int nItem, DWORD dwAddState, DWORD dwRemoveState )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetItemState", __DATE__,__FILE__, __LINE__);
	// set item state ( list )
	ASSERT( 0 <= nItem && nItem < GetCount() );
	SetItemState( GetTreeListItem(nItem), dwAddState, dwRemoveState );
}

void CTreeListCtrl::GetItemImage( int nItem, int& nImage, int& nExpandImage, int& nSelectedImage, int& nExpandSelectedImage )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetItemImage", __DATE__,__FILE__, __LINE__);
	// retrieve index of item image ( list )
	ASSERT( 0 <= nItem && nItem < GetCount() );
	GetItemImage( GetTreeListItem(nItem), nImage, nExpandImage, nSelectedImage, nExpandSelectedImage );
}

void CTreeListCtrl::SetItemImage( int nItem, int nImage, int nExpandImage, int nSelectedImage, int nExpandSelectedImage )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetItemImage", __DATE__,__FILE__, __LINE__);
	// set index of item image ( list )
	ASSERT( 0 <= nItem && nItem < GetCount() );
	SetItemImage( GetTreeListItem(nItem), nImage, nExpandImage, nSelectedImage, nExpandSelectedImage );
}

LPCTSTR CTreeListCtrl::GetItemText( int nItem, int nSubItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetItemText", __DATE__,__FILE__, __LINE__);
	// retrieve text of sub item ( list )
	ASSERT( 0 <= nItem && nItem < GetCount() );
	return GetItemText( GetTreeListItem(nItem), nSubItem );
}

BOOL CTreeListCtrl::SetItemText( int nItem, int nSubItem, LPCTSTR lpszText )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetItemText", __DATE__,__FILE__, __LINE__);
	// set text of sub item ( list )
	ASSERT( 0 <= nItem && nItem < GetCount() );
	return SetItemText( GetTreeListItem(nItem), lpszText, nSubItem );
}

DWORD CTreeListCtrl::GetItemData( int nItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetItemData", __DATE__,__FILE__, __LINE__);
	// retrieve data of item ( list )
	ASSERT( 0 <= nItem && nItem < GetCount() );
	return GetItemData( GetTreeListItem(nItem) );
}

void CTreeListCtrl::SetItemData( int nItem, DWORD dwData )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetItemData", __DATE__,__FILE__, __LINE__);
	// set data of item ( list )
	ASSERT( 0 <= nItem && nItem < GetCount() );
	SetItemData( GetTreeListItem(nItem), dwData );
}

BOOL CTreeListCtrl::GetViewRect( LPRECT lpRect )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetViewRect", __DATE__,__FILE__, __LINE__);
	// retrieve list rect ( list )
	lpRect->left	= m_rcTreeList.left;
	lpRect->right	= m_rcTreeList.right;
	lpRect->top		= m_rcTreeList.top;
	lpRect->bottom	= m_rcTreeList.bottom;

	return m_rcTreeList.IsRectEmpty();
}

int CTreeListCtrl::GetTopIndex()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetTopIndex", __DATE__,__FILE__, __LINE__);
	// retrieve index of first visible item ( list )
	return GetScrollPos( SB_VERT );
}

int CTreeListCtrl::GetCountPerPage()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetCountPerPage", __DATE__,__FILE__, __LINE__);
	// retrieve rows of visible item of one page ( list )
	return m_rcTreeList.Height() / GetItemHeight() + 1;
}

BOOL CTreeListCtrl::GetOrigin( LPPOINT lpPoint )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetOrigin", __DATE__,__FILE__, __LINE__);
	// retrieve origin point of a list ( list )
	lpPoint->x = m_rcTreeList.left;
	lpPoint->y = m_rcTreeList.top;
	return TRUE;
}

UINT CTreeListCtrl::GetSelectedCount()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetSelectedCount", __DATE__,__FILE__, __LINE__);
	// retrieve count of selected item ( list )
	return m_arSelects.GetSize();
}

BOOL CTreeListCtrl::SetColumnOrderArray( int iCount, LPINT piArray )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetColumnOrderArray", __DATE__,__FILE__, __LINE__);
	// set show order of columns
	m_arShows.RemoveAll();

	for( int iShow=0; iShow<iCount; iShow++ )
		m_arShows.Add( piArray[iShow] );

	Invalidate();

	return TRUE;
}

BOOL CTreeListCtrl::GetColumnOrderArray( LPINT piArray, int iCount )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetColumnOrderArray", __DATE__,__FILE__, __LINE__);
	// retrieve column show order
	if( iCount <= 0 )
		iCount = m_arShows.GetSize();

	for( int iShow=0; iShow<m_arShows.GetSize(); iShow++ )
		piArray[iShow] = m_arShows[iShow];

	Invalidate();

	return TRUE;
}

CTreeListHeaderCtrl* CTreeListCtrl::GetHeaderCtrl()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetHeaderCtrl", __DATE__,__FILE__, __LINE__);
	// retrieve header control
	return &m_wndHeader;
}

int CTreeListCtrl::GetSelectionMark()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetSelectionMark", __DATE__,__FILE__, __LINE__);
	// retrieve selection 
	POSITION pos = GetFirstSelectedItemPosition();
	return GetTreeListItem( GetNextSelectedItem( pos ) );
}

int CTreeListCtrl::SetSelectionMark( int iIndex )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetSelectionMark", __DATE__,__FILE__, __LINE__);
	// set selection mark of an item
	int nItem = GetSelectionMark();
	SelectItem( GetTreeListItem( iIndex ), 0, FALSE );

	Invalidate();

	return nItem;
}

void CTreeListCtrl::SetFont()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetFont", __DATE__,__FILE__, __LINE__);
	// set font with font of parent window
	CWnd* pWnd = GetParent();
	if( pWnd == NULL )
		return;

	CFont* pFont = pWnd->GetFont();
	if( pFont == NULL )
		return;

	LOGFONT lf;
	pFont->GetLogFont(&lf);
	m_Font.DeleteObject();
	m_Font.CreateFontIndirect(&lf);

	Invalidate();
}

CFont* CTreeListCtrl::GetFont()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetFont", __DATE__,__FILE__, __LINE__);
	// retrieve font
	return &m_Font;
}

DWORD CTreeListCtrl::SetStyle( DWORD dwStyle )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetStyle", __DATE__,__FILE__, __LINE__);
	// set style of treelist control
	DWORD dwoldStyle = m_dwStyle;
	m_dwStyle = dwStyle;

	if( !(m_dwStyle&TLC_TREELIST ) )
	{
		m_dwStyle &= ~TLC_TREELINE;
		m_dwStyle &= ~TLC_ROOTLINE;
		m_dwStyle &= ~TLC_BUTTON;	
	}

	if( !(m_dwStyle&TLC_HGRID) )
	{
		m_dwStyle &= ~TLC_TGRID;
		m_dwStyle &= ~TLC_HGRID_EXT;
		m_dwStyle &= ~TLC_HGRID_FULL;
	}

	if( !(m_dwStyle&TLC_VGRID) )
	{
		m_dwStyle &= ~TLC_VGRID_EXT;
	}

	if( m_dwStyle&TLC_HGRID && m_dwStyle&TLC_VGRID )
	{
		if( m_dwStyle&TLC_HGRID_EXT && !(m_dwStyle&TLC_VGRID_EXT) )
			m_dwStyle &= ~( TLC_HGRID_EXT | TLC_VGRID_EXT );
		if( !(m_dwStyle&TLC_HGRID_EXT) && m_dwStyle&TLC_VGRID_EXT )
			m_dwStyle &= ~( TLC_HGRID_EXT | TLC_VGRID_EXT );
	}

	if( !(m_dwStyle&TLC_HEADER) )
	{
		m_dwStyle &= ~TLC_HEADDRAGDROP;
		m_dwStyle &= ~TLC_HEADFULLDRAG;
	}

	if( GetStyle()&TLC_HEADDRAGDROP )
		m_wndHeader.SetStyle( m_wndHeader.GetStyle()|TLHS_DRAGDROP );
	else
		m_wndHeader.SetStyle( m_wndHeader.GetStyle()&~TLHS_DRAGDROP );
	
	if( GetStyle()&TLC_HEADFULLDRAG )
		m_wndHeader.SetStyle( m_wndHeader.GetStyle()|TLHS_FULLDRAG );
	else
		m_wndHeader.SetStyle( m_wndHeader.GetStyle()&~TLHS_FULLDRAG );

	if( GetStyle()&TLC_HEADER )
		m_wndHeader.SetStyle( m_wndHeader.GetStyle()|TLHS_SHOWHEAD );
	else
		m_wndHeader.SetStyle( m_wndHeader.GetStyle()&~TLHS_SHOWHEAD );

	if( GetStyle()&TLC_HOTTRACK )
		m_wndHeader.SetStyle( m_wndHeader.GetStyle()|TLHS_HOTTRACK );
	else
		m_wndHeader.SetStyle( m_wndHeader.GetStyle()&~TLHS_HOTTRACK );

	Invalidate();

	return dwoldStyle;
}

DWORD CTreeListCtrl::GetStyle()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetStyle", __DATE__,__FILE__, __LINE__);
	// retrieve treelist control style
	return m_dwStyle;
}

void CTreeListCtrl::SetState( DWORD dwAddState, DWORD dwRemoveState )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetState", __DATE__,__FILE__, __LINE__);
	// set retrieve state of control
	m_dwState |= dwAddState;
	m_dwState &=~dwRemoveState;
	return; 
}

DWORD CTreeListCtrl::GetState( DWORD nStateMask )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetState", __DATE__,__FILE__, __LINE__);
	// retrieve state of control
	return m_dwState&nStateMask;
}

int CTreeListCtrl::GetSelectColumn()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetSelectColumn", __DATE__,__FILE__, __LINE__);
	// retrieve select column
	return m_iSelCol;
}

int CTreeListCtrl::GetColumnCount()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetColumnCount", __DATE__,__FILE__, __LINE__);
	// retrieve column count
	return GetColumns();
}

int CTreeListCtrl::GetVisibleCount( CTreeListItem* pParent )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetVisibleCount", __DATE__,__FILE__, __LINE__);
	// retrieve visible rows
	if( pParent == m_pRootItem )
		return pParent->m_nVisibleChild - 1;
	else
		return pParent->m_nVisibleChild;
}

void CTreeListCtrl::StatChildAdd( CTreeListItem* pItem, int nChild, int nVisibleChild )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::StatChildAdd", __DATE__,__FILE__, __LINE__);
	// stat. number of child & visible child on any level when add some new child
	ASSERT( pItem != NULL );
	
	if( pItem->m_pParent == NULL )
		return;
	
	CTreeListItem* pParent = pItem->m_pParent;

	pParent->m_nChild			+= nChild;

	if( pParent->GetState()&TLIS_EXPANDED )
	{
		pParent->m_nVisibleChild	+= nVisibleChild;
		StatChildAdd( pParent, nChild, nVisibleChild );
	}
	else
	{
		StatChildAdd( pParent, nChild, 0 );
	}
}

void CTreeListCtrl::StatChildDel( CTreeListItem* pItem, int nChild, int nVisibleChild )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::StatChildDel", __DATE__,__FILE__, __LINE__);
	// stat. number of child & visible child on any level when add some new child
	ASSERT( pItem != NULL );
	
	if( pItem->m_pParent == NULL )
		return;
	
	CTreeListItem* pParent = pItem->m_pParent;
	
	pParent->m_nChild			-= nChild;
	
	if( pParent->GetState()&TLIS_EXPANDED )
	{
		pParent->m_nVisibleChild	-= nVisibleChild;
		StatChildDel( pParent, nChild, nVisibleChild );
	}
	else
	{
		StatChildDel( pParent, nChild, 0 );
	}
}

void CTreeListCtrl::StatExpand( CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::StatExpand", __DATE__,__FILE__, __LINE__);
	// stat. number of child & visible child on any level when expand an item
	ASSERT( pItem != NULL );
	if( pItem->m_pChild == NULL )
		return;

	int nVisibleChild = 0;
	
	CTreeListItem* pChild = pItem->m_pChild;
	while( pChild != NULL )
	{
		nVisibleChild += pChild->m_nVisibleChild;
		pChild = pChild->m_pNext;
	}
	pItem->m_nVisibleChild += nVisibleChild;
	
	CTreeListItem* pParent = pItem->m_pParent;
	while( pParent != NULL )
	{
		pParent->m_nVisibleChild += nVisibleChild;
		pParent = pParent->m_pParent;
	}

	return;
}

void CTreeListCtrl::StatCollapse( CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::StatCollapse", __DATE__,__FILE__, __LINE__);
	// stat. number of child & visible child on any level when collapse an item
	ASSERT( pItem != NULL );
	if( pItem->m_pChild == NULL )
		return;

	int nVisibleChild = 0;

	CTreeListItem* pChild = pItem->m_pChild;
	while( pChild != NULL )
	{
		nVisibleChild += pChild->m_nVisibleChild;
		pChild = pChild->m_pNext;
	}
	pItem->m_nVisibleChild -= nVisibleChild;

	CTreeListItem* pParent = pItem->m_pParent;
	while( pParent != NULL )
	{
		pParent->m_nVisibleChild -= nVisibleChild;
		pParent = pParent->m_pParent;
	}

	return;
}

void CTreeListCtrl::SetItemCheck( CTreeListItem* pItem, BOOL bAutoCheck )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetItemCheck", __DATE__,__FILE__, __LINE__);
	// auto set check status ( include parent and child )
	LRESULT lResult;

	lResult = Notify( TLN_ITEMCHECK, pItem, 0 );
	if( lResult == -1 )
		return;

	if( bAutoCheck )
		pItem->SetCheck( !pItem->GetCheck() );

	if( m_dwStyle&TLC_NOAUTOCHECK )
		return;

	if( pItem->GetCheck() )
	{
		SetItemChildStatus( pItem, TLIS_CHECKED, TLIS_CHECKED_NULL );
		SetItemParentStatus( pItem, TLIS_CHECKEDMASK );
	}
	else
	{
		SetItemChildStatus( pItem, TLIS_CHECKED_NULL, TLIS_CHECKED );
		SetItemParentStatus( pItem, TLIS_CHECKEDMASK );
	}

	Invalidate();

	return;
}

void CTreeListCtrl::SetItemLock( CTreeListItem* pItem, BOOL bAutoLock )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetItemLock", __DATE__,__FILE__, __LINE__);
	// auto set lock status ( include parent and child )
	LRESULT lResult;

	lResult = Notify( TLN_ITEMLOCK, pItem, 0 );
	if( lResult == -1 )
		return;

	if( bAutoLock )
		pItem->SetLock( !pItem->GetLock() );

	if( m_dwStyle&TLC_NOAUTOLOCK )
		return;

	if( pItem->GetLock() )
	{
		SetItemChildStatus( pItem, TLIS_LOCKED, TLIS_LOCKED_NULL );
		SetItemParentStatus( pItem, TLIS_LOCKEDMASK );
	}
	else
	{
		SetItemChildStatus( pItem, TLIS_LOCKED_NULL, TLIS_LOCKED );
		SetItemParentStatus( pItem, TLIS_LOCKEDMASK );
	}

	Invalidate();

	return;
}

void CTreeListCtrl::SetItemParentStatus( CTreeListItem* pItem, DWORD dwMask )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetItemParentStatus", __DATE__,__FILE__, __LINE__);
	// set parent items status;
	CTreeListItem* pParent;
	pParent = pItem->m_pParent;

	DWORD dwShow;

	if( pParent == NULL )
		return;

	ASSERT( dwMask == TLIS_CHECKEDMASK || dwMask == TLIS_LOCKEDMASK );
	if( dwMask == TLIS_CHECKEDMASK )
		dwShow = TLIS_SHOWCHECKBOX;
	else
		dwShow = TLIS_SHOWLOCKBOX;
		

	DWORD dwAdd = 0;
	DWORD dwRemove = 0;
	CTreeListItem* pNext = pItem->m_pParent->m_pChild;
	if( pNext->GetState()&dwShow )
		dwAdd |= pNext->GetState();
	while( pNext != NULL )
	{
		if( pNext->GetState()&dwShow )
			dwAdd |= pNext->GetState();
		pNext = pNext->m_pNext;
	}
	dwAdd		 =  dwAdd;
	dwRemove	 = ~dwAdd;
	dwAdd		&=  dwMask;
	dwRemove	&=  dwMask;

	pParent->SetState( dwAdd, dwRemove );
	SetItemParentStatus( pParent, dwMask );

	return;
}


void CTreeListCtrl::SetItemChildStatus( CTreeListItem* pItem, DWORD dwAdd, DWORD dwRemove )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetItemChildStatus", __DATE__,__FILE__, __LINE__);
	// set child items status
	CTreeListItem* pChild;
	pChild = pItem->m_pChild;

	while( pChild != NULL )
	{
		if( pChild->m_pChild != NULL )
			SetItemChildStatus( pChild, dwAdd, dwRemove );

		pChild->SetState( dwAdd, dwRemove );
		pChild = pChild->m_pNext;
	}

	return;
}

CTreeListItem* CTreeListCtrl::GetFirstShowItem( int nShowItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetFirstShowItem", __DATE__,__FILE__, __LINE__);
	// retrieve first visible item after n visible items
	if( nShowItem < 0 )
		return NULL;

	if( nShowItem >= m_pRootItem->m_nVisibleChild )
		return NULL;
	
	CTreeListItem* pItem = m_pRootItem->m_pChild;
	while( nShowItem != 0 )
	{
		ASSERT( nShowItem > 0 );

		if( nShowItem < pItem->m_nVisibleChild )
		{
			nShowItem --;
			pItem = pItem->m_pChild;
		}
		else
		{
			nShowItem -= pItem->m_nVisibleChild;
			pItem = pItem->m_pNext;
		}
	}

	return pItem;
}

CTreeListItem* CTreeListCtrl::GetNextShowItem( CTreeListItem* pItem, BOOL bChild )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetNextShowItem", __DATE__,__FILE__, __LINE__);
	// retrieve next visible item
	ASSERT( pItem != NULL );

	if( pItem->GetState()&TLIS_EXPANDED )
		if( bChild && pItem->m_pChild != NULL )
			return pItem->m_pChild;

	if( pItem->m_pNext != NULL )
		return pItem->m_pNext;

	pItem = pItem->m_pParent;
	if( pItem != m_pRootItem )
		return GetNextShowItem( pItem, FALSE );

	return NULL;
}

BOOL CTreeListCtrl::OnEraseBkgnd(CDC* pDC) 
{
	IMPLEMENT_LOGGING("CTreeListCtrl::OnEraseBkgnd", __DATE__,__FILE__, __LINE__);
	// wm_erasebkgnd
	return TRUE;
}

void CTreeListCtrl::OnPaint() 
{
	IMPLEMENT_LOGGING("CTreeListCtrl::OnPaint", __DATE__,__FILE__, __LINE__);
	// wm_paint
	CPaintDC dc(this);
	CPalette* pPalette = NULL;

	if( dc.GetDeviceCaps(RASTERCAPS)&RC_PALETTE )
	{
		pPalette = dc.SelectPalette( &m_Palette, FALSE );
		dc.RealizePalette();
	}

	dc.ExcludeClipRect( m_rcHeader );

	if( GetState( TLS_OFFSCREENBUFFER ) )
	{
		CTreeListDC TreeListDC(&dc);
		DrawCtrl( &TreeListDC );
	}
	else
	{
		DrawCtrl( &dc );
	}

	if( pPalette != NULL && dc.GetDeviceCaps(RASTERCAPS)&RC_PALETTE )
	{
		dc.SelectPalette( pPalette, FALSE );
	}
}

void CTreeListCtrl::DrawCtrl( CDC* pDC )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DrawCtrl", __DATE__,__FILE__, __LINE__);
	// draw the control
	CRect rcClip;
	if( pDC->GetClipBox( &rcClip ) == ERROR )
		return;

	DrawBkgnd( pDC, rcClip );
	DrawItems( pDC, rcClip );
}

void CTreeListCtrl::DrawBkgnd( CDC* pDC, CRect rcClip )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DrawBkgnd", __DATE__,__FILE__, __LINE__);
	// draw background
	CRect rectClip;
	if (pDC->GetClipBox(&rectClip) == ERROR)
		return;

	if( CWnd::GetStyle()&WS_DISABLED )
	{
		if( m_dwStyle&TLC_BKGNDIMAGE )
		{
			DrawBkgndBmp( pDC, rcClip, &m_bmpBkDisable );
		}
		else if( m_dwStyle&TLC_BKGNDCOLOR )
		{
			pDC->FillSolidRect( rectClip, m_crBkDisable );
		}
		else
		{
			pDC->FillSolidRect( rectClip, m_crBkDisable_0 );
		}
	}
	else
	{
		if( m_dwStyle&TLC_BKGNDIMAGE )
		{
			DrawBkgndBmp( pDC, rcClip, &m_bmpBkBitmap );
		}
		else if( m_dwStyle&TLC_BKGNDCOLOR )
		{
			pDC->FillSolidRect( rectClip, m_crBkColor );
		}
		else
		{
			pDC->FillSolidRect( rectClip, m_crBkColor_0 );
		}
	}

	return;
}

void CTreeListCtrl::DrawBkgndBmp( CDC* pDC, CRect rcClip, CBitmap* pBkgnd )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DrawBkgndBmp", __DATE__,__FILE__, __LINE__);
	// draw bitmap background
	CRect rcClient;
	BITMAP bmBkgnd;
	
	GetClientRect( rcClient );
	pBkgnd->GetBitmap( &bmBkgnd );
	
	CDC dc;
	dc.CreateCompatibleDC( pDC );
	CBitmap* pBmp = dc.SelectObject( pBkgnd );

	pDC->IntersectClipRect( rcClip );

	int nPerfix = - ( GetScrollPos( SB_HORZ ) % bmBkgnd.bmWidth );
	for( int y = 0; y < rcClient.Height(); y += bmBkgnd.bmHeight )
		for( int x = nPerfix; x < rcClient.Width(); x += bmBkgnd.bmWidth )
			pDC->BitBlt( x, y, bmBkgnd.bmWidth, bmBkgnd.bmHeight, &dc, 0, 0, SRCCOPY);

	::ExtSelectClipRgn( pDC->m_hDC, DEFAULT_HRGN, RGN_COPY );

	dc.SelectObject( pBmp );
	dc.DeleteDC();

}

void CTreeListCtrl::DrawItems( CDC* pDC, CRect rcClip )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DrawItems", __DATE__,__FILE__, __LINE__);
	// draw items
	CTreeListItem* pShowItem;
	
	pShowItem = GetFirstVisibleItem();
	if( pShowItem == NULL )
		return;

	CPen Pen( PS_SOLID, 1, m_crGrid );
	CPen* pOldPen	= pDC->SelectObject( &Pen );
	CFont* pOldFont	= pDC->SelectObject( &m_Font );
	int nOldMode = pDC->SetBkMode( TRANSPARENT );

	CRect rcClient;
	GetClientRect( rcClient );

	CRect rcItem;
	rcItem.SetRect( 0, 0, GetWidth(), GetItemHeight() );
	rcItem.OffsetRect( m_rcTreeList.left, m_rcTreeList.top );
	rcItem.OffsetRect( -GetScrollPos( SB_HORZ ), 0 );

	int nFirstRow;
	int nShowRows;

	nFirstRow	=	GetScrollPos( SB_VERT );
	nShowRows	=	m_rcTreeList.Height() / GetItemHeight() + 1;

	pShowItem	=	GetFirstShowItem( nFirstRow );

	for( int iRow=nFirstRow; iRow<= nFirstRow + nShowRows && iRow < GetVisibleCount(); iRow++, pShowItem = GetNextShowItem( pShowItem ) )
	{
		ASSERT( pShowItem != NULL );

		COLORREF crBkColor;

		if( !(m_dwStyle&TLC_BKGNDIMAGE) )
		{
			if( !(CWnd::GetStyle()&WS_DISABLED) )
			{
				if( m_dwStyle&TLC_DOUBLECOLOR )
				{
					if( iRow%2 == 1 )
						crBkColor = m_crBkColor_1;
					else
						crBkColor = m_crBkColor_2;
				}
				else
				{
					crBkColor = m_crBkColor_0;
				}
			}
			else
			{
				if( m_dwStyle&TLC_DOUBLECOLOR )
				{
					if( iRow%2 == 1 )
						crBkColor = m_crBkDisable_1;
					else
						crBkColor = m_crBkDisable_2;
				}
				else
				{
					crBkColor = m_crBkDisable_0;
				}
			}
			pDC->FillSolidRect( rcItem, crBkColor );
		}

		DrawItem( pDC, rcItem, pShowItem ); 
		
		if( m_dwStyle&TLC_HGRID_FULL && rcItem.right<rcClient.right )
		{
			pDC->MoveTo( rcItem.right, rcItem.bottom - 1);
			pDC->LineTo( rcClient.right, rcItem.bottom - 1);
		}

		rcItem.OffsetRect( 0, m_nItemHeight );
	}

	if( GetVisibleCount() > 0 )
	{
		if( m_dwStyle&TLC_HGRID && !(m_dwStyle&TLC_HGRID_EXT) && !(m_dwStyle&TLC_VGRID_EXT) )
		{
			pDC->MoveTo( rcItem.left, rcItem.top - 1);
			pDC->LineTo( rcItem.right-1, rcItem.top - 1);
		}
	}

	for( int iGrid=GetVisibleCount(); iGrid<= nFirstRow + nShowRows; iGrid++ )
	{
		COLORREF crBkColor;

		if( !(m_dwStyle&TLC_BKGNDIMAGE) )
		{
			if( !(CWnd::GetStyle()&WS_DISABLED) )
			{
				if( m_dwStyle&TLC_DOUBLECOLOR )
				{
					if( iGrid%2 == 1 )
						crBkColor = m_crBkColor_1;
					else
						crBkColor = m_crBkColor_2;
				}
				else
				{
					crBkColor = m_crBkColor_0;
				}
			}
			else
			{
				if( m_dwStyle&TLC_DOUBLECOLOR )
				{
					if( iGrid%2 == 1 )
						crBkColor = m_crBkDisable_1;
					else
						crBkColor = m_crBkDisable_2;
				}
				else
				{
					crBkColor = m_crBkDisable_0;
				}
			}
			pDC->FillSolidRect( rcItem, crBkColor );
		}

		DrawGrid( pDC, rcItem );

		if( m_dwStyle&TLC_HGRID_EXT && m_dwStyle&TLC_HGRID_FULL && rcItem.right<rcClient.right )
		{
			pDC->MoveTo( rcItem.right, rcItem.bottom - 1 );
			pDC->LineTo( rcClient.right, rcItem.bottom - 1 );
		}

		rcItem.OffsetRect( 0, m_nItemHeight );
	}

	pDC->SetBkMode( nOldMode );
	pDC->SelectObject( pOldFont );
	pDC->SelectObject( pOldPen );
	return;
}

void CTreeListCtrl::DrawItem( CDC* pDC, CRect rcItem, CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DrawItem", __DATE__,__FILE__, __LINE__);
	// draw item background
	DrawItemBkgnd( pDC, rcItem, pItem );

	// draw all columns
	CTreeListColumnInfo* pColumnInfo;
	int nColPos = 0;
	for( int iShow = 0; iShow < m_arShows.GetSize(); iShow++, nColPos += pColumnInfo->m_nWidth )
	{
		int iCol;
		
		// do NOT draw zero column;
		iCol = m_arShows[iShow];
		pColumnInfo = (CTreeListColumnInfo*)m_arColumns[iCol];
		if( pColumnInfo->m_nWidth == 0 )
			continue;

		CRect rcColumn;
		rcColumn.SetRect( 0, rcItem.top, pColumnInfo->m_nWidth, rcItem.bottom );
		rcColumn.OffsetRect( nColPos, 0 );
		rcColumn.OffsetRect( -GetScrollPos( SB_HORZ ), 0 );

		if( iCol == 0 )
		{
			if ( m_dwStyle&TLC_TREELIST )
				DrawItemTree( pDC, rcColumn, pItem, iCol );
			else
				DrawItemMain( pDC, rcColumn, pItem, iCol );
		}
		else
		{
			DrawItemList( pDC, rcColumn, pItem, iCol );
		}

	}

	CRect rcFocus;
	if( pItem == m_pFocusItem )
	{
		CPen pen;
		pen.CreatePen( PS_SOLID, 1, (COLORREF)0x000000 );
		CPen* ppen = pDC->SelectObject( &pen );

		CRect rcFocus;
		rcFocus.SetRect( 0, rcItem.top, GetWidth(), rcItem.bottom );
		rcFocus.OffsetRect( -GetScrollPos( SB_HORZ ), 0 );

		if( GetStyle()&TLC_HGRID )
			rcFocus.DeflateRect( 0, 0, 0, 1 );

		if( GetStyle()&TLC_VGRID )
			rcFocus.DeflateRect( 0, 0, 1, 0 );

		COLORREF crBkgnd, croldBkgnd;
		if( CWnd::GetStyle()&WS_DISABLED )
		{
			if( m_dwStyle&TLC_BKGNDIMAGE )
				crBkgnd = m_crBkDisable;
			else if( m_dwStyle&TLC_BKGNDCOLOR )
				crBkgnd = m_crBkDisable;
			else
				crBkgnd = m_crBkDisable_0;
		}
		else
		{
			if( m_dwStyle&TLC_BKGNDIMAGE )
				crBkgnd = m_crBkColor;
			else if( m_dwStyle&TLC_BKGNDCOLOR )
				crBkgnd = m_crBkColor;
			else
				crBkgnd = m_crBkColor_0;
		}

		croldBkgnd = pDC->SetBkColor( crBkgnd );
		pDC->SetTextColor( m_crText );
		pDC->DrawFocusRect( rcFocus );
		pDC->SetBkColor( croldBkgnd );

		pDC->SelectObject( ppen );
	}

	return;
}

void CTreeListCtrl::DrawGrid( CDC* pDC, CRect rcItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DrawGrid", __DATE__,__FILE__, __LINE__);
	// draw all grids
	CTreeListColumnInfo* pColumnInfo;
	int nColPos = 0;
	for( int iShow = 0; iShow < m_arShows.GetSize(); iShow++, nColPos += pColumnInfo->m_nWidth )
	{
		int iCol;
		
		iCol = m_arShows[iShow];
		pColumnInfo = (CTreeListColumnInfo*)m_arColumns[iCol];
		if( pColumnInfo->m_nWidth == 0 )
			continue;

		CRect rcColumn;
		rcColumn.SetRect( 0, rcItem.top, pColumnInfo->m_nWidth, rcItem.bottom );
		rcColumn.OffsetRect( nColPos, 0 );
		rcColumn.OffsetRect( -GetScrollPos( SB_HORZ ), 0 );

		if( iCol == 0 )
		{
			// draw vertical grid line of tree
			if( m_dwStyle&TLC_HGRID_EXT && m_dwStyle&TLC_TGRID )
			{
				pDC->MoveTo( rcColumn.left, rcColumn.bottom - 1);
				pDC->LineTo( rcColumn.right-1, rcColumn.bottom - 1);
			}

			if( m_dwStyle&TLC_VGRID_EXT )
			{
				pDC->MoveTo( rcColumn.right-1, rcColumn.bottom - 1);
				pDC->LineTo( rcColumn.right-1, rcColumn.top - 1 );
			}
		}
		else
		{
			// draw vertical grid line and horizonal grid lin
			if( m_dwStyle&TLC_HGRID_EXT )
			{
				pDC->MoveTo( rcColumn.left, rcColumn.bottom - 1);
				pDC->LineTo( rcColumn.right, rcColumn.bottom - 1);
			}

			if( m_dwStyle&TLC_VGRID_EXT )
			{
				pDC->MoveTo( rcColumn.right-1, rcColumn.bottom - 1);
				pDC->LineTo( rcColumn.right-1, rcColumn.top - 1 );
			}
		}
	}
}

void CTreeListCtrl::DrawItemBkgnd( CDC* pDC, CRect rcItem, CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DrawItemBkgnd", __DATE__,__FILE__, __LINE__);
	// draw selected background
	CRect		rcBkgnd;
	COLORREF	crBkgnd;
	if( pItem == m_pTargetItem )
	{
		rcBkgnd.SetRect( 0, rcItem.top, GetWidth(), rcItem.bottom );
		rcBkgnd.OffsetRect( -GetScrollPos( SB_HORZ ), 0 );

		if( GetStyle()&TLC_HGRID )
			rcBkgnd.DeflateRect( 0, 0, 0, 1 );

		if( GetStyle()&TLC_VGRID )
			rcBkgnd.DeflateRect( 0, 0, 1, 0 );

		crBkgnd = m_crBkSelectedColumn;
		
		DrawItemExclude( pDC, rcItem, pItem );
		pDC->FillSolidRect( rcBkgnd, crBkgnd );
		::ExtSelectClipRgn( pDC->m_hDC, DEFAULT_HRGN, RGN_COPY );
	}
	else if( pItem->GetSelected() && GetStyle()&TLC_SHOWSELFULLROWS )
	{
		rcBkgnd.SetRect( 0, rcItem.top, GetWidth(), rcItem.bottom );
		rcBkgnd.OffsetRect( -GetScrollPos( SB_HORZ ), 0 );

		if( GetStyle()&TLC_HGRID )
			rcBkgnd.DeflateRect( 0, 0, 0, 1 );

		if( GetStyle()&TLC_VGRID )
			rcBkgnd.DeflateRect( 0, 0, 1, 0 );

		if( GetState( TLS_FOCUS ) )
		{
			if( !(m_dwStyle&TLC_MULTIPLESELECT) )
			{
				// single select
				crBkgnd = m_crBkSelectedRow;
			}
			else
			{
				// multiple select
				if( m_arSelects.GetSize() > 0 && pItem == (CTreeListItem*)m_arSelects[0] )
					crBkgnd = m_crBkSelectedRow;
				else
					crBkgnd = m_crBkSelected;
			}
			DrawItemExclude( pDC, rcItem, pItem );
			pDC->FillSolidRect( rcBkgnd, crBkgnd );
			::ExtSelectClipRgn( pDC->m_hDC, DEFAULT_HRGN, RGN_COPY );
		}
		else
		{
			if( m_dwStyle&TLC_SHOWSELALWAYS )
			{
				crBkgnd = m_crBkSelectedNoFocus;
				DrawItemExclude( pDC, rcItem, pItem );
				pDC->FillSolidRect( rcBkgnd, crBkgnd );
				::ExtSelectClipRgn( pDC->m_hDC, DEFAULT_HRGN, RGN_COPY );
			}
		}
	}
}

void CTreeListCtrl::DrawItemExclude( CDC* pDC, CRect rcItem, CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DrawItemExclude", __DATE__,__FILE__, __LINE__);
	// exclude rects of selected item
	CTreeListColumnInfo* pColumnInfo;
	int nColPos = 0;

	if( !pItem->GetSelected() || !(GetStyle()&TLC_SHOWSELFULLROWS) )
		return;

	for( int iShow = 0; iShow < m_arShows.GetSize(); iShow++, nColPos += pColumnInfo->m_nWidth )
	{
		int iCol;
		
		iCol = m_arShows[iShow];
		pColumnInfo = (CTreeListColumnInfo*)m_arColumns[iCol];

		CRect rcColumn;
		rcColumn.SetRect( 0, rcItem.top, pColumnInfo->m_nWidth, rcItem.bottom );
		rcColumn.OffsetRect( nColPos, 0 );
		rcColumn.OffsetRect( -GetScrollPos( SB_HORZ ), 0 );

		if( iCol == 0 )
		{
			CRect rcExclude;
			int nPerfix = TLL_BORDER;
			if ( m_dwStyle&TLC_TREELIST )
			{
				nPerfix += TLL_WIDTH * ( pItem->m_nLevel - 1 );

				if( m_dwStyle&(TLC_TREELINE|TLC_BUTTON) )
					nPerfix += TLL_WIDTH;
			}

			if( m_dwStyle&TLC_CHECKBOX )
			{
				if( pItem->GetState()&TLIS_SHOWCHECKBOX )
				{
					CRect rcCheckBox;
					rcCheckBox.SetRect( 0, 0, TLL_WIDTH, TLL_WIDTH );
					rcCheckBox.OffsetRect( rcColumn.left + nPerfix, rcColumn.top + ( rcColumn.Height() - TLL_HEIGHT)/2 );
					rcExclude = rcColumn & rcCheckBox;

					if( !rcExclude.IsRectEmpty() )
						pDC->ExcludeClipRect( rcExclude );

					nPerfix += TLL_HEIGHT;
				}
			}

			if( m_dwStyle&TLC_LOCKBOX )
			{
				if( pItem->GetState()&TLIS_SHOWLOCKBOX )
				{
					CRect rcLockBox;
					rcLockBox.SetRect( 0, 0, TLL_WIDTH / 2, TLL_WIDTH );
					rcLockBox.OffsetRect( rcColumn.left + nPerfix, rcColumn.top + ( rcColumn.Height() - TLL_HEIGHT)/2 );
					rcExclude = rcColumn & rcLockBox;

					if( !rcExclude.IsRectEmpty() )
						pDC->ExcludeClipRect( rcExclude );

					nPerfix += TLL_HEIGHT / 2;
				}
			}
		
			if( m_dwStyle&TLC_IMAGE )
			{
				if( pItem->GetState()&TLIS_SHOWTREEIMAGE )
				{
					CRect rcImage;
					rcImage.SetRect( 0, 0, TLL_WIDTH, TLL_WIDTH );
					rcImage.OffsetRect( rcColumn.left + nPerfix, rcColumn.top + ( rcColumn.Height() - TLL_HEIGHT)/2 );
					rcExclude = rcColumn & rcImage;

					if( !rcExclude.IsRectEmpty() )
						pDC->ExcludeClipRect( rcExclude );

					nPerfix += TLL_HEIGHT;
				}
			}

			break;
		}
	}

}

void CTreeListCtrl::DrawItemTree( CDC* pDC, CRect rcColumn, CTreeListItem* pItem, int iCol )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DrawItemTree", __DATE__,__FILE__, __LINE__);
	// draw tree column of treelist control
	CTreeListColumnInfo* pColumnInfo;
	pColumnInfo = (CTreeListColumnInfo*)m_arColumns[iCol];

	CRect rcGraph;
	rcGraph = rcColumn;
	rcGraph.DeflateRect( TLL_BORDER, 0, TLL_BORDER+1, 0 );
	if( rcGraph.Width() <= 0 )
		return;
	CRect rcTree;
	rcTree = rcGraph;
	rcTree.right = rcTree.left + TLL_WIDTH;

	int nPerfix = TLL_BORDER;
	// draw prefix vertical line
	for( int iLevel=1; iLevel<pItem->m_nLevel; iLevel++, nPerfix += TLL_WIDTH )
	{
		CTreeListItem* pCheckItem = pItem;
		while( pCheckItem->m_nLevel != iLevel )
		{
			pCheckItem = pCheckItem->m_pParent;
		}

		rcTree.left = rcColumn.left + nPerfix;
		rcTree.right = rcTree.left + TLL_WIDTH;

		if( pCheckItem->m_pNext != NULL )
		{
			if( iLevel != 1 || m_dwStyle&TLC_ROOTLINE )
			{
				DrawItemTreeLine( pDC, rcGraph, rcTree, TLL_TOP | TLL_BOTTOM );				// 
			}
		}
	}

	// draw cross line
	if( m_dwStyle&(TLC_TREELINE|TLC_BUTTON) )
	{
		rcTree.left = rcColumn.left + nPerfix;
		rcTree.right = rcTree.left + TLL_WIDTH;
		if( pItem->m_nLevel == 1 )
		{
			// child item of root item
			if( m_dwStyle&TLC_ROOTLINE )
			{
				if( pItem->m_pPrev == NULL && pItem->m_pNext == NULL )
					DrawItemTreeLine( pDC, rcGraph, rcTree, TLL_RIGHT );						//  ( right part )
				else if( pItem->m_pPrev == NULL )
					DrawItemTreeLine( pDC, rcGraph, rcTree, TLL_RIGHT | TLL_BOTTOM );			// 
				else if( pItem->m_pNext == NULL )
					DrawItemTreeLine( pDC, rcGraph, rcTree, TLL_RIGHT | TLL_TOP );				// 
				else
					DrawItemTreeLine( pDC, rcGraph, rcTree, TLL_RIGHT | TLL_TOP | TLL_BOTTOM );	// 
			}
			else
			{
				DrawItemTreeLine( pDC, rcGraph, rcTree, TLL_RIGHT );							//  ( right part )
			}
		}
		else
		{
			// child item of other items ( not root item )
			if( pItem->m_pNext == NULL )
				DrawItemTreeLine( pDC, rcGraph, rcTree, TLL_RIGHT | TLL_TOP );					// 
			else
				DrawItemTreeLine( pDC, rcGraph, rcTree, TLL_RIGHT | TLL_TOP | TLL_BOTTOM );		// 
		}
		
		// draw button of child item of root item
		if( m_dwStyle&TLC_BUTTON )
		{
			if( pItem->GetState()&TLIS_EXPANDEDONCE )
			{
				if( pItem->m_pChild != NULL )
				{
					if( pItem->GetState()&TLIS_EXPANDED )
						DrawItemTreeButton( pDC, rcGraph, rcTree, TLB_MINUS );
					else
						DrawItemTreeButton( pDC, rcGraph, rcTree, TLB_PLUS );
				}				
			}
			else
			{
				DrawItemTreeButton( pDC, rcGraph, rcTree, TLB_UNKNOW );
			}
		}
		nPerfix += TLL_WIDTH;
	}

	// draw check box
	if( m_dwStyle&TLC_CHECKBOX )
	{
		if( pItem->GetState()&TLIS_SHOWCHECKBOX )
		{
			CRect rcCheckBox;
			rcCheckBox = rcGraph;
			rcCheckBox.left = rcColumn.left + nPerfix;
			rcCheckBox.right = rcCheckBox.left + TLL_WIDTH;

			POINT pt;
			pt.x = rcCheckBox.left;
			pt.y = rcCheckBox.top + (rcCheckBox.Height() - TLL_HEIGHT)/2;

			pDC->IntersectClipRect( rcGraph );
			pDC->IntersectClipRect( rcCheckBox );
			
			UINT nStyle = ILD_TRANSPARENT;
			if( pItem == m_pTargetItem || pItem == m_pHoverItem )
				nStyle |= ILD_FOCUS;
			m_imgCheck.Draw( pDC, pItem->GetCheck(), pt, nStyle );
			::ExtSelectClipRgn( pDC->m_hDC, DEFAULT_HRGN, RGN_COPY );

			nPerfix += TLL_WIDTH;
		}
	}

	// draw lock box
	if( m_dwStyle&TLC_LOCKBOX )
	{
		if( pItem->GetState()&TLIS_SHOWLOCKBOX )
		{
			CRect rcLockBox;
			rcLockBox = rcGraph;
			rcLockBox.left = rcColumn.left + nPerfix;
			rcLockBox.right = rcLockBox.left + TLL_WIDTH/2;

			POINT pt;
			pt.x = rcLockBox.left;
			pt.y = rcLockBox.top + (rcLockBox.Height() - TLL_HEIGHT)/2;

			pDC->IntersectClipRect( rcGraph );
			pDC->IntersectClipRect( rcLockBox );
			UINT nStyle = ILD_TRANSPARENT;
			if( pItem == m_pTargetItem || pItem == m_pHoverItem )
				nStyle |= ILD_FOCUS;
			m_imgLock.Draw( pDC, pItem->GetLock(), pt, nStyle );
			::ExtSelectClipRgn( pDC->m_hDC, DEFAULT_HRGN, RGN_COPY );
			
			nPerfix += TLL_WIDTH/2;
		}
	}

	// draw image box
	if( m_dwStyle&TLC_IMAGE )
	{
		if( pItem->GetState()&TLIS_SHOWTREEIMAGE )
		{
			CRect rcImage;
			rcImage = rcGraph;
			rcImage.left = rcColumn.left + nPerfix;
			rcImage.right = rcImage.left + TLL_WIDTH;

			POINT pt;
			pt.x = rcImage.left;
			pt.y = rcImage.top + (rcImage.Height() - TLL_HEIGHT)/2;

			pDC->IntersectClipRect( rcGraph );
			pDC->IntersectClipRect( rcImage );

			UINT nStyle = ILD_TRANSPARENT;
			if( pItem == m_pTargetItem || pItem == m_pHoverItem )
				nStyle |= ILD_FOCUS;

			if( pItem->GetState()&TLIS_EXPANDED && pItem->m_pChild != NULL )
			{
				if( m_arSelects.GetSize() > 0 && pItem == (CTreeListItem*)m_arSelects[0] )
					m_pImageList->Draw( pDC, pItem->m_nExpandSelectedImage, pt, nStyle );
				else
					m_pImageList->Draw( pDC, pItem->m_nSelectedImage, pt, nStyle );
			}
			else
			{
				if( m_arSelects.GetSize() > 0 && pItem == (CTreeListItem*)m_arSelects[0] )
					m_pImageList->Draw( pDC, pItem->m_nExpandImage, pt, nStyle );
				else
					m_pImageList->Draw( pDC, pItem->m_nImage, pt, nStyle );
			}

			::ExtSelectClipRgn( pDC->m_hDC, DEFAULT_HRGN, RGN_COPY );

			nPerfix += TLL_WIDTH;
		}
	}

	// add gaps between text and image 
	if( m_dwStyle&TLC_LOCKBOX || m_dwStyle&TLC_IMAGE )
		nPerfix += 2;

	// draw item text
	CRect rcBkgnd;
	rcBkgnd = rcColumn;
	rcBkgnd.left = rcColumn.left + nPerfix;

	DrawItemTreeText( pDC, rcBkgnd, pItem, iCol );

	// draw vertical grid line of tree
	if( m_dwStyle&TLC_TGRID )
	{
		pDC->MoveTo( rcColumn.left, rcColumn.bottom - 1);
		pDC->LineTo( rcColumn.right-1, rcColumn.bottom - 1);
	}

	if( m_dwStyle&TLC_VGRID )
	{
		pDC->MoveTo( rcColumn.right-1, rcColumn.bottom - 1);
		pDC->LineTo( rcColumn.right-1, rcColumn.top - 1 );
	}

	return;
}

void CTreeListCtrl::DrawItemMain( CDC* pDC, CRect rcColumn, CTreeListItem* pItem, int iCol )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DrawItemMain", __DATE__,__FILE__, __LINE__);
	// draw main column of list
	CTreeListColumnInfo* pColumnInfo;
	pColumnInfo = (CTreeListColumnInfo*)m_arColumns[iCol];

	CRect rcGraph;
	rcGraph = rcColumn;
	rcGraph.DeflateRect( TLL_BORDER, 0, TLL_BORDER+1, 0 );
	if( rcGraph.Width() <= 0 )
		return;
	
	CRect rcTree;
	rcTree = rcGraph;
	rcTree.right = rcTree.left + TLL_WIDTH;

	int nPerfix = TLL_BORDER;

	// draw check box
	if( m_dwStyle&TLC_CHECKBOX )
	{
		if( pItem->GetState()&TLIS_SHOWCHECKBOX )
		{
			CRect rcCheckBox;
			rcCheckBox = rcGraph;
			rcCheckBox.left = rcColumn.left + nPerfix;
			rcCheckBox.right = rcCheckBox.left + TLL_WIDTH;

			POINT pt;
			pt.x = rcCheckBox.left;
			pt.y = rcCheckBox.top + (rcCheckBox.Height() - TLL_HEIGHT)/2;

			pDC->IntersectClipRect( rcGraph );
			pDC->IntersectClipRect( rcCheckBox );
			UINT nStyle = ILD_TRANSPARENT;
			if( pItem == m_pTargetItem || pItem == m_pHoverItem )
				nStyle |= ILD_FOCUS;
			m_imgCheck.Draw( pDC, pItem->GetCheck(), pt, nStyle );
			::ExtSelectClipRgn( pDC->m_hDC, DEFAULT_HRGN, RGN_COPY );

			nPerfix += TLL_WIDTH;
		}
	}

	// draw lock box
	if( m_dwStyle&TLC_LOCKBOX )
	{
		if( pItem->GetState()&TLIS_SHOWLOCKBOX )
		{
			CRect rcLockBox;
			rcLockBox = rcGraph;
			rcLockBox.left = rcColumn.left + nPerfix;
			rcLockBox.right = rcLockBox.left + TLL_WIDTH/2;

			POINT pt;
			pt.x = rcLockBox.left;
			pt.y = rcLockBox.top + (rcLockBox.Height() - TLL_HEIGHT)/2;

			pDC->IntersectClipRect( rcGraph );
			pDC->IntersectClipRect( rcLockBox );
			UINT nStyle = ILD_TRANSPARENT;
			if( pItem == m_pTargetItem || pItem == m_pHoverItem )
				nStyle |= ILD_FOCUS;
			m_imgLock.Draw( pDC, pItem->GetLock(), pt, nStyle );
			::ExtSelectClipRgn( pDC->m_hDC, DEFAULT_HRGN, RGN_COPY );
			
			nPerfix += TLL_WIDTH/2;
		}
	}

	// draw image box
	if( m_dwStyle&TLC_IMAGE )
	{
		if( pItem->GetState()&TLIS_SHOWTREEIMAGE )
		{
			CRect rcImage;
			rcImage = rcGraph;
			rcImage.left = rcColumn.left + nPerfix;
			rcImage.right = rcImage.left + TLL_WIDTH;

			POINT pt;
			pt.x = rcImage.left;
			pt.y = rcImage.top + (rcImage.Height() - TLL_HEIGHT)/2;

			pDC->IntersectClipRect( rcGraph );
			pDC->IntersectClipRect( rcImage );

			UINT nStyle = ILD_TRANSPARENT;
			if( pItem == m_pTargetItem || pItem == m_pHoverItem )
				nStyle |= ILD_FOCUS;
			if( m_arSelects.GetSize() > 0 && pItem == (CTreeListItem*)m_arSelects[0] )
				m_pImageList->Draw( pDC, pItem->m_nExpandImage, pt, nStyle );
			else
				m_pImageList->Draw( pDC, pItem->m_nImage, pt, nStyle );

			::ExtSelectClipRgn( pDC->m_hDC, DEFAULT_HRGN, RGN_COPY );

			nPerfix += TLL_WIDTH;
		}
	}

	// add gaps between text and image 
	if( m_dwStyle&TLC_LOCKBOX || m_dwStyle&TLC_IMAGE )
		nPerfix += 2;

	// draw item text
	CRect rcBkgnd;
	rcBkgnd = rcColumn;
	rcBkgnd.left = rcColumn.left + nPerfix;

	DrawItemTreeText( pDC, rcBkgnd, pItem, iCol );

	// draw vertical grid line of tree
	if( m_dwStyle&TLC_TGRID )
	{
		pDC->MoveTo( rcColumn.left, rcColumn.bottom - 1);
		pDC->LineTo( rcColumn.right-1, rcColumn.bottom - 1);
	}

	if( m_dwStyle&TLC_VGRID )
	{
		pDC->MoveTo( rcColumn.right-1, rcColumn.bottom - 1);
		pDC->LineTo( rcColumn.right-1, rcColumn.top - 1 );
	}
}

void CTreeListCtrl::DrawItemList( CDC* pDC, CRect rcColumn, CTreeListItem* pItem, int iCol )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DrawItemList", __DATE__,__FILE__, __LINE__);
	// draw list
	CTreeListColumnInfo* pColumnInfo;
	pColumnInfo = (CTreeListColumnInfo*)m_arColumns[iCol];

	CRect rcBkgnd;
	rcBkgnd = rcColumn;
	rcBkgnd.left = rcColumn.left;

	DrawItemListText( pDC, rcBkgnd, pItem, iCol );

	// draw vertical grid line and horizonal grid lin
	if( m_dwStyle&TLC_HGRID )
	{
		pDC->MoveTo( rcColumn.left, rcColumn.bottom - 1);
		pDC->LineTo( rcColumn.right, rcColumn.bottom - 1);
	}

	if( m_dwStyle&TLC_VGRID )
	{
		pDC->MoveTo( rcColumn.right-1, rcColumn.bottom - 1);
		pDC->LineTo( rcColumn.right-1, rcColumn.top - 1 );
	}
}

void CTreeListCtrl::DrawItemTreeLine( CDC* pDC, CRect rcColumn, CRect rcTree, DWORD dwFormat )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DrawItemTreeLine", __DATE__,__FILE__, __LINE__);
	// draw line of tree
	if( rcColumn.Width() == 0 || rcColumn.Height() == 0 )
		return;

	if( !(m_dwStyle&TLC_TREELINE) )
		return;

	ASSERT( rcTree.Width()	% 2 == 0 );
	ASSERT( rcTree.Height() % 2 == 0 );

	// clip column
	pDC->IntersectClipRect( rcColumn );
	pDC->IntersectClipRect( rcTree );

	CPoint ptCenter;
	ptCenter.x = rcTree.left + rcTree.Width() / 2 - 1;
	ptCenter.y = rcTree.top  + rcTree.Height() / 2 - 1;

	// draw center-left
	if( dwFormat&TLL_LEFT )
	{
		for( int ix = ptCenter.x; ix >= rcTree.left; ix -= 2 )
			pDC->SetPixelV( ix, ptCenter.y, m_crTreeLine );
	}

	// draw center-right
	if( dwFormat&TLL_RIGHT )
	{
		for( int ix = ptCenter.x; ix <= rcTree.right; ix += 2 )
			pDC->SetPixelV( ix, ptCenter.y, m_crTreeLine );
	}

	// draw center-top
	if( dwFormat&TLL_TOP )
	{
		for( int iy = ptCenter.y; iy >= rcTree.top; iy -= 2 )
			pDC->SetPixelV( ptCenter.x, iy, m_crTreeLine );
	}

	// draw center-bottom
	if( dwFormat&TLL_BOTTOM )
	{
		for( int iy = ptCenter.y; iy <= rcTree.bottom; iy += 2 )
			pDC->SetPixelV( ptCenter.x, iy, m_crTreeLine );
	}

	::ExtSelectClipRgn( pDC->m_hDC, DEFAULT_HRGN, RGN_COPY );
}

void CTreeListCtrl::DrawItemTreeButton( CDC* pDC, CRect rcColumn, CRect rcTree, DWORD dwFormat )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DrawItemTreeButton", __DATE__,__FILE__, __LINE__);
	// draw tree button [+][-][?]
	if( rcColumn.Width() == 0 || rcColumn.Height() == 0 )
		return;

	ASSERT( rcTree.Width()  % 2 == 0 );
	ASSERT( rcTree.Height() % 2 == 0 );

	// clip column
	pDC->IntersectClipRect( rcColumn );
	pDC->IntersectClipRect( rcTree );

	// button center
	CPoint ptCenter;
	ptCenter.x = rcTree.left + rcTree.Width() / 2 - 1;
	ptCenter.y = rcTree.top	+ rcTree.Height() / 2 - 1;

	CRect rcButton;
	rcButton.SetRect( -TLL_BUTTON, -TLL_BUTTON, TLL_BUTTON+1, TLL_BUTTON+1 );
	rcButton.OffsetRect( ptCenter );

	CBrush brButton;
	brButton.CreateSolidBrush( m_crTreeLine );

	if ( !(CWnd::GetStyle()&WS_DISABLED) )
		pDC->FillSolidRect( rcButton, 0x00FFFFFF );
	else
		pDC->FillSolidRect( rcButton, 0x00D0D0D0 );
	pDC->FrameRect( rcButton, &brButton );

	rcButton.DeflateRect( 2, 2, 2, 2 );
	// draw unknow button [+] in grey
	if( dwFormat&TLB_UNKNOW )
	{
		CPen Pen, *pOldPen;
		Pen.CreatePen( PS_SOLID, 1, m_crTreeLine );
		pOldPen = pDC->SelectObject( &Pen );
		pDC->MoveTo( rcButton.left, ptCenter.y );
		pDC->LineTo( rcButton.right, ptCenter.y );
		pDC->MoveTo( ptCenter.x, rcButton.top );
		pDC->LineTo( ptCenter.x, rcButton.bottom );
		pDC->SelectObject( pOldPen );
	}

	// draw plus button [+]
	if( dwFormat&TLB_PLUS )
	{
		CPen Pen, *pOldPen;
		Pen.CreatePen( PS_SOLID, 1, (COLORREF)0x000000 );
		pOldPen = pDC->SelectObject( &Pen );
		pDC->MoveTo( rcButton.left, ptCenter.y );
		pDC->LineTo( rcButton.right, ptCenter.y );
		pDC->MoveTo( ptCenter.x, rcButton.top );
		pDC->LineTo( ptCenter.x, rcButton.bottom );
		pDC->SelectObject( pOldPen );
	}

	// draw minus button [-]
	if( dwFormat&TLB_MINUS )
	{
		CPen Pen, *pOldPen;
		Pen.CreatePen( PS_SOLID, 1, (COLORREF)0x000000 );
		pOldPen = pDC->SelectObject( &Pen );
		pDC->MoveTo( rcButton.left, ptCenter.y );
		pDC->LineTo( rcButton.right, ptCenter.y );
		pDC->SelectObject( pOldPen );
	}

	::ExtSelectClipRgn( pDC->m_hDC, DEFAULT_HRGN, RGN_COPY );
}

void CTreeListCtrl::DrawItemTreeText( CDC* pDC, CRect rcBkgnd, CTreeListItem* pItem, int iCol )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DrawItemTreeText", __DATE__,__FILE__, __LINE__);
	// draw text on tree
	CRect rcText;
	rcText = rcBkgnd;
	rcText.DeflateRect( 2, 0, 3, 0 );
	rcBkgnd.bottom = rcBkgnd.bottom - 1;
	DrawItemText( pDC, rcBkgnd, rcText, pItem, iCol );
}

void CTreeListCtrl::DrawItemListText( CDC* pDC, CRect rcBkgnd, CTreeListItem* pItem, int iCol )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DrawItemListText", __DATE__,__FILE__, __LINE__);
	// draw text on list
	CRect rcText;
	rcText = rcBkgnd;
	rcText.DeflateRect( 2, 0, 3, 0 );
	rcBkgnd.bottom = rcBkgnd.bottom - 1;
	DrawItemText( pDC, rcBkgnd, rcText, pItem, iCol );
}

void CTreeListCtrl::DrawItemText( CDC* pDC, CRect rcBkgnd, CRect rcText, CTreeListItem* pItem, int iCol )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DrawItemText", __DATE__,__FILE__, __LINE__);
	COLORREF crBkgnd;
	COLORREF crText = m_crText;

	CTreeListColumnInfo* pColumnInfo;
	pColumnInfo = (CTreeListColumnInfo*)m_arColumns[iCol];

	// draw selected background
	if( rcBkgnd.Width() <= 0 || rcBkgnd.Height() <= 0 )
		return;

	if( pItem == m_pTargetItem )
	{
		crBkgnd = m_crBkSelectedColumn;
		crText	= m_crTextSelectedColumn;
	}
	else if( pItem->GetSelected() )
	{
		if( GetState( TLS_FOCUS ) )
		{
			if( !(m_dwStyle&TLC_MULTIPLESELECT) )
			{
				// single select
				if( iCol == m_iSelCol && m_dwStyle&TLC_SHOWSELACTIVE )
				{
					crBkgnd = m_crBkSelectedColumn;
					crText	= m_crTextSelectedColumn;
				}
				else
				{
					crBkgnd = m_crBkSelectedRow;
					crText	= m_crTextSelectedRow;
				}
			}
			else
			{
				// multiple select
				if( m_arSelects.GetSize() > 0 && pItem == (CTreeListItem*)m_arSelects[0] )
				{
					if( iCol == m_iSelCol && m_dwStyle&TLC_SHOWSELACTIVE )
					{
						crBkgnd = m_crBkSelectedColumn;
						crText	= m_crTextSelectedColumn;
					}
					else
					{
						crBkgnd = m_crBkSelectedRow;
						crText	= m_crTextSelectedRow;
					}
				}
				else
				{
					crBkgnd = m_crBkSelected;
					crText	= m_crTextSelected;
				}
			}
		
			pDC->FillSolidRect( rcBkgnd, crBkgnd );
		}
		else
		{
			if( m_dwStyle&TLC_SHOWSELALWAYS )
			{
				crBkgnd = m_crBkSelectedNoFocus;
				crText = m_crTextSelectedNoFocus;

				if( !(GetStyle()&TLC_SHOWSELFULLROWS) )
					pDC->FillSolidRect( rcBkgnd, crBkgnd );
			}
		}
	}

	// draw text
	if( rcText.Width() <= 0 || rcText.Height() <= 0 )
		return;

	COLORREF crOldText = pDC->SetTextColor( crText );

	UINT dwFormat = pColumnInfo->GetTextFormat();

	LOGFONT lf;
	GetParent()->GetFont()->GetLogFont(&lf);

	if (pItem->GetState() & TLIS_HIGHLIGHTED)
	{
		lf.lfWeight = FW_BOLD;
	} else {
		lf.lfWeight = FW_NORMAL;
	}

	CFont font;
	font.CreateFontIndirect(&lf);

	pDC->SelectObject(&font);

	pDC->DrawText( pItem->GetText(iCol), rcText, dwFormat );

	pDC->SetTextColor( crOldText );

	font.DeleteObject();
}

void CTreeListCtrl::OnEnable(BOOL bEnable) 
{
	IMPLEMENT_LOGGING("CTreeListCtrl::OnEnable", __DATE__,__FILE__, __LINE__);
	// enable or disable control
	CWnd::OnEnable(bEnable);
	
	m_wndHeader.EnableWindow( bEnable );
	m_wndTip.EnableWindow( bEnable );

	Invalidate();

	return;
}

void CTreeListCtrl::OnNcCalcSize(BOOL bCalcValidRects, NCCALCSIZE_PARAMS FAR* lpncsp) 
{
	IMPLEMENT_LOGGING("CTreeListCtrl::OnNcCalcSize", __DATE__,__FILE__, __LINE__);
	// calculate size of not client area
	if( !(CWnd::GetExStyle()&WS_EX_CLIENTEDGE) )
	{
		LPRECT pRECT = &lpncsp->rgrc[0];
		::InflateRect( pRECT, -1, -1 );
	}

	Default();
	return;
}

void CTreeListCtrl::OnNcPaint() 
{
	IMPLEMENT_LOGGING("CTreeListCtrl::OnNcPaint", __DATE__,__FILE__, __LINE__);
	// draw 3d frame rect
	Default();

	if( !(CWnd::GetExStyle()&WS_EX_CLIENTEDGE) )
	{
		CWindowDC dc(this);
		CRect rcBorder;
		GetWindowRect( &rcBorder );
		ScreenToClient( &rcBorder );
		rcBorder.OffsetRect( -rcBorder.left, -rcBorder.top );
		dc.Draw3dRect( &rcBorder, GetSysColor( COLOR_3DDKSHADOW ), GetSysColor( COLOR_3DHILIGHT ) );
	}

	return;
}

void CTreeListCtrl::OnSize(UINT nType, int cx, int cy) 
{
	IMPLEMENT_LOGGING("CTreeListCtrl::OnSize", __DATE__,__FILE__, __LINE__);
	// resize tree list control
	CWnd::OnSize(nType, cx, cy);
	
	Layout();

	return;
}

BOOL CTreeListCtrl::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
	IMPLEMENT_LOGGING("CTreeListCtrl::OnMouseWheel", __DATE__,__FILE__, __LINE__);
	// translate mouse wheel message
	int nPos = zDelta / 120;

	if( IsWindow(m_wndTip.GetSafeHwnd()) && m_wndTip.IsWindowVisible() )
		m_wndTip.Hide();

	if( nPos > 0 )
	{
		PostMessage( WM_VSCROLL, (WPARAM)SB_PAGEUP );
	}
	else
	{
		PostMessage( WM_VSCROLL, (WPARAM)SB_PAGEDOWN );
	}
	return TRUE;
}


BOOL CTreeListCtrl::InsertColumn( LPCTSTR lpszColumnHeading, DWORD dwFormat, int nWidth, int iImage, int nMin, int nMax)
{
	IMPLEMENT_LOGGING("CTreeListCtrl::InsertColumn", __DATE__,__FILE__, __LINE__);
	// insert a column
	ASSERT( m_pRootItem->m_pChild == NULL );

	CTreeListColumnInfo* pColumnInfo;
	pColumnInfo = new CTreeListColumnInfo;

	pColumnInfo->m_strCaption	= lpszColumnHeading;
	pColumnInfo->m_dwFormat		= dwFormat;
	pColumnInfo->m_nWidth		= nWidth;
	pColumnInfo->m_iImage		= iImage;
	pColumnInfo->m_nMin			= nMin;
	pColumnInfo->m_nMax			= nMax;

	int nCol = m_arColumns.Add( (void*)pColumnInfo );
	
	if( (pColumnInfo->m_dwFormat&TLF_HIDDEN) == 0 )
		m_arShows.Add( nCol );

	if( pColumnInfo->m_dwFormat&TLF_CAPTION_SORT )
		if( pColumnInfo->m_dwFormat&TLF_SORT_MASK )
			m_arSorts.Add( nCol );

	Layout();
	Invalidate();

	return TRUE;
}

BOOL CTreeListCtrl::DeleteColumn( int iCol )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DeleteColumn", __DATE__,__FILE__, __LINE__);
	// delete a column
	ASSERT( m_pRootItem->m_pChild == NULL );

	if( iCol < 0  || m_arColumns.GetSize() <= iCol )
		return FALSE;

	CTreeListColumnInfo* pColumnInfo;
	pColumnInfo = (CTreeListColumnInfo*)m_arColumns[iCol];
	m_arColumns.RemoveAt( iCol );

	for( int iShow=0; iShow<m_arShows.GetSize(); iShow++ )
	{
		if( m_arShows[iShow] == iCol )
		{
			m_arShows.RemoveAt( iCol );
		}

		if( iShow<m_arShows.GetSize() )
		{
			if( m_arShows[iShow] > iCol )
				m_arShows[iShow] -= 1;
		}
	}

	for( int iSort=0; iSort<m_arSorts.GetSize(); iSort++ )
	{
		if( m_arSorts[iSort] == iCol )
			m_arSorts.RemoveAt( iCol );

		if( iSort<m_arSorts.GetSize() )
			if( m_arSorts[iSort] > iCol )
				m_arSorts[iSort] -= 1;
	}

	Layout();
	Invalidate();

	return TRUE;
}

int CTreeListCtrl::InsertItem( int nItem, LPCTSTR lpszItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::InsertItem", __DATE__,__FILE__, __LINE__);
	// insert an new item ( list )
	ASSERT( !(m_dwStyle&TLC_TREELIST) );
	CTreeListItem* pItem;

	if( m_pRootItem->m_pChild == NULL )
	{
		InsertItemNew( lpszItem, m_pRootItem );
		return 0;
	}

	if( nItem == 0 )
	{
		InsertItemFirst( lpszItem, m_pRootItem );
		return 0;
	}

	if( nItem >= GetCount() )
	{
		pItem = InsertItem( lpszItem, TLI_ROOT, TLI_LAST );
		return GetTreeListItem( pItem );
	}

	pItem = GetTreeListItem( nItem );
	pItem = InsertItem( lpszItem, TLI_ROOT, pItem );
	return GetTreeListItem( pItem );
}

BOOL CTreeListCtrl::DeleteItem( int nItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DeleteItem", __DATE__,__FILE__, __LINE__);
	// delete an item ( list )
	ASSERT( !(m_dwStyle&TLC_TREELIST) );
	CTreeListItem* pItem;

	pItem = GetTreeListItem( nItem );

	if( pItem == NULL )
		return FALSE;

	return DeleteItem( pItem );
}

CTreeListItem* CTreeListCtrl::InsertItem( LPCTSTR lpszItem, CTreeListItem* pParent, CTreeListItem* pInsertAfter )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::InsertItem", __DATE__,__FILE__, __LINE__);
	// insert an new item
	ASSERT( m_dwStyle&TLC_TREELIST || pParent == TLI_ROOT || pParent == m_pRootItem ) ;

	// add first level item
	if( pParent == TLI_ROOT )
	{
		return InsertItem( lpszItem, m_pRootItem, pInsertAfter );
	}

	// no child item
	if( pParent->m_pChild == NULL )
	{
		return InsertItemNew( lpszItem, pParent );
	}

	// add befor first child item
	if( pInsertAfter == TLI_FIRST )
	{
		return InsertItemFirst( lpszItem, pParent );
	}

	// add befor or after a child item by sorting
	if( pInsertAfter == TLI_SORT )
	{
		CTreeListItem* pPrevItem = GetSortChildItem( pParent );
		if( pPrevItem == NULL )
			return InsertItemFirst( lpszItem, pParent );
		else
			return InsertItemNext( lpszItem, pParent, pPrevItem );
	}

	// add after a child item
	if( pInsertAfter == TLI_LAST )
	{
		CTreeListItem* pPrevItem = GetLastChildItem( pParent );
		return InsertItemNext( lpszItem, pParent, pPrevItem );
	}
	
	// add after some child item
	{
		CTreeListItem* pPrevItem = GetValidChildItem( pParent, pInsertAfter );
		if( pPrevItem == NULL )
			return InsertItemFirst( lpszItem, pParent );
		else
			return InsertItemNext( lpszItem, pParent, pPrevItem );
	}

	return NULL;
}

BOOL CTreeListCtrl::DeleteItem( CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DeleteItem", __DATE__,__FILE__, __LINE__);
	// delete an item
	if( pItem == TLI_ROOT )
	{
		CTreeListItem* pChild = m_pRootItem->m_pChild;
		CTreeListItem* pDelete;
		while( pChild != NULL )
		{
			pDelete = pChild;
			pChild = pChild->m_pNext;
			DeleteItemX( pDelete );
		}
	}
	else
	{
		DeleteItemX( pItem );
	}

	if( IsWindow( GetSafeHwnd() ) )
	{
		Layout();
		Invalidate();
	}

	return TRUE;
}

BOOL CTreeListCtrl::DeleteAllItems()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DeleteAllItems", __DATE__,__FILE__, __LINE__);
	// delete all items of treelist control
	BOOL bRedraw = IsWindowVisible();
	
	if( bRedraw )
		SetRedraw( FALSE );

	BOOL bDelte = DeleteItem( TLI_ROOT );

	if( bRedraw )
		SetRedraw( TRUE );

	Invalidate();

	return bDelte;
}

BOOL CTreeListCtrl::Expand( CTreeListItem* pItem, int nCode )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::Expand", __DATE__,__FILE__, __LINE__);
	// expand an item
	return Expand( pItem, nCode, m_iSelCol );
}

BOOL CTreeListCtrl::Expand( CTreeListItem* pItem, int nCode, int iSubItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::Expand", __DATE__,__FILE__, __LINE__);
	// expand an item or collapse an item
	ASSERT( m_dwStyle&TLC_TREELIST );
	LRESULT lResult;

	if( pItem == NULL )
		return FALSE;

	if( pItem->GetState()&TLIS_EXPANDEDONCE && pItem->m_pChild == NULL )
		return TRUE;

	if( nCode == TLE_TOGGLE )
	{
		if( pItem->GetState()&TLIS_EXPANDED )
			nCode = TLE_COLLAPSE;
		else
			nCode = TLE_EXPAND;
	}

	switch( nCode )
	{
	case TLE_EXPAND:
		if( pItem->GetState()&TLIS_EXPANDED )
			return TRUE;
		break;

	case TLE_COLLAPSE:
	case TLE_COLLAPSERESET:
		if( !(pItem->GetState()&TLIS_EXPANDED ) )
			return TRUE;
		break;

	default:
		return FALSE;
		break;
	}

	lResult = Notify( TLN_ITEMEXPANDING, pItem, iSubItem );
	if( lResult == -1 )
		return FALSE;

	BOOL bRedraw = IsWindowVisible();
	
	if( bRedraw )
		SetRedraw( FALSE );

	BOOL bExpand = FALSE;

	switch( nCode )
	{
	case TLE_EXPAND:
		pItem->Expand();
		StatExpand( pItem );
		break;

	case TLE_COLLAPSE:
		pItem->Collapse();
		StatCollapse( pItem );
		break;

	case TLE_COLLAPSERESET:
		pItem->Collapse();
		while( pItem->m_pChild != NULL )
		{
			DeleteItem( pItem->m_pChild );
		}
		StatCollapse( pItem );
		break;

	default:
		ASSERT( FALSE );
		break;
	}

	//SelectItem( pItem, iSubItem );
	if( bRedraw )
		SetRedraw( TRUE );

	Layout();
	Invalidate();

	Notify( TLN_ITEMEXPANDED, pItem, iSubItem );

	return bExpand;
}

BOOL CTreeListCtrl::Select( CTreeListItem* pItem, int nCode )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::Select", __DATE__,__FILE__, __LINE__);
	// select an item
	SelectItem( pItem, 0, FALSE );
	return TRUE;
}

CTreeListItem* CTreeListCtrl::InsertItemNew( LPCTSTR lpszItem, CTreeListItem* pParent )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::InsertItemNew", __DATE__,__FILE__, __LINE__);
	// insert a new item
	ASSERT( pParent != NULL );

	CTreeListItem* pNewItem;
	pNewItem = new CTreeListItem(GetColumns());

	pNewItem->SetText( lpszItem );

	pNewItem->m_pParent	= pParent;
	pNewItem->m_nLevel	= pParent->m_nLevel + 1;

	if( pParent->GetCheck() == 1 )
		pNewItem->SetCheck( TRUE );

	if( pParent->GetLock() == 1 )
		pNewItem->SetLock( TRUE );


	pParent->m_pChild	= pNewItem;
	pParent->GetState( TLIS_EXPANDEDONCE );

	return InsertItemX( pParent, pNewItem );
}

CTreeListItem* CTreeListCtrl::InsertItemFirst( LPCTSTR lpszItem, CTreeListItem* pParent )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::InsertItemFirst", __DATE__,__FILE__, __LINE__);
	// insert a child item before first child item
	ASSERT( pParent != NULL );

	CTreeListItem* pNewItem;
	pNewItem = new CTreeListItem(GetColumns());

	pNewItem->SetText( lpszItem );

	CTreeListItem* pFirstItem;
	pFirstItem	= pParent->m_pChild;

	pNewItem->m_pParent = pFirstItem->m_pParent;
	pNewItem->m_pNext	= pFirstItem;
	pNewItem->m_nLevel	= pFirstItem->m_nLevel;

	if( pParent->GetCheck() == 1 )
		pNewItem->SetCheck( TRUE );

	if( pParent->GetLock() == 1 )
		pNewItem->SetLock( TRUE );
		
	pFirstItem->m_pPrev		= pNewItem;
	pParent->m_pChild	= pNewItem;

	return InsertItemX( pParent, pNewItem );
}

CTreeListItem* CTreeListCtrl::InsertItemNext( LPCTSTR lpszItem, CTreeListItem* pParent, CTreeListItem* pInsertAfter )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::InsertItemNext", __DATE__,__FILE__, __LINE__);
	// insert a child item after some item
	ASSERT( pParent != NULL && pInsertAfter != NULL );

	CTreeListItem* pNewItem;
	pNewItem = new CTreeListItem(GetColumns());

	pNewItem->SetText( lpszItem );

	pNewItem->m_pParent = pInsertAfter->m_pParent;
	pNewItem->m_pNext	= pInsertAfter->m_pNext;
	pNewItem->m_pPrev	= pInsertAfter;
	pNewItem->m_nLevel	= pInsertAfter->m_nLevel;

	if( pParent->GetCheck() == 1 )
		pNewItem->SetCheck( TRUE );

	if( pParent->GetLock() == 1 )
		pNewItem->SetLock( TRUE );

	pInsertAfter->m_pNext	= pNewItem;

	if( pNewItem->m_pNext != NULL )
		pNewItem->m_pNext->m_pPrev = pNewItem;

	return InsertItemX( pParent, pNewItem );
}

CTreeListItem* CTreeListCtrl::InsertItemX( CTreeListItem* pParent, CTreeListItem* pNewItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::InsertItemX", __DATE__,__FILE__, __LINE__);
	// insert an item
	ASSERT( pParent != NULL );
	ASSERT( pNewItem != NULL );

	StatChildAdd( pNewItem, pNewItem->m_nChild, pNewItem->m_nVisibleChild );

	pNewItem->SetImage( 0, 1, 2, 3 );

	Layout();
	Invalidate();

	return pNewItem;
}

BOOL CTreeListCtrl::DeleteItemX( CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DeleteItemX", __DATE__,__FILE__, __LINE__);
	// delete an item
	if( pItem->m_pPrev == NULL && pItem->m_pNext == NULL )
	{
		// no brother
		pItem->m_pParent->m_pChild = NULL;
	}
	else if( pItem->m_pPrev == NULL )
	{
		// first item of parent
		pItem->m_pParent->m_pChild = pItem->m_pNext;
		pItem->m_pNext->m_pPrev = NULL;
	}
	else if( pItem->m_pNext == NULL )
	{
		// last item of parent
		pItem->m_pPrev->m_pNext = NULL;
	}
	else
	{
		// middle item of parent
		pItem->m_pPrev->m_pNext = pItem->m_pNext;
		pItem->m_pNext->m_pPrev = pItem->m_pPrev;
	}

	StatChildDel( pItem, pItem->m_nChild, pItem->m_nVisibleChild );

	DeleteItemFast( pItem );

	return TRUE;
}

BOOL CTreeListCtrl::DeleteItemFast( CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DeleteItemFast", __DATE__,__FILE__, __LINE__);
	// fast delete child items of pItem
	CTreeListItem* pChild;
	CTreeListItem* pDelete;

	pChild = pItem->m_pChild;
	while( pChild != NULL )
	{
		pDelete = pChild;
		pChild = pChild->m_pNext;
		DeleteItemFast( pDelete );
	}

	for( int iSel=0; iSel<m_arSelects.GetSize(); iSel++ )
	{
		if( pItem == (CTreeListItem*)m_arSelects[iSel] )
		{
			m_arSelects.RemoveAt( iSel );
			break;
		}
	}

	if( pItem == m_pFocusItem )
	{
		m_pFocusItem = NULL;
	}

	Notify( TLN_SELCHANGING, pItem, 0 );

	delete pItem;

	return TRUE;
}

void CTreeListCtrl::ActiveItem( CTreeListItem* pItem, int iSubItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::ActiveItem", __DATE__,__FILE__, __LINE__);
	// active a selected item
	ASSERT( pItem != NULL && pItem->GetSelected() );

	if( !(m_dwStyle&TLC_MULTIPLESELECT) )
	{
		// single selected
		ASSERT( pItem == (CTreeListItem*)m_arSelects[0] );
		ASSERT( pItem == m_pFocusItem );
		m_iSelCol = iSubItem;
	}
	else
	{
		// multi selected
		BOOL bFound = FALSE;
		int iItem;
		for( iItem=0; iItem<m_arSelects.GetSize(); iItem++ )
		{
			if( (void*)pItem == m_arSelects[iItem] )
			{
				bFound = TRUE;
				break;
			}
		}

		ASSERT( bFound );
		
		m_arSelects.RemoveAt( iItem );
		m_arSelects.InsertAt( 0, (void*)pItem );
		m_iSelCol = iSubItem;
		m_pFocusItem = pItem;
	}
	
	EnsureVisible( pItem, iSubItem );
}

void CTreeListCtrl::SetTargetItem( CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetTargetItem", __DATE__,__FILE__, __LINE__);
	// set drop target item
	m_pTargetItem = pItem;
	Invalidate();
}

CTreeListItem* CTreeListCtrl::GetTargetItem()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetTargetItem", __DATE__,__FILE__, __LINE__);
	return m_pTargetItem;
}

void CTreeListCtrl::SetHoverItem( CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetHoverItem", __DATE__,__FILE__, __LINE__);
	// set hover target item
	m_pHoverItem = pItem;
	m_iHoverItem = GetTreeListItem( pItem );
	Invalidate();
}

void CTreeListCtrl::SelectItem( CTreeListItem* pItem, int iSubItem, BOOL bClearLastSelected )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SelectItem", __DATE__,__FILE__, __LINE__);
	LRESULT lResult;

	if( m_arSelects.GetSize() > 0 )
	{
		CTreeListItem* pSelItem = (CTreeListItem*)m_arSelects[0];
		if( pItem == pSelItem && iSubItem == m_iSelCol )
		{
			EnsureVisible( pItem, iSubItem );
			Invalidate();
			return;
		}
	}

	lResult = Notify( TLN_SELCHANGING, pItem, iSubItem );
	if( lResult == -1 )
		return;

	// select a new item
	if( pItem == NULL )
	{
		int iItem;
		for( iItem=0; iItem<m_arSelects.GetSize(); iItem++ )
		{
			CTreeListItem* pOldItem;
			pOldItem = (CTreeListItem*)m_arSelects[iItem];
			pOldItem->SetSelected( FALSE );
		}

		m_arSelects.RemoveAll();

		Notify( TLN_SELCHANGED, pItem, iSubItem );
		Invalidate();
		return;
	}

	// select a new item
	if( !(m_dwStyle&TLC_MULTIPLESELECT) )
	{
		// single select
		ASSERT( m_arSelects.GetSize() <= 1 );

		if( m_arSelects.GetSize() == 1 )
		{
			CTreeListItem* pOldItem = (CTreeListItem*)m_arSelects[0];
			pOldItem->SetSelected( FALSE );
			m_arSelects.RemoveAll();
		}

		pItem->SetSelected();
		m_iSelCol = iSubItem;
		m_pFocusItem = pItem;
		m_arSelects.Add( (void*) pItem );
	}
	else
	{
		// multi select
		if( bClearLastSelected )
		{
			int iItem;
			for( iItem=0; iItem<m_arSelects.GetSize(); iItem++ )
			{
				CTreeListItem* pOldItem;
				pOldItem = (CTreeListItem*)m_arSelects[iItem];
				pOldItem->SetSelected( FALSE );
			}

			m_arSelects.RemoveAll();
			pItem->SetSelected();
			m_iSelCol = iSubItem;
			m_pFocusItem = pItem;
			m_arSelects.Add( (void*)pItem );
		}
		else
		{
			BOOL bFound = FALSE;
			int iItem;
			for( iItem=0; iItem<m_arSelects.GetSize(); iItem++ )
			{
				if( (void*)pItem == m_arSelects[iItem] )
				{
					bFound = TRUE;
					break;
				}
			}

			if( bFound )
			{
				pItem->SetSelected( FALSE );
				m_arSelects.RemoveAt( iItem );
				m_iSelCol = iSubItem;
				m_pFocusItem = pItem;
			}
			else
			{
				pItem->SetSelected();
				m_arSelects.InsertAt( 0, (void*)pItem );
				m_iSelCol = iSubItem;
				m_pFocusItem = pItem;
			}
		}
	}

	Notify( TLN_SELCHANGED, pItem, iSubItem );
	EnsureVisible( pItem, iSubItem );
	Invalidate();
	return;
}

/*
void CTreeListCtrl::SetImageList( CTreeListItem* pItem, CImageList* pImageList )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetImageList", __DATE__,__FILE__, __LINE__);
	// 
	ASSERT( FALSE );

	pItem->SetImageList( pImageList );

	return;
}
*/

CTreeListItem* CTreeListCtrl::GetLastChildItem( CTreeListItem* pParent )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetLastChildItem", __DATE__,__FILE__, __LINE__);
	// retrieve last child item
	if( pParent == TLI_ROOT )
		return GetLastChildItem( m_pRootItem );

	if( pParent->m_pChild == NULL )
		return NULL;

	CTreeListItem* pChildItem;
	pChildItem = pParent->m_pChild;

	while( pChildItem->m_pNext != NULL )
	{
		pChildItem = pChildItem->m_pNext;
	}

	return pChildItem;
}

CTreeListItem* CTreeListCtrl::GetSortChildItem( CTreeListItem* pParent )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetSortChildItem", __DATE__,__FILE__, __LINE__);
	// retrieve sort child item
	if( pParent == TLI_ROOT )
		return GetSortChildItem( m_pRootItem );

	return GetLastChildItem( pParent );
}

CTreeListItem* CTreeListCtrl::GetValidChildItem( CTreeListItem* pParent, CTreeListItem* pChild )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetValidChildItem", __DATE__,__FILE__, __LINE__);
	// is it a valid child item
	ASSERT( pChild != NULL );

	if( pParent == TLI_ROOT )
	{
		return GetValidChildItem( m_pRootItem, pChild );
	}

	if( pParent->m_pChild == NULL )
	{
		return NULL;
	}

	if( pChild == NULL )
	{
		return NULL;
	}

	BOOL bValid = FALSE;
	CTreeListItem* pChildItem;
	pChildItem = pParent->m_pChild;

	while( pChildItem != NULL )
	{
		if( pChildItem == pChild )
		{
			bValid = TRUE;
			break;
		}
		pChildItem = pChildItem->m_pNext;
	}
	
	if( !bValid )
	{
		return NULL;
	}

	return pChild;
}

void CTreeListCtrl::OnMouseMove(UINT nFlags, CPoint point) 
{
	IMPLEMENT_LOGGING("CTreeListCtrl::OnMouseMove", __DATE__,__FILE__, __LINE__);
	// on mouse move
	int nFlag, iSubItem;
	
	BOOL bHide = TRUE;
	
	CRect rcText;
	CTreeListItem* pItem;

	if( GetState( TLS_DRAG ) )
	{
		ASSERT( GetState( TLS_CAPTURE ) );
		DoDraging( point );
	}
	else
	{
		pItem = HitTest( point, &nFlag, &iSubItem, &rcText );
		if( pItem != NULL )
		{
			if( GetState( TLS_FOCUS ) && !(nFlags&(MK_LBUTTON | MK_RBUTTON | MK_MBUTTON)) )
			{
				CTreeListColumnInfo* pColumnInfo = (CTreeListColumnInfo*)m_arColumns[iSubItem];
				m_wndTip.Show( rcText, pItem->GetText( iSubItem ), pColumnInfo->GetTextFormat() );
			}
			else
			{
				if( IsWindow( m_wndTip.GetSafeHwnd() ) && m_wndTip.IsWindowVisible() )
					m_wndTip.Hide();
			}

			if( nFlag&TLHT_ONITEMBUTTON )
			{
				SetCursor( m_hButton );
			} 

			if( nFlag&TLHT_ONITEMCHECKBOX )
			{
				SetCursor( m_hCheck );
			}

			if( nFlag&TLHT_ONITEMLOCKBOX )
			{
				SetCursor( m_hLock );
			}
		}
		else
		{
			if( IsWindow( m_wndTip.GetSafeHwnd() ) )
				m_wndTip.Hide();
		}

	}
	
	CWnd::OnMouseMove(nFlags, point);
}

void CTreeListCtrl::OnMButtonDown(UINT nFlags, CPoint point) 
{
	IMPLEMENT_LOGGING("CTreeListCtrl::OnMButtonDown", __DATE__,__FILE__, __LINE__);
	//CWheelWnd_OnMButtonDown;
	//The scrolling mechanism is nasty - this really screws up the display
      
    // If panning was not started, continue as usual...
    CWnd::OnMButtonDown(nFlags, point);
}

void CTreeListCtrl::OnLButtonDown(UINT nFlags, CPoint point) 
{
	IMPLEMENT_LOGGING("CTreeListCtrl::OnLButtonDown", __DATE__,__FILE__, __LINE__);
	// left button was down
	SetFocus();

	CWnd::OnLButtonDown( nFlags, point );

	Notify(NM_LDOWN, NULL, NULL);

	int nFlag, iSubItem;
	CRect rcText;

	CTreeListItem* pItem = HitTest( point, &nFlag, &iSubItem, &rcText );
	if( pItem != NULL )
	{
		if( m_dwStyle&TLC_TREELIST )
		{
			if( nFlag&TLHT_ONITEMBUTTON )
			{
				if( pItem->GetState() & TLIS_EXPANDED )
				{
					Expand( pItem, TLE_COLLAPSE, iSubItem );
				}
				else
				{
					Expand( pItem, TLE_EXPAND, iSubItem );
				}
				SetCursor( m_hButton );
			}
		}

		if( nFlag&TLHT_ONITEMCHECKBOX )
		{
			SetItemCheck( pItem );
			SetCursor( m_hCheck );
		}

		if( nFlag&TLHT_ONITEMLOCKBOX )
		{
			SetItemLock( pItem );
			SetCursor( m_hLock );
		}

		if( nFlag&TLHT_ONITEMTEXT && GetState( TLS_MODIFY ) )
		{
			if( pItem == m_pModifyItem )
				BeginModify( pItem, iSubItem );
			else
				FinishModify();
		}

		if( nFlag&TLHT_ONITEM && !(nFlag&TLHT_ONITEMBUTTON) && !(nFlag&TLHT_ONITEMCHECKBOX) && !(nFlag&TLHT_ONITEMLOCKBOX))
		{
			BOOL bDraging;
			if( !pItem->GetSelected() )
			{
				if( !(nFlags&MK_CONTROL) )
				{
					SelectItem( pItem, iSubItem );
				}
				else
				{
					SelectItem( pItem, iSubItem, FALSE );
				}
				bDraging = BeginDraging( point );
			}
			else
			{
				ActiveItem( pItem, iSubItem );
				bDraging = BeginDraging( point );
			}

			if( bDraging )
			{
				ASSERT( !GetState( TLS_CAPTURE ) );
				SetState( TLS_CAPTURE );
				SetCapture();
			}
		}

		Layout();
		Invalidate();
	}
	else
	{
		FinishModify();
		SelectItem( NULL );
		Layout();
		Invalidate();
	}

}

void CTreeListCtrl::OnLButtonUp(UINT nFlags, CPoint point) 
{
	IMPLEMENT_LOGGING("CTreeListCtrl::OnLButtonUp", __DATE__,__FILE__, __LINE__);
	// left button was up
	int nFlag, iSubItem;
	CTreeListItem* pItem;

	if( GetState( TLS_DRAG ) )
	{
		ReleaseCapture();
		SetState( 0, TLS_CAPTURE );

		EndDraging( point );

		Invalidate();
	}
	else
	{
		pItem = HitTest( point, &nFlag, &iSubItem );
		if( pItem != NULL )
		{
			if( nFlag&TLHT_ONITEMBUTTON )
			{
				SetCursor( m_hButton );
			}

			if( nFlag&TLHT_ONITEMCHECKBOX )
			{
				SetCursor( m_hCheck );
			}

			if( nFlag&TLHT_ONITEMLOCKBOX )
			{
				SetCursor( m_hLock );
			}
		}
	}
	
	ASSERT( !GetState( TLS_CAPTURE ) );

	CWnd::OnLButtonUp(nFlags, point);
}

void CTreeListCtrl::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	IMPLEMENT_LOGGING("CTreeListCtrl::OnLButtonDblClk", __DATE__,__FILE__, __LINE__);
	// left button double click
	CWnd::OnLButtonDblClk( nFlags, point );

	Notify(NM_DBLCLK, NULL, NULL);

	int nFlag, iSubItem;

	CTreeListItem* pItem = HitTest( point, &nFlag, &iSubItem );
	if( pItem != NULL )
	{
		if( nFlag&TLHT_ONITEMBUTTON )
		{
			SetCursor( m_hButton );
		}

		if( nFlag&TLHT_ONITEMCHECKBOX )
		{
			SetCursor( m_hCheck );
		}

		if( nFlag&TLHT_ONITEMLOCKBOX )
		{
			SetCursor( m_hLock );
		}

		if( m_dwStyle&TLC_TREELIST )
		{
			if ( nFlag&TLHT_ONITEMIMAGE || nFlag&TLHT_ONITEMTEXT )
			{
				//SelectItem( pItem, iSubItem );
				Expand( pItem, TLE_TOGGLE, iSubItem );
			}
		}

		Layout();
		Invalidate();
	}

}

void CTreeListCtrl::OnRButtonDown(UINT nFlags, CPoint point) 
{
	IMPLEMENT_LOGGING("CTreeListCtrl::OnRButtonDown", __DATE__,__FILE__, __LINE__);
	// right button was down
	SetFocus();

	CWnd::OnRButtonDown(nFlags, point);
	Notify(NM_RCLICK, NULL, NULL);

	int nFlag, iSubItem;
	CRect rcText;

	CTreeListItem* pItem = HitTest( point, &nFlag, &iSubItem, &rcText );
	if( pItem != NULL )
	{
		if( nFlag&TLHT_ONITEM && !(nFlag&TLHT_ONITEMBUTTON) )
		{
			if( !(nFlags&MK_CONTROL) )
				SelectItem( pItem, iSubItem );
			else
				SelectItem( pItem, iSubItem, FALSE ); 
		}

		Layout();
		Invalidate();
	}
	else
	{
		if( GetState( TLS_MODIFY ) )
		{
			CancelModify();
		}

		SelectItem( NULL );
		Layout();
		Invalidate();
	}
}

void CTreeListCtrl::OnRButtonUp(UINT nFlags, CPoint point) 
{
	IMPLEMENT_LOGGING("CTreeListCtrl::OnRButtonUp", __DATE__,__FILE__, __LINE__);
	// right button was up
	CWnd::OnRButtonUp(nFlags, point);
}

CTreeListItem* CTreeListCtrl::HitTest( CPoint pt, int* pFlag, int* pSubItem, CRect* prcText )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::HitTest", __DATE__,__FILE__, __LINE__);
	// hittest of all items
	int nFlag, nSubItem;
	CRect rcText;

	if( pFlag == NULL )
		pFlag = &nFlag;

	if( pSubItem == NULL )
		pSubItem = &nSubItem;

	if( prcText == NULL )
		prcText = &rcText;

	*pFlag		= 0;
	*pSubItem	= 0;
	rcText.SetRect( 0, 0, 0, 0 );

	int nFirstRow	=	GetScrollPos( SB_VERT );
	int	nShowRows	=	m_rcTreeList.Height() / GetItemHeight() + 1;

	CRect rcItem;
	rcItem.SetRect( 0, 0, GetWidth(), GetItemHeight() );
	rcItem.OffsetRect( m_rcTreeList.left, m_rcTreeList.top );
	rcItem.OffsetRect( -GetScrollPos( SB_HORZ ), 0 );

	for( int i = nFirstRow; i <= nFirstRow + nShowRows && i < GetVisibleCount(); i++ )
	{
		if( rcItem.PtInRect( pt ) )
		{
			CTreeListItem* pItem = GetFirstShowItem( i );
			ASSERT( pItem != NULL );

			return HitTest( pt, pFlag, pSubItem, prcText, rcItem, pItem );
		}
		rcItem.OffsetRect( 0, m_nItemHeight );
	}
	return NULL;
}

CTreeListItem* CTreeListCtrl::HitTest( CPoint pt, int* pFlag, int* pSubItem, CRect* prcText, CRect rcItem, CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::HitTest", __DATE__,__FILE__, __LINE__);
	// hittest of one item
	ASSERT( pFlag != NULL && pSubItem != NULL );

	int nColPos = 0;

	CTreeListColumnInfo* pColumnInfo;

	for( int iShow = 0; iShow < m_arShows.GetSize(); iShow++, nColPos += pColumnInfo->m_nWidth )
	{
		int iCol;

		// do NOT test zero column;
		iCol = m_arShows[iShow];
		pColumnInfo = (CTreeListColumnInfo*)m_arColumns[iCol];
		if( pColumnInfo->m_nWidth == 0 )
			continue;

		CRect rcColumn;
		rcColumn.SetRect( 0, rcItem.top, pColumnInfo->m_nWidth, rcItem.bottom );
		rcColumn.OffsetRect( nColPos, 0 );
		rcColumn.OffsetRect( rcItem.left, 0 );

		if( rcColumn.PtInRect( pt ) )
		{
			*pSubItem = iCol;
			if( iCol == 0 )
			{
				if ( m_dwStyle&TLC_TREELIST )
					return HitTestTree( pt, pFlag, prcText, rcColumn, pItem );
				else
					return HitTestMain( pt, pFlag, prcText, rcColumn, pItem );
			}
			else
			{
				*prcText = rcColumn;
				return HitTestList( pt, pFlag, prcText, rcColumn, pItem );
			}
		}
	}

	return NULL;
}

CTreeListItem* CTreeListCtrl::HitTestTree( CPoint pt, int* pFlag, CRect* prcText, CRect rcColumn, CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::HitTestTree", __DATE__,__FILE__, __LINE__);
	// hittest of tree column of treelist control
	ASSERT( pFlag != NULL );
	ASSERT( pItem != NULL );

	int nPerfix = 0;
	CRect rcGraph;
	
	// skip perfix and tree line
	nPerfix += TLL_BORDER;
	nPerfix += (pItem->m_nLevel-1) * TLL_WIDTH;

	// check tree button
	if( m_dwStyle&(TLC_TREELINE|TLC_BUTTON) )
	{
		if( !(pItem->GetState()&TLIS_EXPANDEDONCE) || pItem->m_pChild != NULL )
		{
			rcGraph.SetRect( 0, rcColumn.top, TLL_WIDTH, rcColumn.bottom );
			rcGraph.OffsetRect( nPerfix, 0 );
			rcGraph.OffsetRect( rcColumn.left, 0 );

			CPoint ptCenter;
			ptCenter.x = rcGraph.left + rcGraph.Width() / 2 - 1;
			ptCenter.y = rcGraph.top + rcGraph.Height() / 2 - 1;

			CRect rcButton;
			rcButton.SetRect( -TLL_BUTTON, -TLL_BUTTON, TLL_BUTTON+1, TLL_BUTTON+1 );
			rcButton.OffsetRect( ptCenter );

			if( rcButton.PtInRect( pt ) )
				*pFlag |= TLHT_ONITEMBUTTON;
		}
		nPerfix += TLL_WIDTH;
	}

	// check tree check box;
	if( m_dwStyle&TLC_CHECKBOX )
	{
		if( pItem->GetState()&TLIS_SHOWCHECKBOX )
		{
			rcGraph.SetRect( 0, rcColumn.top, TLL_WIDTH, rcColumn.bottom );
			rcGraph.OffsetRect( nPerfix, 0 );
			rcGraph.OffsetRect( rcColumn.left, 0 );

			if( rcGraph.PtInRect( pt ) )
				*pFlag |= TLHT_ONITEMCHECKBOX;

			nPerfix += TLL_WIDTH;
		}
	}

	// check tree lock box;
	if( m_dwStyle&TLC_LOCKBOX )
	{
		if( pItem->GetState()&TLIS_SHOWLOCKBOX )
		{
			rcGraph.SetRect( 0, rcColumn.top, TLL_WIDTH/2, rcColumn.bottom );
			rcGraph.OffsetRect( nPerfix, 0 );
			rcGraph.OffsetRect( rcColumn.left, 0 );

			if( rcGraph.PtInRect( pt ) )
				*pFlag |= TLHT_ONITEMLOCKBOX;

			nPerfix += TLL_WIDTH/2;
		}
	}

	// check tree image
	if( m_dwStyle&TLC_IMAGE )
	{
		if( pItem->GetState()&TLIS_SHOWTREEIMAGE )
		{
			rcGraph.SetRect( 0, rcColumn.top, TLL_WIDTH, rcColumn.bottom );
			rcGraph.OffsetRect( nPerfix, 0 );
			rcGraph.OffsetRect( rcColumn.left, 0 );
	
			if( rcGraph.PtInRect( pt ) )
				*pFlag |= TLHT_ONITEMIMAGE;

			nPerfix += TLL_WIDTH;
		}
	}

	// add gaps between text and image/lock 
	if( m_dwStyle&TLC_LOCKBOX || m_dwStyle&TLC_IMAGE )
		nPerfix += 2;

	// check tree text
	CRect rcText;
	rcText = rcColumn;
	rcText.left = rcText.left + nPerfix;

	if( rcText.Width() > 0 )
	{
		if( rcText.PtInRect( pt ) )
		{
			*pFlag |= TLHT_ONITEMTEXT;
			*prcText = rcText;
		}
	}

	if( *pFlag == 0 )
		*pFlag |= TLHT_ONITEMSPACE;

	return pItem;
}

CTreeListItem* CTreeListCtrl::HitTestMain( CPoint pt, int* pFlag, CRect* prcText, CRect rcColumn, CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::HitTestMain", __DATE__,__FILE__, __LINE__);
	// hittest of main column of list control
	ASSERT( pFlag != NULL );
	ASSERT( pItem != NULL );

	int nPerfix = 0;
	CRect rcGraph;
	
	// skip perfix and tree line
	nPerfix += TLL_BORDER;

	// check tree check box;
	if( m_dwStyle&TLC_CHECKBOX )
	{
		if( pItem->GetState()&TLIS_SHOWCHECKBOX )
		{
			rcGraph.SetRect( 0, rcColumn.top, TLL_WIDTH, rcColumn.bottom );
			rcGraph.OffsetRect( nPerfix, 0 );
			rcGraph.OffsetRect( rcColumn.left, 0 );

			if( rcGraph.PtInRect( pt ) )
				*pFlag |= TLHT_ONITEMCHECKBOX;

			nPerfix += TLL_WIDTH;
		}
	}

	// check tree lock box;
	if( m_dwStyle&TLC_LOCKBOX )
	{
		if( pItem->GetState()&TLIS_SHOWLOCKBOX )
		{
			rcGraph.SetRect( 0, rcColumn.top, TLL_WIDTH/2, rcColumn.bottom );
			rcGraph.OffsetRect( nPerfix, 0 );
			rcGraph.OffsetRect( rcColumn.left, 0 );

			if( rcGraph.PtInRect( pt ) )
				*pFlag |= TLHT_ONITEMLOCKBOX;

			nPerfix += TLL_WIDTH/2;
		}
	}

	// check tree image
	if( m_dwStyle&TLC_IMAGE )
	{
		if( pItem->GetState()&TLIS_SHOWTREEIMAGE )
		{
			rcGraph.SetRect( 0, rcColumn.top, TLL_WIDTH, rcColumn.bottom );
			rcGraph.OffsetRect( nPerfix, 0 );
			rcGraph.OffsetRect( rcColumn.left, 0 );
	
			if( rcGraph.PtInRect( pt ) )
				*pFlag |= TLHT_ONITEMIMAGE;

			nPerfix += TLL_WIDTH;
		}
	}

	// add gaps between text and image/lock 
	if( m_dwStyle&TLC_LOCKBOX || m_dwStyle&TLC_IMAGE )
		nPerfix += 2;

	// check tree text
	CRect rcText;
	rcText = rcColumn;
	rcText.left = rcText.left + nPerfix;

	if( rcText.Width() > 0 )
	{
		if( rcText.PtInRect( pt ) )
		{
			*pFlag |= TLHT_ONITEMTEXT;
			*prcText = rcText;
		}
	}

	if( *pFlag == 0 )
		*pFlag |= TLHT_ONITEMSPACE;

	return pItem;}

CTreeListItem* CTreeListCtrl::HitTestList( CPoint pt, int* pFlag, CRect* prcText, CRect rcColumn, CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::HitTestList", __DATE__,__FILE__, __LINE__);
	// hittest of other columns of treelist control
	ASSERT( pFlag != NULL );
	ASSERT( pItem != NULL );

	*pFlag = TLHT_ONITEMTEXT;
	*prcText = rcColumn;

	return pItem;
}

void CTreeListCtrl::OnSetFocus( CWnd* pOldWnd ) 
{
	IMPLEMENT_LOGGING("CTreeListCtrl::OnSetFocus", __DATE__,__FILE__, __LINE__);
	// set focus
	CWnd::OnSetFocus( pOldWnd );
	
	SetCtrlFocus( pOldWnd, TRUE );

	Invalidate();

	return;
}

void CTreeListCtrl::OnKillFocus( CWnd* pNewWnd ) 
{
	IMPLEMENT_LOGGING("CTreeListCtrl::OnKillFocus", __DATE__,__FILE__, __LINE__);
	// lost focus
	CWnd::OnKillFocus( pNewWnd );

	SetCtrlFocus( pNewWnd, FALSE );

	Invalidate();

	return;
}

BOOL CTreeListCtrl::PreTranslateMessage(MSG* pMsg) 
{
	IMPLEMENT_LOGGING("CTreeListCtrl::PreTranslateMessage", __DATE__,__FILE__, __LINE__);
	// pre translate some messages
	switch( pMsg->message )
	{
	case WM_MOUSEMOVE:
		// cancel tips
		{
			POINTS points;
			CPoint pt;

			points = MAKEPOINTS( pMsg->lParam );
			pt.x = points.x;
			pt.y = points.y;

			if( m_rcHeader.PtInRect( pt ) )
			{
				if( IsWindow( m_wndTip.GetSafeHwnd() ) && m_wndTip.IsWindowVisible() )
					m_wndTip.Hide();
			}
		}
		break;

	case WM_LBUTTONDOWN:
		// cancel midified controls
		if( GetState( TLS_MODIFY ) )
		{
			POINTS points;
			CPoint pt;

			points = MAKEPOINTS( pMsg->lParam );
			pt.x = points.x;
			pt.y = points.y;

			if( pMsg->hwnd == m_wndHeader.GetSafeHwnd() && m_rcHeader.PtInRect( pt ) )
			{
				if( !FinishModify() )
				{
					return TRUE;
				}			
			}

			int nFlag;
			int iSubItem;
			CRect rcText;
			CTreeListItem* pItem = HitTest( pt, &nFlag, &iSubItem, &rcText );
			if( pMsg->hwnd == GetSafeHwnd() && pItem != NULL && pItem != m_pModifyItem )
			{
				if( !FinishModify() )
				{
					return TRUE;
				}			
			}
		}
		break;

	case WM_LBUTTONDBLCLK:
		// filter dbclk message on button & chanbox & lockbox of treelist control
		{
			CTreeListItem* pItem = NULL;
			POINTS points;
			CPoint pt;
			int nFlag;
			int iSubItem;

			points = MAKEPOINTS( pMsg->lParam );
			pt.x = points.x;
			pt.y = points.y;

			pItem = HitTest( pt, &nFlag, &iSubItem );
			if( pItem != NULL && nFlag&TLHT_ONITEMBUTTON )
			{
				pMsg->message = WM_LBUTTONDOWN;
				return PreTranslateMessage( pMsg );
			}
			if( pItem != NULL && nFlag&TLHT_ONITEMCHECKBOX )
			{
				pMsg->message = WM_LBUTTONDOWN;
				return PreTranslateMessage( pMsg );
			}

			if( pItem != NULL && nFlag&TLHT_ONITEMLOCKBOX )
			{
				pMsg->message = WM_LBUTTONDOWN;
				return PreTranslateMessage( pMsg );
			}
		}
		break;

	case WM_RBUTTONDOWN:
		// dragging...
		if( GetState( TLS_DRAG ) )
		{
			return TRUE;
		}

		// cancel modified control
		if( GetState( TLS_MODIFY ) )
		{
			POINTS points;
			CPoint pt;

			points = MAKEPOINTS( pMsg->lParam );
			pt.x = points.x;
			pt.y = points.y;

			if( pMsg->hwnd == m_wndHeader.GetSafeHwnd() && m_rcHeader.PtInRect( pt ) )
			{
				CancelModify();
				return TRUE;
			}
			
			int nFlag;
			int iSubItem;
			CRect rcText;
			CTreeListItem* pItem = HitTest( pt, &nFlag, &iSubItem, &rcText );
			if( pMsg->hwnd == GetSafeHwnd() && m_rcTreeList.PtInRect( pt ) )
			{
				if( pItem != m_pModifyItem )
				{
					CancelModify();
					return TRUE;
				}
				else
				{
					return TRUE;
				}
			}
		}
		break;

	case WM_RBUTTONDBLCLK:
		// filter dbclk message on button & chanbox & lockbox of treelist control
		if( pMsg->hwnd == GetSafeHwnd() )
		{
			pMsg->message = WM_RBUTTONDOWN;
			return PreTranslateMessage( pMsg );
		}
		break;

	case WM_NCLBUTTONDOWN:
		// cancel modified controls
		if( GetState( TLS_MODIFY ) )
		{
			CancelModify();
			PostMessage( WM_NCLBUTTONDOWN, pMsg->wParam, pMsg->lParam );
			PostMessage( WM_NCLBUTTONUP,   pMsg->wParam, pMsg->lParam );
			return TRUE;
		}
		break;

	case WM_NCRBUTTONDOWN:
		// cancel modified controls
		if( GetState( TLS_MODIFY ) )
		{
			CancelModify();
			PostMessage( WM_NCRBUTTONDOWN, pMsg->wParam, pMsg->lParam );
			PostMessage( WM_NCRBUTTONUP,   pMsg->wParam, pMsg->lParam );
			return TRUE;
		}
		break;

	case WM_KEYDOWN:
		// filter keydown mesages by status
		{
			ASSERT( GetState( TLS_FOCUS ) );

			if( GetState( TLS_DRAG ) )
			{
				return TRUE;
			}

			if( !GetState( TLS_MODIFY ) )
			{
				if( PreTranslateKeyDownMessage( pMsg ) )
					return TRUE;
			}
			else
			{
				if( PreTranslateKeyDownMessage2( pMsg ) )
					return TRUE;
			}
		}
		break;

	case WM_CHAR:
	case WM_SYSCHAR:
	case WM_DEADCHAR:
	case WM_SYSDEADCHAR:

	case WM_KEYUP:
	case WM_SYSKEYDOWN:
	case WM_SYSKEYUP:

	case WM_COMMAND:
	case WM_SYSCOMMAND:
//	case WM_MENUCOMMAND:
		// filter keyup mesages by status
		{
			if( GetState( TLS_DRAG ) )
			{
				return TRUE;
			}
		}

	default:
		break;
	}

	return CWnd::PreTranslateMessage(pMsg);
}

BOOL CTreeListCtrl::PreTranslateKeyDownMessage( MSG* pMsg )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::PreTranslateKeyDownMessage", __DATE__,__FILE__, __LINE__);
	// pre translate key down messages
	ASSERT( pMsg->message == WM_KEYDOWN );
	CTreeListItem* pItem;
	int iShow;
	BOOL bFound;

	if( m_arShows.GetSize() == 0 )
		return TRUE;

	switch( pMsg->wParam )
	{
	case VK_ADD:
		{
			if( GetStyle()&TLC_TREELIST )
			{
				Expand( m_pFocusItem, TLE_EXPAND );
				return TRUE;
			}
		}
		break;

	case VK_SUBTRACT:
		{
			if( GetStyle()&TLC_TREELIST )
			{
				Expand( m_pFocusItem, TLE_COLLAPSE );
				return TRUE;
			}
		}
		break;

	case VK_SPACE:
		{
			if( GetStyle()&TLC_TREELIST )
			{
				Expand( m_pFocusItem, TLE_TOGGLE );
				return TRUE;
			}
		}
		break;

	case VK_LEFT:
		{
			if( GetKeyState( VK_SHIFT ) < 0 && GetStyle()&TLC_TREELIST )
			{
				Expand( m_pFocusItem, TLE_COLLAPSE );
				return TRUE;
			}

			for( iShow=0; iShow<m_arShows.GetSize(); iShow++ )
			{
				if( m_arShows[iShow] == m_iSelCol )
					break;
			}

			if( iShow == m_arShows.GetSize() )
			{
				m_iSelCol = m_arShows[0];
			}
			else
			{
				iShow = iShow - 1;
				if( iShow < 0 )
					iShow = 0;
				m_iSelCol = m_arShows[iShow];
			}

			if( m_pFocusItem != NULL )
				pItem = m_pFocusItem;
			else
				pItem = GetNextVisibleItem( m_pRootItem );

			if( pItem != NULL )
				SelectItem( pItem, m_iSelCol );

			return TRUE;
		}
		break;

	case VK_RIGHT:
		{
			if( GetKeyState( VK_SHIFT ) < 0 && GetStyle()&TLC_TREELIST )
			{
				Expand( m_pFocusItem, TLE_EXPAND );
				return TRUE;
			}

			for( iShow=0; iShow<m_arShows.GetSize(); iShow++ )
			{
				if( m_arShows[iShow] == m_iSelCol )
					break;
			}

			if( iShow == m_arShows.GetSize() )
			{
				m_iSelCol = m_arShows[0];
			}
			else
			{
				iShow = iShow + 1;
				if( iShow >= m_arShows.GetSize() )
					iShow = m_arShows.GetUpperBound();
				m_iSelCol = m_arShows[iShow];
			}


			if( m_pFocusItem != NULL )
				pItem = m_pFocusItem;
			else
				pItem = GetNextVisibleItem( m_pRootItem );

			if( pItem != NULL )
				SelectItem( pItem, m_iSelCol );

			return TRUE;
		}
		break;

	case VK_UP:
		{
			if( m_pFocusItem != NULL )
				pItem = GetPrevVisibleItem( m_pFocusItem );
			else
				pItem = GetNextVisibleItem( m_pRootItem );

			for( iShow=0, bFound=FALSE; iShow<m_arShows.GetSize(); iShow++ )
			{
				if( m_arShows[iShow] == m_iSelCol )
				{
					bFound = TRUE;
					break;
				}
			}

			if( !bFound )
			{
				m_iSelCol = 0;
			}

			if( pItem != NULL )
				SelectItem( pItem, m_iSelCol );

			return TRUE;
		}
		break;

	case VK_DOWN:
		{
			if( m_pFocusItem != NULL )
				pItem = GetNextVisibleItem( m_pFocusItem );
			else
				pItem = GetNextVisibleItem( m_pRootItem );

			for( iShow=0, bFound=FALSE; iShow<m_arShows.GetSize(); iShow++ )
			{
				if( m_arShows[iShow] == m_iSelCol )
				{
					bFound = TRUE;
					break;
				}
			}

			if( !bFound )
			{
				m_iSelCol = 0;
			}

			if( pItem != NULL )
				SelectItem( pItem, m_iSelCol );

			return TRUE;
		}
		break;

	case VK_PRIOR:	// VK_PAGE_UP
		{
			int nFirstItem;
			int nLastItem;
			int nCount;
			CTreeListItem* pFirstItem = NULL;

			nCount		= m_rcTreeList.Height() / GetItemHeight();
			if( m_rcTreeList.Height() >= GetItemHeight() )
			{
				nCount -= 1;
			}

			nFirstItem	= GetScrollPos( SB_VERT );
			nLastItem	= nFirstItem + nCount;

			pFirstItem = GetTreeListItem( nFirstItem );

			for( iShow=0, bFound=FALSE; iShow<m_arShows.GetSize(); iShow++ )
			{
				if( m_arShows[iShow] == m_iSelCol )
				{
					bFound = TRUE;
					break;
				}
			}

			if( !bFound )
			{
				m_iSelCol = 0;
			}

			if( m_pFocusItem != NULL )
			{
				int nFocusItem = GetTreeListItem( m_pFocusItem );
				if( nFirstItem > nFocusItem )
				{
					EnsureVisible( m_pFocusItem, m_iSelCol );
					return TRUE;
				}
			}

			if( pFirstItem != NULL )
			{
				if( pFirstItem != m_pFocusItem )
				{
					SelectItem( pFirstItem, m_iSelCol );
				}
				else
				{
					nFirstItem = nFirstItem - nCount - 1;
					if( nFirstItem < 0 )
						nFirstItem = 0;

					nLastItem = nLastItem - nCount - 1;
					if( nLastItem < 0 )
						nLastItem = 0;

					pFirstItem = GetTreeListItem( nFirstItem );
					SetScrollPos( SB_VERT, nFirstItem );
					SelectItem( pFirstItem, m_iSelCol );
				}
			}

			return TRUE;

		}
	case VK_NEXT:	// VK_PAGE_DOWN
		{
			int nFirstItem;
			int nLastItem;
			int nCount;
			CTreeListItem* pLastItem = NULL;
			
			nCount		= m_rcTreeList.Height() / GetItemHeight();
			if( m_rcTreeList.Height() >= GetItemHeight() )
			{
				nCount -= 1;
			}

			nFirstItem	= GetScrollPos( SB_VERT );
			nLastItem	= nFirstItem + nCount;

			if( nLastItem >= GetVisibleCount() )
			{
				nLastItem  = GetVisibleCount() - 1;
			}

			pLastItem = GetTreeListItem( nLastItem );

			for( iShow=0, bFound=FALSE; iShow<m_arShows.GetSize(); iShow++ )
			{
				if( m_arShows[iShow] == m_iSelCol )
				{
					bFound = TRUE;
					break;
				}
			}

			if( !bFound )
			{
				m_iSelCol = 0;
			}

			if( m_pFocusItem != NULL )
			{
				int nFocusItem = GetTreeListItem( m_pFocusItem );
				if( nLastItem < nFocusItem )
				{
					EnsureVisible( m_pFocusItem, m_iSelCol );
					return TRUE;
				}
			}
			
			if( pLastItem != NULL )
			{
				if( pLastItem != m_pFocusItem )
				{
					SelectItem( pLastItem, m_iSelCol );
				}
				else
				{
					if( nFirstItem < GetVisibleCount() - nCount - 1 )
					{
						if( nFirstItem + nCount + 1 < GetVisibleCount() - nCount - 1 )
						{
							nFirstItem = nFirstItem + nCount + 1;
						}
					}
					
					nLastItem	= nLastItem + nCount + 1;
					if( nLastItem >= GetVisibleCount() )
					{
						nLastItem = GetVisibleCount() - 1;
					}

					pLastItem = GetTreeListItem( nLastItem );
					SetScrollPos( SB_VERT, nFirstItem );
					SelectItem( pLastItem, m_iSelCol );
				}
			}

			return TRUE;
		}
		break;

	case VK_HOME:
		{
			pItem = GetNextVisibleItem( m_pRootItem );

			for( iShow=0, bFound=FALSE; iShow<m_arShows.GetSize(); iShow++ )
			{
				if( m_arShows[iShow] == m_iSelCol )
				{
					bFound = TRUE;
					break;
				}
			}

			if( !bFound )
			{
				m_iSelCol = 0;
			}

			if( pItem != NULL )
				SelectItem( pItem, m_iSelCol );

			return TRUE;
		}
		break;

	case VK_END:
		{
			CTreeListItem* pNextItem;
			pItem = GetNextVisibleItem( m_pRootItem );
			pNextItem = pItem;
			while( pNextItem != NULL )
			{
				pItem = pNextItem;
				pNextItem = GetNextVisibleItem( pNextItem );
			}

			for( iShow=0, bFound=FALSE; iShow<m_arShows.GetSize(); iShow++ )
			{
				if( m_arShows[iShow] == m_iSelCol )
				{
					bFound = TRUE;
					break;
				}
			}

			if( !bFound )
			{
				m_iSelCol = 0;
			}

			if( pItem != NULL )
				SelectItem( pItem, m_iSelCol );

			return TRUE;
		}

		break;

	case VK_RETURN:
		/*
		if( m_arSelects.GetSize() > 0 )
		{
			pItem = (CTreeListItem*)m_arSelects[0];
			int iModify = m_iSelCol;

			if( 0 <= iModify && iModify < m_arColumns.GetSize() )
				BeginModify( pItem, iModify );
		}
		return TRUE;
		*/
		break;

	default:
		break;

	}

	return FALSE;
}

BOOL CTreeListCtrl::PreTranslateKeyDownMessage2( MSG* pMsg )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::PreTranslateKeyDownMessage2", __DATE__,__FILE__, __LINE__);
	// pre translate key down messages
	ASSERT( pMsg->message == WM_KEYDOWN );
	int iPrevCol;
	int iNextCol;

	if( m_arShows.GetSize() == 0 )
		return TRUE;

	switch( pMsg->wParam )
	{
	case VK_TAB:
		if( GetKeyState( VK_SHIFT ) < 0 )
		{
			iPrevCol = GetPrevModifyCol();
			if( iPrevCol != -1 )
				BeginModify( m_pModifyItem, iPrevCol );
		}
		else
		{
			iNextCol = GetNextModifyCol();
			if( iNextCol != -1 )
				BeginModify( m_pModifyItem, iNextCol );
		}

		return TRUE;
		break;

	case VK_ESCAPE:
		if( GetState( TLS_MODIFY ) )
		{
			CancelModify();
			return TRUE;
		}
		break;

	case VK_RETURN:
		{
			FinishModify();
			return TRUE;
		}
		break;

	default:
		break;
	}

	return FALSE;
}

CTreeListItem* CTreeListCtrl::GetTreeListItem( int nItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetTreeListItem", __DATE__,__FILE__, __LINE__);
	// retrieve pointer of item by index of item
	if( GetCount() == 0 )
		return NULL;

	if( nItem < 0 || GetCount() <= nItem )
		return NULL;

	CTreeListItem* pItem = m_pRootItem->m_pChild;
	while( nItem != 0 )
	{
		ASSERT( nItem > 0 );

		if( nItem < pItem->m_nVisibleChild )
		{
			nItem --;
			pItem = pItem->m_pChild;
		}
		else
		{
			nItem -= pItem->m_nVisibleChild;
			pItem = pItem->m_pNext;
		}
	}

	return pItem;
}

int CTreeListCtrl::GetTreeListItem( CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetTreeListItem", __DATE__,__FILE__, __LINE__);
	// retrieve index of item by pointer of item
	if( GetCount() == 0 )
		return -1;

	if( pItem == NULL )
		return -1;

	if( pItem == m_pRootItem )
		return -1;

	int iItem = 1;
	while( pItem->m_pPrev != NULL )
	{
		pItem = pItem->m_pPrev;
		iItem = iItem + pItem->m_nVisibleChild;
	}

	pItem = pItem->m_pParent;
	iItem = iItem + GetTreeListItem( pItem );
	return iItem;
}

BOOL CTreeListCtrl::SetCtrlFocus( CWnd* pWnd, BOOL bFocus )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::SetCtrlFocus", __DATE__,__FILE__, __LINE__);
	// set focus

	// hidden tooltips
	if( IsWindow( m_wndTip.GetSafeHwnd() ) )
		m_wndTip.Hide();

	if( bFocus )
	{
		CWnd* pOldWnd = pWnd;
		if( pOldWnd == this )
			return FALSE;

		if( IsChild( pOldWnd ) )
			return FALSE;

		SetState( TLS_FOCUS );
	}
	else
	{
		CWnd* pNewWnd = pWnd;
		if( pNewWnd == this )
			return FALSE;

		if( IsChild( pNewWnd ) )
			return FALSE;

		if( GetState( TLS_MODIFY ) )
		{
			CancelModify();
		}

		SetState( 0, TLS_FOCUS );
	}
	
	Invalidate();

	return TRUE;
}

BOOL CTreeListCtrl::BeginModify( CTreeListItem* pItem, int iCol )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::BeginModify", __DATE__,__FILE__, __LINE__);
	// begin modify a column of an item
	ASSERT( pItem != NULL && 0 <= iCol && iCol < GetColumns() );

	if( (GetStyle()&TLC_READONLY) )
		return FALSE;

	// save values in m_pUpdateItem
	if( !GetState( TLS_MODIFY ) )
	{
		ASSERT( m_pModifyItem == NULL && m_pUpdateItem == NULL && m_iModifyCol == -1 );

		m_pModifyItem	= pItem;
		m_iModifyCol	= iCol;

		m_pUpdateItem	= new CTreeListItem( GetColumns() );
		for( int iColumn = 0; iColumn < GetColumns(); iColumn++ )
		{
			m_pUpdateItem->SetText( pItem->GetText( iColumn ), iColumn );
		}
	}

	// update modified columns
	if( GetState( TLS_MODIFY ) )
	{
		if( pItem == m_pModifyItem )
		{
			if( !UpdateModifyColumn() )
			{
				ASSERT( !GetState( TLS_MODIFY ) );
				SetState( TLS_MODIFY );
				RestoreControl( pItem, m_iModifyCol );
				return FALSE;
			}
			else
			{
				ASSERT( !GetState( TLS_MODIFY ) );
				SetState( TLS_MODIFY );
				return BeginControl( pItem, iCol );
			}
		}
		else
		{
			if( !UpdateModify() )
			{
				ASSERT( !GetState( TLS_MODIFY ) );
				return FALSE;
			}
			else
			{
				ASSERT( !GetState( TLS_MODIFY ) );
				return BeginModify( pItem, iCol );
			}
		}
	}
	
	// begin modify a new column
	ASSERT( !GetState( TLS_MODIFY ) );

	SetState( TLS_MODIFY );
	BeginControl( pItem, iCol );

	Invalidate();

	return TRUE;
}

BOOL CTreeListCtrl::UpdateModify()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::UpdateModify", __DATE__,__FILE__, __LINE__);
	// update modified column ( must cancel modified item )

	// avoid reentry
	if( GetState( TLS_MODIFYING ) )
		return TRUE;

	if( !GetState( TLS_MODIFY ) )
		return FALSE;

	// update all columns of an item
	ASSERT( GetState( TLS_MODIFY ) );
	ASSERT( m_pModifyItem != NULL && m_pUpdateItem != NULL && m_iModifyCol != -1 );

	ASSERT( !GetState( TLS_MODIFYING ) );

	LRESULT lr = 0;
	SetState( TLS_MODIFYING );

	// cancel control of an item
	CancelControl();

	// update modified column
 	CString strText;
	strText = m_pUpdateItem->GetText( m_iModifyCol );

	if( strText.Compare( m_pModifyItem->GetText( m_iModifyCol ) ) != 0 )
	{
		lr = Notify( TLN_ITEMUPDATING, m_pModifyItem, m_iModifyCol );

		if( lr < 0 )
		{
			m_pModifyItem->SetText( strText, m_iModifyCol );
			Notify( TLN_ITEMFINISHED, m_pModifyItem, -1 );
			delete m_pUpdateItem;
			m_pUpdateItem = NULL;
			m_pModifyItem = NULL;
			m_iModifyCol  = -1;
			SetState( 0, TLS_MODIFY | TLS_MODIFYING );
			Invalidate();
			return FALSE;
		}
	}

	// check if need update
	BOOL bUpdate = FALSE;
	for( int iCol=0; iCol<m_arColumns.GetSize(); iCol++ )
	{
		CString strText;
		strText = m_pUpdateItem->GetText( iCol );
		if( strText.Compare( m_pModifyItem->GetText( iCol ) ) != 0 )
		{
			bUpdate = TRUE;
			break;
		}
	}

	// update all columns
	if( bUpdate )
	{
		lr = Notify( TLN_ITEMUPDATED, m_pModifyItem, -1 );
	
		if( lr < 0 )
		{
			for( iCol = 0; iCol<m_arColumns.GetSize(); iCol++ )
			{
				CString strText;
				strText = m_pUpdateItem->GetText( iCol );
				m_pModifyItem->SetText( strText, iCol );
			}
			Notify( TLN_ITEMFINISHED, m_pModifyItem, -1 );
			delete m_pUpdateItem;
			m_pUpdateItem = NULL;
			m_pModifyItem = NULL;
			m_iModifyCol  = -1;
			SetState( 0, TLS_MODIFY | TLS_MODIFYING );
			Invalidate();
			return FALSE;
		}
		else
		{
			Notify( TLN_ITEMFINISHED, m_pModifyItem, 0 );
			delete m_pUpdateItem;
			m_pUpdateItem = NULL;
			m_pModifyItem = NULL;
			m_iModifyCol  = -1;
			SetState( 0, TLS_MODIFY | TLS_MODIFYING );
			Invalidate();
			return TRUE;
		}
	}
	else
	{
		Notify( TLN_ITEMFINISHED, m_pModifyItem, 1 );
		delete m_pUpdateItem;
		m_pUpdateItem = NULL;
		m_pModifyItem = NULL;
		m_iModifyCol  = -1;
		SetState( 0, TLS_MODIFY | TLS_MODIFYING );
		Invalidate();
		return TRUE;
	}
}

BOOL CTreeListCtrl::FinishModify()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::FinishModify", __DATE__,__FILE__, __LINE__);
	// finish modify
	
	// avoid reentry
	if( GetState( TLS_MODIFYING ) )
		return TRUE;

	if( !GetState( TLS_MODIFY ) )
		return FALSE;

	if( !UpdateModifyColumns() )
	{
		ASSERT( !GetState( TLS_MODIFY ) );
		SetState( TLS_MODIFY );
		RestoreControl( m_pModifyItem, m_iModifyCol );
		Invalidate();
		return FALSE;
	}

	Invalidate();
	return TRUE;
}

BOOL CTreeListCtrl::CancelModify()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::CancelModify", __DATE__,__FILE__, __LINE__);
	// cancel modify

	// avoid reentry
	if( GetState( TLS_MODIFYING ) )
		return TRUE;

	if( !GetState( TLS_MODIFY ) )
		return FALSE;

	ASSERT( GetState( TLS_MODIFY ) );
	ASSERT( m_pModifyItem != NULL && m_pUpdateItem != NULL && m_iModifyCol != -1 );

	// cancel all columns of an item
	CancelControl();

	// restore values from m_pUpdateItem
	for( int iColumn = 0; iColumn < GetColumns(); iColumn++ )
	{
		m_pModifyItem->SetText( m_pUpdateItem->GetText( iColumn ), iColumn );
	}

	Notify( TLN_ITEMFINISHED, m_pModifyItem, 1 );
	delete m_pUpdateItem;
	m_pUpdateItem = NULL;
	m_pModifyItem = NULL;
	m_iModifyCol  = -1;
	SetState( 0, TLS_MODIFY );

	Invalidate();
	
	return TRUE;
}

BOOL CTreeListCtrl::UpdateModifyColumn()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::UpdateModifyColumn", __DATE__,__FILE__, __LINE__);
	// update modified column ( do not cancel modified item )

	// avoid reentry
	if( GetState( TLS_MODIFYING ) )
		return TRUE;

	if( !GetState( TLS_MODIFY ) )
		return TRUE;

	ASSERT( GetState( TLS_MODIFY ) );
	ASSERT( m_pModifyItem != NULL && m_pUpdateItem != NULL && m_iModifyCol != -1 );

	ASSERT( !GetState( TLS_MODIFYING ) );

	LRESULT lr = 0;
	SetState( TLS_MODIFYING );

	// cancel control of an item
	CancelControl();

	// update active column
 	CString strText;
	strText = m_pUpdateItem->GetText( m_iModifyCol );

	if( strText.Compare( m_pModifyItem->GetText( m_iModifyCol ) ) != 0 )
	{
		lr = Notify( TLN_ITEMUPDATING, m_pModifyItem, m_iModifyCol );

		if( lr < 0 )
		{
			m_pModifyItem->SetText( strText, m_iModifyCol );
			SetState( 0, TLS_MODIFY | TLS_MODIFYING );
			return FALSE;
		}
	}

	SetState( 0, TLS_MODIFY | TLS_MODIFYING );
	return TRUE;
}

BOOL CTreeListCtrl::UpdateModifyColumns()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::UpdateModifyColumns", __DATE__,__FILE__, __LINE__);
	// update modified columns ( cancel modified item on suceed )

	// avoid reentry
	if( GetState( TLS_MODIFYING ) )
		return TRUE;

	if( !GetState( TLS_MODIFY ) )
		return TRUE;

	ASSERT( GetState( TLS_MODIFY ) );
	ASSERT( m_pModifyItem != NULL && m_pUpdateItem != NULL && m_iModifyCol != -1 );

	ASSERT( !GetState( TLS_MODIFYING ) );

	LRESULT lr = 0;
	SetState( TLS_MODIFYING );

	// cancel control of an item
	CancelControl();

	// update active column
 	CString strText;
	strText = m_pUpdateItem->GetText( m_iModifyCol );

	if( strText.Compare( m_pModifyItem->GetText( m_iModifyCol ) ) != 0 )
	{
		lr = Notify( TLN_ITEMUPDATING, m_pModifyItem, m_iModifyCol );

		if( lr < 0 )
		{
			SetState( 0, TLS_MODIFY | TLS_MODIFYING );
			return FALSE;
		}
	}

	// check if need update
	BOOL bUpdate = FALSE;
	for( int iCol=0; iCol<m_arColumns.GetSize(); iCol++ )
	{
		CString strText;
		strText = m_pUpdateItem->GetText( iCol );
		if( strText.Compare( m_pModifyItem->GetText( iCol ) ) != 0 )
		{
			bUpdate = TRUE;
			break;
		}
	}

	// update all columns
	if( bUpdate )
	{
		lr = Notify( TLN_ITEMUPDATED, m_pModifyItem, -1 );
	
		if( lr < 0 )
		{
			SetState( 0, TLS_MODIFY | TLS_MODIFYING );
			return FALSE;
		}
		else
		{
			Notify( TLN_ITEMFINISHED, m_pModifyItem, 0 );
			delete m_pUpdateItem;
			m_pUpdateItem = NULL;
			m_pModifyItem = NULL;
			m_iModifyCol  = -1;
			SetState( 0, TLS_MODIFY | TLS_MODIFYING );
			return TRUE;
		}
	}
	else
	{
		Notify( TLN_ITEMFINISHED, m_pModifyItem, 1 );
		delete m_pUpdateItem;
		m_pUpdateItem = NULL;
		m_pModifyItem = NULL;
		m_iModifyCol  = -1;
		SetState( 0, TLS_MODIFY | TLS_MODIFYING );
		return TRUE;
	}
}

CWnd* CTreeListCtrl::GetControl( CTreeListItem* pItem, int iCol )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetControl", __DATE__,__FILE__, __LINE__);
	// get pointer of modified control

	// check parameter
	if( pItem == NULL )
		return NULL;

	if( iCol < 0 ||  m_arColumns.GetSize() <= iCol )
		return NULL;

	// check state of control
	if( !GetState( TLS_MODIFY ) )
		return NULL;

	CWnd* pWnd;
	// restore a control
	switch( GetColumnModify( iCol ) )
	{
	case TLM_STATIC:
		pWnd = (CWnd*)&m_wndStatic;
		break;

	case TLM_EDIT:
		pWnd = (CWnd*)&m_wndEdit;
		break;

	case TLM_COMBO:
		pWnd = (CWnd*)&m_wndCombo;
		break;

	default:
		break;
	}

	return pWnd;
}

int CTreeListCtrl::GetPrevModifyCol()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetPrevModifyCol", __DATE__,__FILE__, __LINE__);
	// retrieve previous modify column
	ASSERT( GetState( TLS_MODIFY ) );

	int  iShow;
	int  iShowCol	= -1;
	int  iPrevCol	= -1;
	BOOL bFound		= FALSE;

	for( iShow = 0; iShow < m_arShows.GetSize(); iShow++ )
	{
		if( m_arShows[iShow] == m_iModifyCol )
		{
			iShowCol = iShow;
			bFound = TRUE;
			break;
		}
	}

	ASSERT( bFound == TRUE );

	for( iShow = iShowCol - 1; iShow >= 0; iShow-- )
	{
		if( GetColumnModify( m_arShows[iShow] ) )
		{
			iPrevCol = m_arShows[iShow];
			break;
		}
	}

	return iPrevCol;
}

int CTreeListCtrl::GetNextModifyCol()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::GetNextModifyCol", __DATE__,__FILE__, __LINE__);
	// retrieve next modify column
	ASSERT( GetState( TLS_MODIFY ) );

	int  iShow;
	int  iShowCol	= -1;
	int  iNextCol	= -1;
	BOOL bFound		= FALSE;

	for( iShow = 0; iShow < m_arShows.GetSize(); iShow++ )
	{
		if( m_arShows[iShow] == m_iModifyCol )
		{
			iShowCol = iShow;
			bFound	 = TRUE;
			break;
		}
	}

	ASSERT( bFound == TRUE );

	for( iShow = iShowCol + 1; iShow < m_arShows.GetSize(); iShow++ )
	{
		if( GetColumnModify( m_arShows[iShow] ) )
		{
			iNextCol = m_arShows[iShow];
			break;
		}
	}

	return iNextCol;
}

BOOL CTreeListCtrl::BeginControl( CTreeListItem* pItem, int iCol )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::BeginControl", __DATE__,__FILE__, __LINE__);
	// begin a modify control
	ASSERT( GetState( TLS_MODIFY ) );

	// clean first
	m_wndStatic.SetWindowText( _T("") );
	m_wndEdit.SetWindowText( _T("") );
	m_wndCombo.ResetContent();

	// notify
	Notify( TLN_BEGINCONTROL, m_pModifyItem, iCol );

	// get text rect
	CRect rcText;
	GetItemRect( pItem, iCol, &rcText, TRUE );

	// begin a control
	switch( GetColumnModify( iCol ) )
	{
	case TLM_STATIC:
		BeginStatic( pItem, iCol, rcText );
		break;

	case TLM_EDIT:
		BeginEdit( pItem, iCol, rcText );
		break;

	case TLM_COMBO:
		rcText.OffsetRect( 0, -1 );
		BeginCombo( pItem, iCol, rcText );
		break;

	default:
		break;
	}

	return TRUE;
}

BOOL CTreeListCtrl::CancelControl()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::CancelControl", __DATE__,__FILE__, __LINE__);
	// end modify a column of an item
	ASSERT( GetState( TLS_MODIFY ) );

	// cancel a control
	switch( GetColumnModify( m_iModifyCol ) )
	{
	case TLM_STATIC:
		CancelStatic();
		break;

	case TLM_EDIT:
		CancelEdit();
		break;

	case TLM_COMBO:
		CancelCombo();
		break;

	default:
		break;
	}

	// notify
	Notify( TLN_ENDCONTROL, m_pModifyItem, m_iModifyCol );

	return TRUE;
}

BOOL CTreeListCtrl::RestoreControl( CTreeListItem* pItem, int iCol )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::RestoreControl", __DATE__,__FILE__, __LINE__);
	// restore a modify control
	ASSERT( GetState( TLS_MODIFY ) );

	// get text rect
	CRect rcText;
	GetItemRect( pItem, iCol, &rcText, TRUE );

	// restore a control
	switch( GetColumnModify( iCol ) )
	{
	case TLM_STATIC:
		RestoreStatic( pItem, iCol, rcText );
		break;

	case TLM_EDIT:
		RestoreEdit( pItem, iCol, rcText );
		break;

	case TLM_COMBO:
		RestoreCombo( pItem, iCol, rcText );
		break;

	default:
		break;
	}

	return TRUE;
}

BOOL CTreeListCtrl::BeginStatic( CTreeListItem* pItem, int iCol, CRect rcText )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::BeginStatic", __DATE__,__FILE__, __LINE__);
	// begin static a column of an item
	ASSERT( 0 <= iCol && iCol <= m_arColumns.GetSize() );

	if( rcText.IsRectEmpty() )
		return FALSE;

	CTreeListColumnInfo* pColumnInfo = (CTreeListColumnInfo*)m_arColumns[iCol];
	int nMin = GetStringWidth( pItem->GetText( iCol ) );
	DWORD dwStyle = 0;

	if( rcText.Width() < pColumnInfo->GetMinWidth() )
		rcText.right = rcText.left + pColumnInfo->GetWidth();

	if( rcText.Width() < nMin )
		rcText.right = rcText.left + pColumnInfo->GetWidth();

	if( GetStyle()&TLC_HGRID )
		rcText.top--;

	if( GetStyle()&TLC_VGRID )
		rcText.left--;

	if( pColumnInfo->GetFormat()&TLF_TEXTALIGN_LEFT )
		dwStyle |= SS_LEFT;
	else if( pColumnInfo->GetFormat()&TLF_TEXTALIGN_RIGHT )
		dwStyle |= SS_RIGHT;
	else if( pColumnInfo->GetFormat()&TLF_TEXTALIGN_CENTER )
		dwStyle |= SS_CENTER;
	else
		dwStyle |= SS_LEFT;

	m_pModifyItem	= pItem;
	m_iModifyCol	= iCol;

	m_wndStatic.ModifyStyle( SS_LEFT|SS_RIGHT|SS_CENTER, 0, SS_LEFT|SS_RIGHT|SS_CENTER );
	m_wndStatic.ModifyStyle( 0, dwStyle, dwStyle );
	m_wndStatic.MoveWindow( rcText );
	m_wndStatic.EnableWindow( TRUE );
	m_wndStatic.SetWindowText( GetItemText( pItem, iCol ) );
	m_wndStatic.ShowWindow( SW_SHOW );
	m_wndStatic.SetFocus();

	return TRUE;
}

BOOL CTreeListCtrl::CancelStatic()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::CancelStatic", __DATE__,__FILE__, __LINE__);
	// cancel an static control
	m_wndStatic.ShowWindow( SW_HIDE );
	m_wndStatic.EnableWindow( FALSE );

	return TRUE;
}

BOOL CTreeListCtrl::RestoreStatic( CTreeListItem* pItem, int iCol, CRect rcText )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::RestoreStatic", __DATE__,__FILE__, __LINE__);
	// restore an static control
	ASSERT( m_pModifyItem	== pItem );
	ASSERT(m_iModifyCol		== iCol  );

	m_wndStatic.MoveWindow( rcText );
	m_wndStatic.EnableWindow( TRUE );
	m_wndStatic.ShowWindow( SW_SHOW );
	m_wndStatic.SetFocus();

	return TRUE;
}

BOOL CTreeListCtrl::BeginEdit( CTreeListItem* pItem, int iCol, CRect rcText )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::BeginEdit", __DATE__,__FILE__, __LINE__);
	// begin edit a column of an item
	ASSERT( 0 <= iCol && iCol <= m_arColumns.GetSize() );

	if( rcText.IsRectEmpty() )
		return FALSE;

	CTreeListColumnInfo* pColumnInfo = (CTreeListColumnInfo*)m_arColumns[iCol];
	int nMin = GetStringWidth( pItem->GetText( iCol ) );
	DWORD dwStyle = 0;

	if( rcText.Width() < pColumnInfo->GetMinWidth() )
		rcText.right = rcText.left + pColumnInfo->GetWidth();

	if( rcText.Width() < nMin )
		rcText.right = rcText.left + pColumnInfo->GetWidth();

	if( GetStyle()&TLC_HGRID )
		rcText.top--;

	if( GetStyle()&TLC_VGRID )
		rcText.left--;

	if( pColumnInfo->GetFormat()&TLF_TEXTALIGN_LEFT )
		dwStyle |= ES_LEFT;
	else if( pColumnInfo->GetFormat()&TLF_TEXTALIGN_RIGHT )
		dwStyle |= ES_RIGHT;
	else if( pColumnInfo->GetFormat()&TLF_TEXTALIGN_CENTER )
		dwStyle |= ES_CENTER;
	else
		dwStyle |= ES_LEFT;

	m_pModifyItem	= pItem;
	m_iModifyCol	= iCol;

	m_wndEdit.ModifyStyle( ES_LEFT|ES_RIGHT|ES_CENTER, 0, ES_LEFT|ES_RIGHT|ES_CENTER );
	m_wndEdit.ModifyStyle( 0, dwStyle, dwStyle );
	m_wndEdit.MoveWindow( rcText );
	m_wndEdit.EnableWindow( TRUE );
	m_wndEdit.SetWindowText( GetItemText( pItem, iCol ) );
	m_wndEdit.SetSel( 0, -1 );
	m_wndEdit.ShowWindow( SW_SHOW );
	m_wndEdit.SetFocus();

	return TRUE;
}

BOOL CTreeListCtrl::CancelEdit()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::CancelEdit", __DATE__,__FILE__, __LINE__);
	// cancel an edit control
	CString strText;
	m_wndEdit.GetWindowText( strText );

	m_pModifyItem->SetText( strText, m_iModifyCol );

	m_wndEdit.ShowWindow( SW_HIDE );
	m_wndEdit.EnableWindow( FALSE );

	return TRUE;
}

BOOL CTreeListCtrl::RestoreEdit( CTreeListItem* pItem, int iCol, CRect rcText )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::RestoreEdit", __DATE__,__FILE__, __LINE__);
	// restore an edit control
	ASSERT( m_pModifyItem	== pItem );
	ASSERT(m_iModifyCol		== iCol  );

	m_wndEdit.MoveWindow( rcText );
	m_wndEdit.EnableWindow( TRUE );
	m_wndEdit.ShowWindow( SW_SHOW );
	m_wndEdit.SetFocus();

	return TRUE;
}

BOOL CTreeListCtrl::BeginCombo( CTreeListItem* pItem, int iCol, CRect rcText )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::BeginCombo", __DATE__,__FILE__, __LINE__);
	// begin combo a column of an item
	ASSERT( 0 <= iCol && iCol <= m_arColumns.GetSize() );

	if( rcText.IsRectEmpty() )
		return FALSE;

	rcText.bottom += GetItemHeight() * 6;

	CTreeListColumnInfo* pColumnInfo = (CTreeListColumnInfo*)m_arColumns[iCol];
	int nMin = GetStringWidth( pItem->GetText( iCol ) );
	DWORD dwStyle = 0;

	m_pModifyItem	= pItem;
	m_iModifyCol	= iCol;

	CString strInit = GetColumnDefaultText( iCol );
	CString strItem;

	int nBeg = 0;
	int nEnd = strInit.Find( _T("|"), nBeg );
	while( nEnd != -1 )
	{
		strItem = strInit.Mid( nBeg, nEnd-nBeg );
		m_wndCombo.InsertString( -1, strItem );
		nBeg = nEnd + 1;
		nEnd = strInit.Find( _T("|"), nBeg );
	}
	strItem = strInit.Mid( nBeg );
	m_wndCombo.InsertString( -1, strItem );

	m_wndCombo.MoveWindow( rcText );
	m_wndCombo.EnableWindow( TRUE );
	if( m_wndCombo.SelectString( -1, GetItemText( pItem, iCol ) ) == -1 )
	{
		m_wndCombo.InsertString( 0, GetItemText( pItem, iCol ) );
		m_wndCombo.SelectString( -1, GetItemText( pItem, iCol ) );
	}

	m_wndCombo.ShowWindow( SW_SHOW );
	m_wndCombo.SetFocus();

	return TRUE;
}

BOOL CTreeListCtrl::CancelCombo()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::CancelCombo", __DATE__,__FILE__, __LINE__);
	// cancel a combo control
	CString strText;
	m_wndCombo.GetWindowText( strText );

	m_pModifyItem->SetText( strText, m_iModifyCol );

	m_wndCombo.ShowWindow( SW_HIDE );
	m_wndCombo.EnableWindow( FALSE );

	return TRUE;
}

BOOL CTreeListCtrl::RestoreCombo( CTreeListItem* pItem, int iCol, CRect rcText)
{
	IMPLEMENT_LOGGING("CTreeListCtrl::RestoreCombo", __DATE__,__FILE__, __LINE__);
	// restore a combo control
	ASSERT( m_pModifyItem	== pItem );
	ASSERT(m_iModifyCol		== iCol  );

	m_wndCombo.MoveWindow( rcText );
	m_wndCombo.EnableWindow( TRUE );
	m_wndCombo.ShowWindow( SW_SHOW );
	m_wndCombo.SetFocus();

	return TRUE;
}

BOOL CTreeListCtrl::BeginDraging( CPoint point )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::BeginDraging", __DATE__,__FILE__, __LINE__);
	// begin a draging
	ASSERT( GetSelectedItem() != NULL );
	
	if( !(GetStyle()&TLC_DRAG) )
		return FALSE;

	LRESULT lResult = NotifyDrop( TLN_BEGINDRAG, this, NULL );
	if( lResult < 0 )
		return FALSE;

	SetState( TLS_DRAG );
	
	m_ptBegin = point;
	m_ptDelta.x = -160;
	m_ptDelta.y = -160;

	return TRUE;
}

BOOL CTreeListCtrl::EndDraging( CPoint point )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::EndDraging", __DATE__,__FILE__, __LINE__);
	// end a draging
	ASSERT( GetStyle()&TLC_DRAG );

	if( m_pDragWnd != NULL )
	{
		m_pDragWnd->Hide();
		m_pDragWnd->DestroyWindow();
		delete m_pDragWnd;
		m_pDragWnd = NULL;
	}

	if( m_pDropWnd != NULL )
	{
		m_pDropWnd->Hide();
		m_pDropWnd->DestroyWindow();
		delete m_pDropWnd;
		m_pDropWnd = NULL;
	}
	
	ClientToScreen( &point );

	CWnd* pWnd;
	pWnd = CWnd::WindowFromPoint( point );
	if( pWnd != NULL && pWnd->IsKindOf( RUNTIME_CLASS(CTreeListCtrl) ) )
	{
		CTreeListCtrl* pTargetWnd;
		pTargetWnd = (CTreeListCtrl*)pWnd;
		pTargetWnd->Drop( this, point );
	}

	SetState( 0, TLS_DRAG );
	
	return TRUE;
}

BOOL CTreeListCtrl::EndDraging()
{
	IMPLEMENT_LOGGING("CTreeListCtrl::EndDraging", __DATE__,__FILE__, __LINE__);
	// cancel a draging
	ASSERT( GetStyle()&TLC_DRAG );

	if( m_pDragWnd != NULL )
	{
		m_pDragWnd->Hide();
		m_pDragWnd->DestroyWindow();
		delete m_pDragWnd;
		m_pDragWnd = NULL;
	}

	if( m_pDropWnd != NULL )
	{
		m_pDropWnd->Hide();
		m_pDropWnd->DestroyWindow();
		delete m_pDropWnd;
		m_pDropWnd = NULL;
	}

	if( m_pTargetWnd != NULL && m_pTargetWnd->IsKindOf( RUNTIME_CLASS( CTreeListCtrl ) ) )
	{
		m_pTargetWnd->DragLeave( this );
		m_pTargetWnd = NULL;
	}

	SetState( 0, TLS_DRAG );
	return TRUE;
}

BOOL CTreeListCtrl::DoDraging( CPoint point )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DoDraging", __DATE__,__FILE__, __LINE__);
	// draging
	ASSERT( GetStyle()&TLC_DRAG );

	if( m_pDragWnd == NULL )
	{
		int cx = GetSystemMetrics( SM_CXDRAG );
		int cy = GetSystemMetrics( SM_CYDRAG );

		CRect rcDrag;
		rcDrag.SetRect( -cx, -cy, cx, cy );
		rcDrag.OffsetRect( m_ptBegin );

		if( !rcDrag.PtInRect( point ) )
		{
			CRect rcDragWnd;
			rcDragWnd.SetRect( 0, 0, 320, 320 );
			rcDragWnd.OffsetRect( point );
			rcDragWnd.OffsetRect( m_ptDelta );
			ClientToScreen( rcDragWnd );

			m_pDragWnd = new CTLCDragWnd;
			m_pDragWnd->Create( rcDragWnd, this );
			m_pDragWnd->Show();

			m_pDropWnd = new CTLCDropWnd;
			m_pDropWnd->Create( this );

			DoDraging( point );
		}
	}
	else
	{
		CRect rcDragWnd;
		m_pDragWnd->GetClientRect( &rcDragWnd );
		rcDragWnd.OffsetRect( point );
		if( m_pDragWnd->m_bLayeredWindows )
			rcDragWnd.OffsetRect( m_ptDelta );
		else
			rcDragWnd.OffsetRect( 20, 4 );

		ClientToScreen( rcDragWnd );

		m_pDragWnd->MoveWindow( rcDragWnd );

		CPoint pt;
		pt = point;
		ClientToScreen( &pt );

		CWnd* pWnd;
		pWnd = CWnd::WindowFromPoint( pt );
		if( pWnd != NULL && pWnd->IsKindOf( RUNTIME_CLASS(CTreeListCtrl) ) )
		{
			CTreeListCtrl* pTargetWnd;
			pTargetWnd = (CTreeListCtrl*)pWnd;

			if( m_pTargetWnd != NULL && m_pTargetWnd != pTargetWnd )
			{
				m_pTargetWnd->DragLeave( this );
				m_pTargetWnd = NULL;
			}

			if( m_pTargetWnd == NULL && pTargetWnd->DragEnter( this ))
			{
				m_pTargetWnd = pTargetWnd;
			}

			if( m_pTargetWnd != NULL )
			{
				CTreeListItem* pItem = NULL;
				CRect rcItem;
				CRect rcClient;
				int nDrag = m_pTargetWnd->DragOver( this, pt, &pItem );
				switch( nDrag )
				{
				case DROP_NONE:
					SetCursor( m_hStop );
					m_pDropWnd->Hide();
					break;

				case DROP_SELF:
					SetCursor( m_hArrow );
					m_pDropWnd->Hide();
					break;

				case DROP_ITEM:
					SetCursor( m_hDrop );
					m_pTargetWnd->GetItemRect( pItem, &rcItem );
					m_pTargetWnd->ClientToScreen( &rcItem );
					m_pTargetWnd->GetClientRect( &rcClient );
					m_pTargetWnd->ClientToScreen( &rcClient );
					m_pDropWnd->Show( CPoint( rcClient.left, ( rcItem.top + rcItem.bottom ) / 2 ) );
					break;

				case DROP_ROOT:
					SetCursor( m_hDrop );
					m_pTargetWnd->GetItemRect( pItem, &rcItem );
					m_pTargetWnd->ClientToScreen( &rcItem );
					m_pTargetWnd->GetClientRect( &rcClient );
					m_pTargetWnd->ClientToScreen( &rcClient );
					m_pDropWnd->Show( CPoint( rcClient.left, ( rcItem.top + rcItem.bottom ) / 2 ) );
					break;

				default:
					ASSERT( FALSE );
					break;
				}
			}
			else
			{
				SetCursor( m_hStop );
				m_pDropWnd->Hide();
			}
		}
		else
		{
			if( m_pTargetWnd != NULL )
			{
				m_pTargetWnd->DragLeave( this );
				m_pTargetWnd = NULL;
			}
			SetCursor( m_hStop );
			m_pDropWnd->Hide();
		}
	}

	return TRUE;
}

void CTreeListCtrl::ExchangeItem( CTreeListItem* pItemA, CTreeListItem* pItemB )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::ExchangeItem", __DATE__,__FILE__, __LINE__);
	// exchange itemA and itemB ( the same parent )
	ASSERT( pItemA != NULL && pItemB != NULL );

	if( pItemA == pItemB )
		return;

	if( pItemA->m_pParent != pItemB->m_pParent )
		return;

	CTreeListItem* pParentA;
	CTreeListItem* pParentB;

	pParentA = pItemA->m_pParent;
	pParentB = pItemB->m_pParent;

	if( pParentA->m_pChild == pItemA )
	{
		pParentA->m_pChild = pItemB;
	}
	else if( pParentB->m_pChild == pItemB )
	{
		pParentB->m_pChild = pItemA;
	}

	if( pItemA->m_pNext == pItemB )
	{
		pItemA->m_pNext = pItemB->m_pNext;
		pItemB->m_pPrev = pItemA->m_pPrev;

		pItemA->m_pPrev->m_pNext = pItemB;
		pItemB->m_pNext->m_pPrev = pItemA;
		
		pItemA->m_pPrev = pItemB;
		pItemB->m_pNext = pItemA;
	}
	else if( pItemA->m_pPrev == pItemB )
	{
		pItemB->m_pNext	= pItemA->m_pNext;
		pItemA->m_pPrev = pItemB->m_pPrev;

		pItemB->m_pPrev->m_pNext = pItemA;
		pItemA->m_pNext->m_pPrev = pItemB;

		pItemB->m_pPrev = pItemA;
		pItemA->m_pNext	= pItemB;
	}
	else
	{
		CTreeListItem* pPrevA;
		CTreeListItem* pPrevB;

		pPrevA	= pItemA->m_pPrev;
		pPrevB	= pItemB->m_pPrev;

		pItemA->m_pPrev	= pPrevB;
		pItemB->m_pPrev = pPrevA;

		if( pPrevA != NULL )
			pPrevA->m_pNext	= pItemB;

		if( pPrevB != NULL )
			pPrevB->m_pNext	= pItemA;

		CTreeListItem* pNextA;
		CTreeListItem* pNextB;

		pNextA	= pItemA->m_pNext;
		pNextB	= pItemB->m_pNext;

		pItemA->m_pNext	= pNextB;
		pItemB->m_pNext	= pNextA;

		if( pNextA != NULL )
			pNextA->m_pPrev	= pItemB;

		if( pNextB != NULL )
			pNextB->m_pPrev = pItemA;
	}

	Invalidate();

	return;
}

void CTreeListCtrl::MoveAfterItem( CTreeListItem* pItem, CTreeListItem* pAfter )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::MoveAfterItem", __DATE__,__FILE__, __LINE__);
	// move pItemA after pAfter ( the same parent )
	ASSERT( pItem != NULL );

	CTreeListItem* pParent;
	CTreeListItem* pFirst;
	CTreeListItem* pPrev;
	CTreeListItem* pNext;
		 
	pParent = pItem->m_pParent;
	pFirst = pParent->m_pChild;
	pPrev = pItem->m_pPrev;
	pNext = pItem->m_pNext;

	if( pAfter == NULL )
	{
		if( pParent->m_pChild == pItem )
			return;
	}
	else
	{
		if( pItem == pAfter )
			return;
	}

	if( pParent->m_pChild == pItem )
		pParent->m_pChild = pItem->m_pNext;

	if( pPrev != NULL )
		pPrev->m_pNext = pItem->m_pNext;

	if( pNext != NULL )
		pNext->m_pPrev = pItem->m_pPrev;

	if( pAfter == NULL )
	{
		pParent->m_pChild = pItem;

		pItem->m_pPrev = pFirst->m_pPrev;

		pItem->m_pNext = pFirst;
		pFirst->m_pPrev = pItem;
	}
	else
	{
		ASSERT( pParent == pAfter->m_pParent );
		
		pPrev = pAfter;
		pNext = pAfter->m_pNext;

		pItem->m_pPrev = pPrev;
		pPrev->m_pNext = pItem;

		pItem->m_pNext = pNext;
		if( pNext != NULL )
			pNext->m_pPrev = pItem;
	}

	Invalidate();

	return;
}

BOOL CTreeListCtrl::DragEnter( CTreeListCtrl* pTreeListCtrl )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DragEnter", __DATE__,__FILE__, __LINE__);
	// drag enter
	if( !(GetStyle()&TLC_DROP) )
		return FALSE;

	LRESULT lResult = NotifyDrop( TLN_DRAGENTER, pTreeListCtrl, NULL );
	if( lResult < 0 )
		return FALSE;

	return TRUE;
}

BOOL CTreeListCtrl::DragLeave( CTreeListCtrl* pTreeListCtrl )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DragLeave", __DATE__,__FILE__, __LINE__);
	// drag leave
	NotifyDrop( TLN_DRAGLEAVE, pTreeListCtrl, NULL );
	SetTargetItem( NULL );
	SetHoverItem( NULL );
	return TRUE;
}

int CTreeListCtrl::DragOver( CTreeListCtrl* pTreeListCtrl, CPoint point, CTreeListItem** pp )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::DragOver", __DATE__,__FILE__, __LINE__);
	// drag over
	ASSERT( pTreeListCtrl != NULL );
	ASSERT_VALID( pTreeListCtrl );

	LRESULT lResult;
	CTreeListItem* pItem;
	CTreeListItem* pSelItem;

	ScreenToClient( &point );
	pItem = HitTest( point );

	if( pp != NULL )
		*pp = pItem;

	int nDropEffect;

	if( pTreeListCtrl != this )
	{
		if( GetStyle()&TLC_DROP )
		{
			if( pItem != NULL )
			{
				lResult = NotifyDrop( TLN_DRAGOVER, pTreeListCtrl, pItem );
				if( lResult >= 0 )
					nDropEffect = DROP_ITEM;
				else
					nDropEffect = DROP_NONE;
			}
			else
			{
				if( GetStyle()&TLC_DROPROOT )
					nDropEffect = DROP_ROOT;
				else
					nDropEffect = DROP_NONE;
			}
		}
		else
		{
			nDropEffect = DROP_NONE;
		}
	}
	else
	{
		if( GetStyle()&TLC_DROP && GetStyle()&TLC_DROPTHIS )
		{
			if( pItem != NULL )
			{
				if( !IsInSelectsTree( pItem ) )
				{
					lResult = NotifyDrop( TLN_DRAGOVER, pTreeListCtrl, pItem );
					if( lResult >= 0 )
						nDropEffect = DROP_ITEM;
					else
						nDropEffect = DROP_NONE;
				}
				else
				{
					nDropEffect = DROP_SELF;
				}
			}
			else
			{
				if( GetStyle()&TLC_DROPROOT )
				{
					pSelItem = (CTreeListItem*)m_arSelects[0];
					if( pSelItem->m_pParent != m_pRootItem )
						nDropEffect = DROP_ROOT;
					else
						nDropEffect = DROP_SELF;
				}
				else
				{
					nDropEffect = DROP_SELF;
				}
			}
		}
		else
		{
			nDropEffect = DROP_SELF;
		}
	}

	switch( nDropEffect )
	{
	case DROP_NONE:
		SetTargetItem( NULL );
		SetHoverItem( pItem );
		if( GetStyle()&TLC_TREELIST )
		{
			SetTimer( 2, 100, NULL );
			SetTimer( 1, 500, NULL );
		}
		break;

	case DROP_SELF:
	case DROP_ROOT:
		SetTargetItem( NULL );
		SetHoverItem( NULL );
		break;

	case DROP_ITEM:
		SetTargetItem( pItem );
		SetHoverItem( pItem );
		if( GetStyle()&TLC_TREELIST )
		{
			SetTimer( 2, 200, NULL );
			SetTimer( 1,1000, NULL );
		}
		break;

	default:
		ASSERT( FALSE );
		break;
	}

	return nDropEffect;
}

int CTreeListCtrl::Drop( CTreeListCtrl* pTreeListCtrl, CPoint point )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::Drop", __DATE__,__FILE__, __LINE__);
	int		nDrag;
	LRESULT lResult;

	CTreeListItem* pItem;

	nDrag = DragOver( pTreeListCtrl, point, &pItem );
	DragLeave( pTreeListCtrl );
	switch( nDrag )
	{
	case DROP_NONE:
	case DROP_SELF:
		break;

	case DROP_ITEM:
	case DROP_ROOT:
		lResult = NotifyDrop( TLN_DROP, pTreeListCtrl, pItem );
		break;

	default:
		ASSERT( FALSE );
		break;
	}
	return 0;
}

BOOL CTreeListCtrl::IsInSelectsTree( CTreeListItem* pItem )
{
	IMPLEMENT_LOGGING("CTreeListCtrl::IsInSelectsTree", __DATE__,__FILE__, __LINE__);
	ASSERT( pItem != NULL );
	ASSERT( m_arSelects.GetSize() > 0 );
	
	CTreeListItem* pSelItem;

	pSelItem	= (CTreeListItem*)m_arSelects[0];

	while( pItem->m_nLevel > pSelItem->m_nLevel )
	{
		pItem = pItem->m_pParent;
	}
	
	BOOL bFound = FALSE;
	for( int iSel = 0; iSel < m_arSelects.GetSize(); iSel++ )
	{
		pSelItem = (CTreeListItem*)m_arSelects[iSel];
		if( pItem == pSelItem )
		{
			bFound = TRUE;
			break;
		}
	}

	return bFound;
}

void CTreeListCtrl::OnTimer(UINT nIDEvent) 
{
	IMPLEMENT_LOGGING("CTreeListCtrl::OnTimer", __DATE__,__FILE__, __LINE__);
	CPoint point;
	CTreeListItem* pItem;
	int nItem;
	int nPosition;
	int nPage;

	switch( nIDEvent )
	{
	case 1:
		::GetCursorPos( &point );
		ScreenToClient( &point );
		pItem = HitTest( point  );

		if( pItem == m_pHoverItem )
		{
			Expand( pItem, TLE_EXPAND );
		}

		KillTimer( 1 );

		break;

	case 2:
		nPosition	= GetScrollPos( SB_VERT );
		nPage		= GetCountPerPage(); 

		::GetCursorPos( &point );
		ScreenToClient( &point );
		pItem = HitTest( point );

		nItem = GetTreeListItem( pItem );
		if( nItem == 0 )
		{
			KillTimer( 2 );
		}
		else if( nItem == nPosition )
		{
			KillTimer( 1 );
			SendMessage( WM_VSCROLL, (WPARAM)SB_LINEUP );
			pItem = HitTest( point );
			SetHoverItem( pItem );

			if( m_pDropWnd != NULL )
				m_pDropWnd->Hide();
		}
		else if( nItem >= nPosition + nPage - 1 )
		{
			KillTimer( 1 );
			SendMessage( WM_VSCROLL, (WPARAM)SB_LINEDOWN );
			pItem = HitTest( point );
			SetHoverItem( pItem );

			if( m_pDropWnd != NULL )
				m_pDropWnd->Hide();
		}
		else
		{
			KillTimer( 2 );
		}

		break;

	default:
		break;
	}
	
	CWnd::OnTimer(nIDEvent);
}

void CTreeListCtrl::OnCancelMode() 
{
	IMPLEMENT_LOGGING("CTreeListCtrl::OnCancelMode", __DATE__,__FILE__, __LINE__);
	// cancel internal event
	if( GetState( TLS_DRAG ) )
	{
		ASSERT( GetState( TLS_CAPTURE ) );
		EndDraging();
	}

	if( GetState( TLS_CAPTURE ) )
	{
		ReleaseCapture();
		SetState( 0, TLS_CAPTURE );
	}

	ASSERT( !GetState( TLS_CAPTURE ) );
	CWnd::OnCancelMode();
}



//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTreeListDC::CTreeListDC( CDC* pDC ) :
	m_pOldBitmap( NULL ),
	m_pDC( NULL ),
	m_rcRect( 0, 0, 0, 0 ),
	m_bPrint( FALSE )
{
	IMPLEMENT_LOGGING("m_bPrint", __DATE__,__FILE__, __LINE__);
	ASSERT( pDC != NULL );

	m_pDC		= pDC;
	m_bPrint	= pDC->IsPrinting();

	if( m_bPrint )
	{
		m_bPrinting	= pDC->m_bPrinting;
		m_hDC		= pDC->m_hDC;
		m_hAttribDC	= pDC->m_hAttribDC;
	}
	else
	{
		pDC->GetClipBox( &m_rcRect );
		CreateCompatibleDC( pDC );
		m_Bitmap.CreateCompatibleBitmap( pDC, m_rcRect.Width(), m_rcRect.Height() );
		m_pOldBitmap = SelectObject( &m_Bitmap );
		SetWindowOrg( m_rcRect.left, m_rcRect.top );
	}
}

CTreeListDC::~CTreeListDC()
{
	IMPLEMENT_LOGGING("CTreeListDC::~CTreeListDC", __DATE__,__FILE__, __LINE__);
	ASSERT_VALID( m_pDC );

	if( m_bPrint )
	{
		m_hDC		= NULL;
		m_hAttribDC = NULL;
	}
	else
	{
		m_pDC->BitBlt( m_rcRect.left, m_rcRect.top, m_rcRect.Width(), m_rcRect.Height(), 
					   this, m_rcRect.left, m_rcRect.top, SRCCOPY );
		SelectObject( m_pOldBitmap );
	}
}



//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTreeListItem::CTreeListItem( int nSubItem ) :
	m_pParent( NULL ),
	m_pChild( NULL ),
	m_pPrev( NULL ),
	m_pNext( NULL ),
	m_dwState( TLIS_SHOWCHECKBOX | TLIS_SHOWLOCKBOX | TLIS_CHECKED_NULL | TLIS_LOCKED_NULL ),
	m_dwData( 0 ),
//	m_pImageList( NULL ),
	m_nLevel( 0 ),
	m_nChild( 1 ),
	m_nVisibleChild( 1 )
{
	IMPLEMENT_LOGGING("m_nVisibleChild", __DATE__,__FILE__, __LINE__);
	for( int iSubItem = 0; iSubItem < nSubItem; iSubItem++ )
	{
		CString* pSubItem = new CString;
		m_arSubItems.Add( pSubItem );
	}
}

CTreeListItem::~CTreeListItem()
{
	IMPLEMENT_LOGGING("CTreeListItem::~CTreeListItem", __DATE__,__FILE__, __LINE__);
	while( m_arSubItems.GetSize() > 0 )
	{
		CString* pSubItem;
		pSubItem = (CString*)m_arSubItems[m_arSubItems.GetUpperBound()];
		m_arSubItems.RemoveAt( m_arSubItems.GetUpperBound() );
		delete pSubItem;
	}
}

void CTreeListItem::Expand()
{
	IMPLEMENT_LOGGING("CTreeListItem::Expand", __DATE__,__FILE__, __LINE__);
	SetState( TLIS_EXPANDED, 0 );
}

void CTreeListItem::Collapse()
{
	IMPLEMENT_LOGGING("CTreeListItem::Collapse", __DATE__,__FILE__, __LINE__);
	SetState( 0, TLIS_EXPANDED );
}

DWORD CTreeListItem::GetState( DWORD dwStateMask )
{
	IMPLEMENT_LOGGING("CTreeListItem::GetState", __DATE__,__FILE__, __LINE__);
	return m_dwState&dwStateMask;
}

void CTreeListItem::SetState( DWORD dwAddStatus, DWORD dwRemoveStatus )
{
	IMPLEMENT_LOGGING("CTreeListItem::SetState", __DATE__,__FILE__, __LINE__);
	if( dwAddStatus&TLIS_EXPANDED )
		dwAddStatus |= TLIS_EXPANDEDONCE;

	if( dwRemoveStatus&TLIS_EXPANDEDONCE )
		dwRemoveStatus |= TLIS_EXPANDED;

	m_dwState |= dwAddStatus;
	m_dwState &=~dwRemoveStatus;
	return;
}

void CTreeListItem::GetImage( int& nImage, int& nSelectedImage, int& nExpandImage, int& nExpandSelectedImage )
{
	IMPLEMENT_LOGGING("CTreeListItem::GetImage", __DATE__,__FILE__, __LINE__);
	nImage					= m_nImage;
	nSelectedImage			= m_nSelectedImage;
	nExpandImage			= nExpandImage;
	nExpandSelectedImage	= nExpandSelectedImage;
}

void CTreeListItem::SetImage( int nImage, int nSelectedImage, int nExpandImage, int nExpandSelectedImage )
{
	IMPLEMENT_LOGGING("CTreeListItem::SetImage", __DATE__,__FILE__, __LINE__);
	m_nImage				= nImage;
	m_nSelectedImage		= nSelectedImage;
	m_nExpandImage			= nExpandImage;
	m_nExpandSelectedImage	= nExpandSelectedImage;
}

LPCTSTR CTreeListItem::GetText( int nIndex )
{
	IMPLEMENT_LOGGING("CTreeListItem::GetText", __DATE__,__FILE__, __LINE__);
	if( nIndex >= m_arSubItems.GetSize() )
	{
		return NULL;
	}

	CString* pString = (CString*)m_arSubItems[nIndex];
	return *pString;
}

BOOL CTreeListItem::SetText( LPCTSTR lpszItem, int nIndex )
{
	IMPLEMENT_LOGGING("CTreeListItem::SetText", __DATE__,__FILE__, __LINE__);
	if( nIndex >= m_arSubItems.GetSize() )
		return FALSE;

	CString* pString = (CString*)m_arSubItems[nIndex];
	*pString = lpszItem;

	return TRUE;
}

DWORD CTreeListItem::GetData()
{
	IMPLEMENT_LOGGING("CTreeListItem::GetData", __DATE__,__FILE__, __LINE__);
	return m_dwData;
}

void CTreeListItem::SetData( DWORD dwData )
{
	IMPLEMENT_LOGGING("CTreeListItem::SetData", __DATE__,__FILE__, __LINE__);
	m_dwData = dwData;
}

BOOL CTreeListItem::IsShowTreeImage()
{
	IMPLEMENT_LOGGING("CTreeListItem::IsShowTreeImage", __DATE__,__FILE__, __LINE__);
	if( m_dwState&TLIS_SHOWTREEIMAGE )
		return TRUE;
	else
		return FALSE;
}

void CTreeListItem::ShowTreeImage( BOOL bShow )
{
	IMPLEMENT_LOGGING("CTreeListItem::ShowTreeImage", __DATE__,__FILE__, __LINE__);
	if( bShow )
		m_dwState |= TLIS_SHOWTREEIMAGE;
	else
		m_dwState &=~TLIS_SHOWTREEIMAGE;
}

BOOL CTreeListItem::IsShowCheckBox()
{
	IMPLEMENT_LOGGING("CTreeListItem::IsShowCheckBox", __DATE__,__FILE__, __LINE__);
	if( m_dwState&TLIS_SHOWCHECKBOX )
		return TRUE;
	else
		return FALSE;
}

void CTreeListItem::ShowCheckBox( BOOL bShow )
{
	IMPLEMENT_LOGGING("CTreeListItem::ShowCheckBox", __DATE__,__FILE__, __LINE__);
	if( bShow )
		m_dwState |= TLIS_SHOWCHECKBOX;
	else
		m_dwState &=~TLIS_SHOWCHECKBOX;
}

BOOL CTreeListItem::IsShowLockBox()
{
	IMPLEMENT_LOGGING("CTreeListItem::IsShowLockBox", __DATE__,__FILE__, __LINE__);
	if( m_dwState&TLIS_SHOWLOCKBOX )
		return TRUE;
	else
		return FALSE;
}

void CTreeListItem::ShowLockBox( BOOL bShow )
{
	IMPLEMENT_LOGGING("CTreeListItem::ShowLockBox", __DATE__,__FILE__, __LINE__);
	if( bShow )
		m_dwState |= TLIS_SHOWLOCKBOX;
	else
		m_dwState &=~TLIS_SHOWLOCKBOX;
}

BOOL CTreeListItem::GetSelected()
{
	IMPLEMENT_LOGGING("CTreeListItem::GetSelected", __DATE__,__FILE__, __LINE__);
	if( GetState()&TLIS_SELECTED )
		return TRUE;
	else
		return FALSE;
}

void CTreeListItem::SetSelected( BOOL bSelected )
{
	IMPLEMENT_LOGGING("CTreeListItem::SetSelected", __DATE__,__FILE__, __LINE__);
	if( bSelected )
		SetState( TLIS_SELECTED, 0 );
	else
		SetState( 0, TLIS_SELECTED );
}

int CTreeListItem::GetCheck()
{
	IMPLEMENT_LOGGING("CTreeListItem::GetCheck", __DATE__,__FILE__, __LINE__);
	ASSERT( GetState()&TLIS_CHECKED || GetState()&TLIS_CHECKED_NULL );
	
	if( ( GetState()&TLIS_CHECKEDMASK) == (DWORD)TLIS_CHECKEDPART )
		return 2;
	
	if( GetState()&TLIS_CHECKED )
		return 1;

	return 0;
}

void CTreeListItem::SetCheck( BOOL bCheck )
{
	IMPLEMENT_LOGGING("CTreeListItem::SetCheck", __DATE__,__FILE__, __LINE__);
	if( bCheck )
		SetState( TLIS_CHECKED, TLIS_CHECKED_NULL );
	else
		SetState( TLIS_CHECKED_NULL, TLIS_CHECKED );
}

int CTreeListItem::GetLock()
{
	IMPLEMENT_LOGGING("CTreeListItem::GetLock", __DATE__,__FILE__, __LINE__);
	ASSERT( GetState()&TLIS_LOCKED || GetState()&TLIS_LOCKED_NULL );

	if( ( GetState()&TLIS_LOCKEDMASK) == (DWORD)TLIS_LOCKEDPART )
		return 2;
	
	if( GetState()&TLIS_LOCKED )
		return 1;

	return 0;
}

void CTreeListItem::SetLock( BOOL bLock )
{
	IMPLEMENT_LOGGING("CTreeListItem::SetLock", __DATE__,__FILE__, __LINE__);
	if( bLock )
		SetState( TLIS_LOCKED, TLIS_LOCKED_NULL );
	else
		SetState( TLIS_LOCKED_NULL, TLIS_LOCKED );
}
/*
void CTreeListItem::SetImageList( CImageList* pImageList )
{
	IMPLEMENT_LOGGING("CTreeListItem::SetImageList", __DATE__,__FILE__, __LINE__);
	ASSERT( pImageList != NULL );
	m_pImageList = pImageList;
}

void CTreeListItem::SetImage( int nIndex )
{
	IMPLEMENT_LOGGING("CTreeListItem::SetImage", __DATE__,__FILE__, __LINE__);
	return;
}

int CTreeListItem::GetImage()
{
	IMPLEMENT_LOGGING("CTreeListItem::GetImage", __DATE__,__FILE__, __LINE__);
	return 0;
}
*/







/////////////////////////////////////////////////////////////////////////////
// CTreeListStaticCtrl

CTreeListStaticCtrl::CTreeListStaticCtrl()
{
	IMPLEMENT_LOGGING("CTreeListStaticCtrl::CTreeListStaticCtrl", __DATE__,__FILE__, __LINE__);
}

CTreeListStaticCtrl::~CTreeListStaticCtrl()
{
	IMPLEMENT_LOGGING("CTreeListStaticCtrl::~CTreeListStaticCtrl", __DATE__,__FILE__, __LINE__);
}


BEGIN_MESSAGE_MAP(CTreeListStaticCtrl, CStatic)
	//{{AFX_MSG_MAP(CTreeListStaticCtrl)
	ON_WM_CREATE()
	ON_WM_KILLFOCUS()
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTreeListStaticCtrl message handlers

int CTreeListStaticCtrl::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	IMPLEMENT_LOGGING("CTreeListStaticCtrl::OnCreate", __DATE__,__FILE__, __LINE__);
	if (CStatic::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	m_pTreeListCtrl = (CTreeListCtrl*)GetParent();
	
	return 0;
}

void CTreeListStaticCtrl::OnKillFocus(CWnd* pNewWnd) 
{
	IMPLEMENT_LOGGING("CTreeListStaticCtrl::OnKillFocus", __DATE__,__FILE__, __LINE__);
	CStatic::OnKillFocus(pNewWnd);
	
	m_pTreeListCtrl->SetCtrlFocus( pNewWnd, FALSE );
}

void CTreeListStaticCtrl::OnPaint() 
{
	IMPLEMENT_LOGGING("CTreeListStaticCtrl::OnPaint", __DATE__,__FILE__, __LINE__);
	CPaintDC dc(this); // device context for painting

	CString strText;
	GetWindowText( strText );
	
	CRect rcClient;
	GetClientRect( rcClient );
	dc.FillSolidRect( rcClient, GetSysColor( COLOR_BTNFACE ) );
	
	rcClient.DeflateRect( 3, 1, 4, 0 );
	if( !rcClient.IsRectEmpty() )
	{
		UINT nFormat = DT_VCENTER | DT_SINGLELINE;
		if( GetStyle()&SS_LEFT )
			nFormat |= DT_LEFT;
		else if( GetStyle()&SS_RIGHT )
			nFormat |= DT_RIGHT;
		else if( GetStyle()&SS_CENTER )
			nFormat |= DT_CENTER;
		else
			nFormat |= DT_LEFT;

		CFont* pOldFont = dc.SelectObject( GetFont() );
		int nBkMode = dc.SetBkMode( TRANSPARENT );
		dc.DrawText( strText, rcClient, nFormat );
		dc.SelectObject( pOldFont );
		dc.SetBkMode( nBkMode );
	}

}

BOOL CTreeListStaticCtrl::PreTranslateMessage(MSG* pMsg) 
{
	IMPLEMENT_LOGGING("CTreeListStaticCtrl::PreTranslateMessage", __DATE__,__FILE__, __LINE__);
	switch( pMsg->message )
	{
	case WM_KEYDOWN:
		{
			if( PreTranslateKeyDownMessage( pMsg ) )
				return TRUE;
		}
		break;

	default:
		break;
	}
	
	return CStatic::PreTranslateMessage(pMsg);
}

BOOL CTreeListStaticCtrl::PreTranslateKeyDownMessage( MSG* pMsg )
{
	IMPLEMENT_LOGGING("CTreeListStaticCtrl::PreTranslateKeyDownMessage", __DATE__,__FILE__, __LINE__);
	switch( pMsg->wParam )
	{
	case VK_UP:
	case VK_DOWN:
	case VK_LEFT:
	case VK_RIGHT:
		return TRUE;
		break;

	default:
		break;
	}

	return FALSE;
}
















BOOL InitLayeredWindows()
{
	IMPLEMENT_LOGGING("InitLayeredWindows", __DATE__,__FILE__, __LINE__);
	if( g_lpfnUpdateLayeredWindow == NULL || g_lpfnSetLayeredWindowAttributes == NULL )
	{
		HMODULE hUser32 = GetModuleHandle(_T("USER32.DLL"));

		g_lpfnUpdateLayeredWindow =	(lpfnUpdateLayeredWindow)GetProcAddress( hUser32, _T("UpdateLayeredWindow") );
		g_lpfnSetLayeredWindowAttributes = (lpfnSetLayeredWindowAttributes)GetProcAddress( hUser32, _T("SetLayeredWindowAttributes") );

		if( g_lpfnUpdateLayeredWindow == NULL || g_lpfnSetLayeredWindowAttributes == NULL )
			return FALSE;
	}

	return TRUE;
}
/////////////////////////////////////////////////////////////////////////////
// CTreeListTipCtrl

CTreeListTipCtrl::CTreeListTipCtrl() : 
	m_dwFadeStep( 10L ), 
	m_dwFadeInTime( 500L ), 
	m_dwFadeOutTime( 200L ),
	m_bLayeredWindows( FALSE ),
	m_nAlpha( 0 ),
	m_rcItem( 0, 0, 0, 0 ),
	m_rcTip( 0, 0, 0, 0 ),
	m_uFormat( 0 ),
	m_pTreeListCtrl( NULL )
{
	IMPLEMENT_LOGGING("m_pTreeListCtrl", __DATE__,__FILE__, __LINE__);
	// construction
	WNDCLASS wndcls;
	HINSTANCE hInst = AfxGetInstanceHandle();

	if( ::GetClassInfo( hInst, TREELISTTIPCTRL_CLASSNAME, &wndcls ) == FALSE )
	{
		wndcls.style =  CS_DBLCLKS ;
		wndcls.lpfnWndProc		= ::DefWindowProc;
		wndcls.cbClsExtra		= 0;
		wndcls.cbWndExtra		= 0;
		wndcls.hInstance		= hInst;
		wndcls.hIcon			= NULL;
		wndcls.hCursor			= LoadCursor(hInst, IDC_ARROW);
		wndcls.hbrBackground	= (HBRUSH)(COLOR_INFOBK+1);
		wndcls.lpszMenuName		= NULL;
		wndcls.lpszClassName	= TREELISTTIPCTRL_CLASSNAME;

		if( !AfxRegisterClass(&wndcls) )
			AfxThrowResourceException();
	}

	// for layer windows
	m_bLayeredWindows = InitLayeredWindows();
}

CTreeListTipCtrl::~CTreeListTipCtrl()
{
	IMPLEMENT_LOGGING("CTreeListTipCtrl::~CTreeListTipCtrl", __DATE__,__FILE__, __LINE__);
	// deconstruction
	m_Font.DeleteObject();
}

BEGIN_MESSAGE_MAP(CTreeListTipCtrl, CWnd)
	//{{AFX_MSG_MAP(CTreeListTipCtrl)
	ON_WM_PAINT()
	ON_WM_TIMER()
	ON_WM_MOUSEACTIVATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTreeListTipCtrl message handlers
BOOL CTreeListTipCtrl::Create( CTreeListCtrl* pTreeListCtrl )
{
	IMPLEMENT_LOGGING("CTreeListTipCtrl::Create", __DATE__,__FILE__, __LINE__);
	// create tip
	ASSERT_VALID( pTreeListCtrl );

	m_pTreeListCtrl = pTreeListCtrl;

	DWORD dwExStyle = WS_EX_TOPMOST | WS_EX_TOOLWINDOW | WS_EX_TRANSPARENT;
	DWORD dwStyle	= WS_BORDER | WS_POPUP;
	CRect rcRect ( 0, 0, 0, 0 );

	if( m_bLayeredWindows )
		dwExStyle |= WS_EX_LAYERED;

	return CreateEx( dwExStyle, TREELISTTIPCTRL_CLASSNAME, NULL, dwStyle, rcRect, pTreeListCtrl, 0, NULL ); 
}

BOOL CTreeListTipCtrl::Show( CRect rcItem, LPCTSTR lpszText, UINT uFormat )
{
	IMPLEMENT_LOGGING("CTreeListTipCtrl::Show", __DATE__,__FILE__, __LINE__);
	// show tip
	BOOL bResult = FALSE;

	ASSERT(::IsWindow(GetSafeHwnd()));

	if(	rcItem.IsRectEmpty() )// || GetFocus() == NULL ) 
	{
		Hide();
		return FALSE;
	}

	if( m_rcItem != rcItem )// || !IsWindowVisible() )
	{
		m_rcItem	= rcItem;

		CClientDC dc(this);
		CFont *pOldFont = dc.SelectObject( &m_Font );
		CSize size = dc.GetTextExtent(lpszText, _tcslen(lpszText));
		dc.SelectObject(pOldFont);

		if( size.cx + 5 > rcItem.Width() )
		{
			m_strText = lpszText;
			m_uFormat = uFormat;

			m_rcTip		= rcItem;
			m_pTreeListCtrl->ClientToScreen(m_rcTip);
				
			m_rcTip.left -= 1;
			m_rcTip.top -= 1;
			if( uFormat&DT_RIGHT )
			{
				m_rcTip.left = m_rcTip.right - size.cx - 6;
			}
			else if( uFormat&DT_CENTER )
			{
				int nFix = ( size.cx - m_rcItem.Width() ) / 2;
				m_rcTip.left = m_rcTip.left - nFix - 3;
				m_rcTip.right = m_rcTip.right + nFix + 2;
			}
			else // if( uFormat&DT_LEFT )
			{
				m_rcTip.right = m_rcTip.left + size.cx + 5;
			}

			Invalidate();
			UpdateWindow();

			KillTimer( 2 );
			SetWindowPos( &wndTop, m_rcTip.left, m_rcTip.top, m_rcTip.Width(), m_rcTip.Height(),
				SWP_NOOWNERZORDER|SWP_SHOWWINDOW|SWP_NOACTIVATE	);

			if(m_bLayeredWindows)
			{
				if(m_nAlpha < 255)
				{
					KillTimer( 2 );
					SetTimer( 1, m_dwFadeInTime/(255/m_dwFadeStep), NULL );
				}
				g_lpfnSetLayeredWindowAttributes( GetSafeHwnd(), RGB(0xFF, 0, 0xFF), m_nAlpha, ULW_ALPHA );
			}

			bResult = TRUE;
		}
		else
		{
			Hide();
			bResult = FALSE;
		}
	}
	else
	{
		bResult = TRUE;
	}

	return bResult;

}

void CTreeListTipCtrl::Hide()
{
	IMPLEMENT_LOGGING("CTreeListTipCtrl::Hide", __DATE__,__FILE__, __LINE__);
	// hide tip
	ASSERT(::IsWindow(GetSafeHwnd()));

	if ( IsWindowVisible() )
	{
		if( m_bLayeredWindows )
		{
			KillTimer( 1 );
			SetTimer( 2, m_dwFadeOutTime/(255/m_dwFadeStep), NULL );
		}
		else
		{
			ShowWindow( SW_HIDE );
		}
	}
}

BOOL CTreeListTipCtrl::PreTranslateMessage(MSG* pMsg) 
{
	IMPLEMENT_LOGGING("CTreeListTipCtrl::PreTranslateMessage", __DATE__,__FILE__, __LINE__);
	// post message of mouse and key to parent window
	DWORD	dwTick = 0;
	BOOL	bDoubleClick = FALSE;

/*
	CWnd*	pWnd;
	int		nHitTest;
	switch( pMsg->message )
	{
	case WM_LBUTTONDOWN:
	case WM_LBUTTONUP:
	case WM_LBUTTONDBLCLK:

	case WM_RBUTTONDOWN:
	case WM_RBUTTONUP:
	case WM_RBUTTONDBLCLK:

	case WM_MBUTTONDOWN:
	case WM_MBUTTONUP:
	case WM_MBUTTONDBLCLK:

	case WM_MOUSEMOVE:

	case WM_NCLBUTTONDOWN:
	case WM_NCLBUTTONUP:
	case WM_NCLBUTTONDBLCLK:

	case WM_NCRBUTTONDOWN:
	case WM_NCRBUTTONUP:
	case WM_NCRBUTTONDBLCLK:

	case WM_NCMBUTTONDOWN:
	case WM_NCMBUTTONUP:
	case WM_NCMBUTTONDBLCLK:

	case WM_NCMOUSEMOVE:

		POINTS points = MAKEPOINTS( pMsg->lParam );
		POINT pt;
		pt.x = points.x;
		pt.y = points.y;

		ClientToScreen( &pt );
		pWnd = WindowFromPoint( pt );

		if( pWnd == this )
			pWnd = m_pTreeListCtrl;

		if( pWnd != NULL )
		{
			nHitTest = (int)pWnd->SendMessage( WM_NCHITTEST, 0, MAKELONG( pt.x, pt.y ) );

			if( nHitTest == HTCLIENT )
			{
				switch( pMsg->message )
				{
				case WM_NCLBUTTONDOWN:		pMsg->message = WM_LBUTTONDOWN;		break;
				case WM_NCLBUTTONUP:		pMsg->message = WM_LBUTTONUP;		break;
				case WM_NCLBUTTONDBLCLK:	pMsg->message = WM_LBUTTONDBLCLK;	break;

				case WM_NCRBUTTONDOWN:		pMsg->message = WM_RBUTTONDOWN;		break;
				case WM_NCRBUTTONUP:		pMsg->message = WM_RBUTTONUP;		break;
				case WM_NCRBUTTONDBLCLK:	pMsg->message = WM_RBUTTONDBLCLK;	break;

				case WM_NCMBUTTONDOWN:		pMsg->message = WM_MBUTTONDOWN;		break;
				case WM_NCMBUTTONUP:		pMsg->message = WM_MBUTTONUP;		break;
				case WM_NCMBUTTONDBLCLK:	pMsg->message = WM_MBUTTONDBLCLK;	break;

				case WM_NCMOUSEMOVE:		pMsg->message = WM_MOUSEMOVE;		break;

				default:	break;
				}
			}
			else
			{
				switch( pMsg->message )
				{
				case WM_LBUTTONDOWN:		pMsg->message = WM_NCLBUTTONDOWN;	break;
				case WM_LBUTTONUP:			pMsg->message = WM_NCLBUTTONUP;		break;
				case WM_LBUTTONDBLCLK:		pMsg->message = WM_NCLBUTTONDBLCLK;	break;

				case WM_RBUTTONDOWN:		pMsg->message = WM_NCRBUTTONDOWN;	break;
				case WM_RBUTTONUP:			pMsg->message = WM_NCRBUTTONUP;		break;
				case WM_RBUTTONDBLCLK:		pMsg->message = WM_NCRBUTTONDBLCLK;	break;

				case WM_MBUTTONDOWN:		pMsg->message = WM_NCMBUTTONDOWN;	break;
				case WM_MBUTTONUP:			pMsg->message = WM_NCMBUTTONUP;		break;
				case WM_MBUTTONDBLCLK:		pMsg->message = WM_NCMBUTTONDBLCLK;	break;
	
				case WM_MOUSEMOVE:			pMsg->message = WM_NCMOUSEMOVE;		break;
	
				default:	break;
				}
			}

			pWnd->ScreenToClient( &pt );
			pMsg->lParam = MAKELONG( pt.x, pt.y );
			pWnd->SendMessage( pMsg->message, pMsg->wParam, pMsg->lParam );
			return TRUE;
		}
		break;

	default:
		;
	}
*/
	return CWnd::PreTranslateMessage(pMsg);
}

//////////////////////////////////////////////////////////////////////////////////////////////////
void CTreeListTipCtrl::OnPaint() 
{
	IMPLEMENT_LOGGING("CTreeListTipCtrl::OnPaint", __DATE__,__FILE__, __LINE__);
	// paint the control
	CPaintDC dc(this); // device context for painting

	CRect rcText;
	rcText.SetRect( 0, 0, m_rcTip.Width(), m_rcTip.Height() );
	dc.FillSolidRect( rcText, 0xF0FFFF );

	CFont* pOldFont = dc.SelectObject( &m_Font );
	int nBkMode = dc.SetBkMode( TRANSPARENT );
	rcText.DeflateRect( 2, 0, 3, 0 );
	dc.DrawText( m_strText, rcText, DT_LEFT | DT_VCENTER | DT_SINGLELINE );
	dc.SetBkMode( nBkMode );
	dc.SelectObject( pOldFont );
}

void CTreeListTipCtrl::OnTimer(UINT nIDEvent) 
{
	IMPLEMENT_LOGGING("CTreeListTipCtrl::OnTimer", __DATE__,__FILE__, __LINE__);
	// fade in/out timer
	switch( nIDEvent )
	{
	case 1:	// fade in
		ASSERT( m_bLayeredWindows == TRUE );

		if( m_nAlpha < 255 )
		{
			Invalidate();
			UpdateWindow();
			m_nAlpha += m_dwFadeStep;
			if( m_nAlpha >= 255 )
			{
				m_nAlpha = 255;
				KillTimer( 1 );
			}
			g_lpfnSetLayeredWindowAttributes( GetSafeHwnd(), 0xFF00FF, m_nAlpha, ULW_ALPHA );
		}
		else
		{
			m_nAlpha = 255;
			KillTimer( 1 );
		}

		break;
	
	case 2: // fade out
		ASSERT( m_bLayeredWindows == TRUE );

		if( m_nAlpha > 0 )
		{
			Invalidate();
			UpdateWindow();
			m_nAlpha -= m_dwFadeStep;
			if( m_nAlpha <= 0 )
			{
				m_nAlpha = 0;
				KillTimer( 2 );
				if( IsWindowVisible() )
					ShowWindow( SW_HIDE );
			}
			g_lpfnSetLayeredWindowAttributes( GetSafeHwnd(), 0xFF00FF, m_nAlpha, ULW_ALPHA );
		}
		else 
		{
			m_nAlpha = 0;
			KillTimer( 2 );
			if( IsWindowVisible() )
				ShowWindow( SW_HIDE );
			g_lpfnSetLayeredWindowAttributes( GetSafeHwnd(), 0xFF00FF, m_nAlpha, ULW_ALPHA );
		}

		break;

	default:
		break;
	}

	CWnd::OnTimer(nIDEvent);
}

int CTreeListTipCtrl::OnMouseActivate(CWnd* pDesktopWnd, UINT nHitTest, UINT message) 
{
	IMPLEMENT_LOGGING("CTreeListTipCtrl::OnMouseActivate", __DATE__,__FILE__, __LINE__);
	// active a window with mouse click
	// * do NOT write into PreTranslateMessage
	// * because MFC call it first.
	return MA_NOACTIVATE;
}

void CTreeListTipCtrl::SetFont()
{
	IMPLEMENT_LOGGING("CTreeListTipCtrl::SetFont", __DATE__,__FILE__, __LINE__);
	// set font with font of parent window
	if( m_pTreeListCtrl == NULL )
		return;

	CFont* pFont = m_pTreeListCtrl->GetFont();
	if( pFont == NULL )
		return;

	LOGFONT lf;
	pFont->GetLogFont(&lf);
	m_Font.DeleteObject();
	m_Font.CreateFontIndirect(&lf);
}

CFont* CTreeListTipCtrl::GetFont()
{
	IMPLEMENT_LOGGING("CTreeListTipCtrl::GetFont", __DATE__,__FILE__, __LINE__);
	// get font
	return &m_Font;
}


CTreeListEditCtrl::CTreeListEditCtrl() : 
	m_pTreeListCtrl( NULL )
{
	IMPLEMENT_LOGGING("m_pTreeListCtrl", __DATE__,__FILE__, __LINE__);
}

CTreeListEditCtrl::~CTreeListEditCtrl()
{
	IMPLEMENT_LOGGING("CTreeListEditCtrl::~CTreeListEditCtrl", __DATE__,__FILE__, __LINE__);
}

BEGIN_MESSAGE_MAP(CTreeListEditCtrl, CEdit)
	//{{AFX_MSG_MAP(CTreeListEditCtrl)
	ON_WM_CREATE()
	ON_WM_KILLFOCUS()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTreeListEditCtrl message handlers
int CTreeListEditCtrl::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	IMPLEMENT_LOGGING("CTreeListEditCtrl::OnCreate", __DATE__,__FILE__, __LINE__);
	if (CEdit::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	m_pTreeListCtrl = (CTreeListCtrl*)GetParent();

	return 0;
}

void CTreeListEditCtrl::OnKillFocus(CWnd* pNewWnd) 
{
	IMPLEMENT_LOGGING("CTreeListEditCtrl::OnKillFocus", __DATE__,__FILE__, __LINE__);
	CEdit::OnKillFocus(pNewWnd);

	m_pTreeListCtrl->SetCtrlFocus( pNewWnd, FALSE );
}










/////////////////////////////////////////////////////////////////////////////
// CTreeListHeaderCtrl

CTreeListHeaderCtrl::CTreeListHeaderCtrl() :
	m_dwStyle( 0 ),
	m_bSubclassFromCreate( FALSE ),
	m_bOffScreenBuffer( TRUE ),
	m_pImageList( NULL ),
	m_pTreeListCtrl( NULL ),
	m_hSize( NULL ),
	m_hSplit( NULL ),
	m_hArrow( NULL ),
	m_bSplit( FALSE ),
	m_bSizing( FALSE ),
	m_bDraging( FALSE ),
	m_nHeaderHeight( 0 ),
	m_pDragWnd( NULL ),
	m_pDropWnd( NULL )
{
	IMPLEMENT_LOGGING("m_pDropWnd", __DATE__,__FILE__, __LINE__);
	// register window class
	WNDCLASS wndclass;
	HINSTANCE hInst = AfxGetInstanceHandle();
	//CTreeListResource	TreeListResource;

	if( !::GetClassInfo(hInst, TREELISTHEADERCTRL_CLASSNAME, &wndclass) )
	{
        wndclass.style			= CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW;
		wndclass.lpfnWndProc	= ::DefWindowProc;
		wndclass.cbClsExtra		= 0;
		wndclass.cbWndExtra		= 0;
		wndclass.hInstance		= hInst;
		wndclass.hIcon			= NULL;
		wndclass.hCursor		=  AfxGetApp()->LoadStandardCursor( IDC_ARROW );
		wndclass.hbrBackground	= (HBRUSH)COLOR_WINDOW;
		wndclass.lpszMenuName	= NULL;
		wndclass.lpszClassName	= TREELISTHEADERCTRL_CLASSNAME;

		if (!AfxRegisterClass(&wndclass))
			AfxThrowResourceException();
	}

	// default color
	m_cr3DBkgnd		= 0x808080;
	m_cr3DFace		= GetSysColor( COLOR_3DFACE );
	m_cr3DLight		= GetSysColor( COLOR_3DHILIGHT );
	m_cr3DShadow	= GetSysColor( COLOR_3DSHADOW );
	m_crText		= GetSysColor( COLOR_BTNTEXT );

	// default font
	m_Font.CreateStockObject( SYSTEM_FIXED_FONT );

	// load cursors
	m_hSize		= AfxGetApp()->LoadCursor( IDC_CURSOR_SIZE );
	m_hSplit	= AfxGetApp()->LoadCursor( IDC_CURSOR_SPLIT );
	m_hArrow	= AfxGetApp()->LoadStandardCursor( IDC_ARROW );
}

CTreeListHeaderCtrl::~CTreeListHeaderCtrl()
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::~CTreeListHeaderCtrl", __DATE__,__FILE__, __LINE__);
	// free font
	if( m_Font.m_hObject != NULL )
		m_Font.DeleteObject();

	// free cursors
	if( m_hSize != NULL )
		DestroyCursor( m_hSize );

	if( m_hSplit != NULL )
		DestroyCursor( m_hSplit );

	if( m_hArrow != NULL )
		DestroyCursor( m_hArrow );
}

BEGIN_MESSAGE_MAP(CTreeListHeaderCtrl, CWnd)
	//{{AFX_MSG_MAP(CTreeListHeaderCtrl)
	ON_WM_ERASEBKGND()
	ON_WM_PAINT()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_CANCELMODE()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
	ON_WM_RBUTTONDBLCLK()
	//}}AFX_MSG_MAP
	ON_MESSAGE(HDM_LAYOUT, OnLayout)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTreeListHeaderCtrl message handlers
BOOL CTreeListHeaderCtrl::Create()
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::Create", __DATE__,__FILE__, __LINE__);
	// create sort image
	if( !m_imgSort.Create( IDB_TREELIST_SORT, 8, 4, 0xFF00FF ) )
		return FALSE;

	// create header image
	if( !m_imgHeader.Create( IDB_TREELIST_HEADER, 16, 4, 0xFF00FF ) )
		return FALSE;

	m_pImageList = &m_imgHeader;

	return TRUE;
}

BOOL CTreeListHeaderCtrl::Create( DWORD dwStyle, const RECT& rect, CTreeListCtrl* pParentWnd, UINT nID )
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::Create", __DATE__,__FILE__, __LINE__);
	// create window
	ASSERT_VALID( pParentWnd );

	m_dwStyle = dwStyle;
	m_pTreeListCtrl = pParentWnd;
	m_bSubclassFromCreate = TRUE;

	dwStyle |= WS_CHILD | WS_VISIBLE | WS_EX_TRANSPARENT;

	if( !CWnd::Create( TREELISTHEADERCTRL_CLASSNAME, NULL, dwStyle, rect, pParentWnd, nID ) )
		return FALSE;

	if( !Create() )
		return FALSE;

	return TRUE;
}

void CTreeListHeaderCtrl::SetHeaderFont()
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::SetHeaderFont", __DATE__,__FILE__, __LINE__);
	// set font
	CWnd* pWnd = GetParent();
	CTreeListCtrl* pTreeListCtrl = (CTreeListCtrl*)pWnd;

	if( pTreeListCtrl != NULL )
	{
		CFont* pFont = pTreeListCtrl->GetFont();
		if( pFont != NULL )
		{
			LOGFONT lf;
			pFont->GetLogFont( &lf );

			m_Font.DeleteObject();
			m_Font.CreateFontIndirect( &lf );
		}
	}
}

CFont* CTreeListHeaderCtrl::GetHeaderFont()
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::GetHeaderFont", __DATE__,__FILE__, __LINE__);
	// get font
	return &m_Font;
}


int CTreeListHeaderCtrl::SetHeaderHeight( int nHeight )
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::SetHeaderHeight", __DATE__,__FILE__, __LINE__);
	// set header height
	int nOldHeight = m_nHeaderHeight;

	if( nHeight == -1 )
	{
		ASSERT( m_Font.m_hObject != NULL );
		
		LOGFONT lf;
		m_Font.GetLogFont( &lf );

		if( lf.lfHeight > 0 )
		{
			m_nHeaderHeight = lf.lfHeight + 6;
		}
		else if( lf.lfHeight == 0 )
		{
			m_nHeaderHeight = DEFAULT_HEIGHT;
		}
		else
		{
			m_nHeaderHeight = -lf.lfHeight + 6;
		}
	}
	else
	{
		m_nHeaderHeight = nHeight;
	}

	return nOldHeight;
}

int CTreeListHeaderCtrl::GetHeaderHeight()
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::GetHeaderHeight", __DATE__,__FILE__, __LINE__);
	// get header height
	if( !(GetStyle()&TLHS_SHOWHEAD) )
		return 0;

	return m_nHeaderHeight;
}

int CTreeListHeaderCtrl::GetHeaderWidth()
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::GetHeaderWidth", __DATE__,__FILE__, __LINE__);
	// get header width
	ASSERT( m_pTreeListCtrl != NULL );
	return m_pTreeListCtrl->GetWidth();
}

BOOL CTreeListHeaderCtrl::GetHeaderRect( CRect& rcHeader, int iCol )
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::GetHeaderRect", __DATE__,__FILE__, __LINE__);
	// get header rect
	CPtrArray&			arColumns	= m_pTreeListCtrl->m_arColumns;
	CArray<int,int>&	arShows		= m_pTreeListCtrl->m_arShows;

	BOOL bFound = FALSE;
	int nPerfix = 0;
	for( int iShow=0; iShow<arShows.GetSize(); iShow++ )
	{
		CTreeListColumnInfo* pColumnInfo;

		int iColumn = arShows[iShow];
		pColumnInfo = (CTreeListColumnInfo*)arColumns[iColumn];
		if( iColumn == iCol )
		{
			bFound = TRUE;
			rcHeader.SetRect( 0, 0, pColumnInfo->m_nWidth, GetHeaderHeight() );
			rcHeader.OffsetRect( nPerfix, 0 );
			break;
		}

		nPerfix += pColumnInfo->m_nWidth;
	}

	return bFound;
}

BOOL CTreeListHeaderCtrl::OnEraseBkgnd(CDC* pDC) 
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::OnEraseBkgnd", __DATE__,__FILE__, __LINE__);
	// do not erase bkgnd
	return TRUE;
}

void CTreeListHeaderCtrl::OnPaint() 
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::OnPaint", __DATE__,__FILE__, __LINE__);
	// paint header control
	CPaintDC dc(this);

	if( m_bOffScreenBuffer )
	{
		CTreeListDC TreeListDC( &dc );
		DrawCtrl( &TreeListDC );
	}
	else
	{
		DrawCtrl( &dc );
	}
}

BOOL CTreeListHeaderCtrl::DrawCtrl( CDC* pDC )
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::DrawCtrl", __DATE__,__FILE__, __LINE__);
	// draw control
	ASSERT_VALID( pDC );

	CRect rcClip;
	if( pDC->GetClipBox( &rcClip ) == ERROR )
		return FALSE;

	CRect rcClient;
	GetClientRect( &rcClient );

	if( m_pTreeListCtrl->GetStyle()&TLC_BKGNDCOLOR )
	{
		pDC->FillSolidRect( &rcClient, m_cr3DBkgnd );	
	}
	else
	{
		pDC->FillSolidRect( &rcClient, m_cr3DFace );
		pDC->Draw3dRect( &rcClient, m_cr3DLight, m_cr3DShadow );
	}

	DrawHeaders( pDC );

	return TRUE;
}

BOOL CTreeListHeaderCtrl::DrawHeaders( CDC* pDC )
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::DrawHeaders", __DATE__,__FILE__, __LINE__);
	// draw every column headers
	CPtrArray&			Columns = m_pTreeListCtrl->m_arColumns;
	CArray<int, int>&	Shows	= m_pTreeListCtrl->m_arShows;

	// no columns or zero size
	if( m_pTreeListCtrl->GetWidth() == 0 )
		return TRUE;

	int nWinPos = GetParent()->GetScrollPos( SB_HORZ );

	// get client area
	CRect rcClient;
	GetClientRect( &rcClient );
	
	// draw background
	CRect rcFace;
	rcFace.SetRect( 0, rcClient.top, GetHeaderWidth(), rcClient.bottom );
	rcFace.OffsetRect( -nWinPos, 0 );

	pDC->FillSolidRect( &rcFace, m_cr3DFace );
	pDC->Draw3dRect( &rcFace, m_cr3DLight, m_cr3DShadow );
	
	CTreeListColumnInfo* pColumnInfo;
	int nHeaderPos = 0;
	for( int iShow=0; iShow<Shows.GetSize(); iShow++, nHeaderPos += pColumnInfo->m_nWidth )
	{
		int iCol;
		
		// do NOT draw zero column
		iCol = Shows[iShow];  //*m_pShow[iShow];
		pColumnInfo = (CTreeListColumnInfo*)Columns[iCol];
		if( pColumnInfo->m_nWidth == 0 )
			continue;

		CRect rcHeader;
		rcHeader.SetRect( 0, 0, pColumnInfo->m_nWidth, GetHeaderHeight() );
		rcHeader.OffsetRect( -nWinPos, 0 );
		rcHeader.OffsetRect( nHeaderPos, 0 );

		// do NOT draw out of screen column
		if( rcHeader.right < rcClient.left || rcHeader.left > rcClient.right )
			continue;

		DrawHeader( pDC, rcHeader, iCol );
	}

	return TRUE;
}

BOOL CTreeListHeaderCtrl::DrawHeader( CDC* pDC, CRect rcHeader, int iCol )
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::DrawHeader", __DATE__,__FILE__, __LINE__);
	// draw one column header
	CPtrArray&			Columns = m_pTreeListCtrl->m_arColumns;
	CArray<int, int>&	Shows	= m_pTreeListCtrl->m_arShows;
	CArray<int, int>&	Sorts	= m_pTreeListCtrl->m_arSorts;
	CTreeListColumnInfo* pColumn = (CTreeListColumnInfo*)Columns[iCol];

	ASSERT( Shows.GetSize() > 0 );

	// draw drag of columns
	CRect rcBkgnd;
	rcBkgnd = rcHeader;
	if( rcBkgnd.Height() > 0 )
	{
		if( m_bDraging && iCol == m_iCol )
			pDC->Draw3dRect( rcBkgnd, 0xFF0000, 0xFF0000 );
	}

	// draw gaps between columns
	CRect rcGaps;
	rcGaps = rcHeader;
	rcGaps.DeflateRect( 0, 2, 0, 2 );
	if( rcGaps.Height() > 0 )
	{
		pDC->IntersectClipRect( rcHeader );
		pDC->IntersectClipRect( rcGaps );

		if( !m_bDraging || iCol != m_iCol )
		{
			if( iCol != Shows[0] )
				pDC->DrawEdge( rcGaps, BDR_RAISEDINNER, BF_LEFT );

			if( iCol != Shows[Shows.GetUpperBound()] )
				pDC->DrawEdge( rcGaps, BDR_RAISEDINNER, BF_RIGHT );
		}

		::ExtSelectClipRgn( pDC->m_hDC, DEFAULT_HRGN, RGN_COPY );
	}

	if( !m_bDraging || iCol != m_iCol)
		rcHeader.DeflateRect( 0, 0, 1, 1 );
	else
		rcHeader.DeflateRect( 1, 1, 0, 0 );

	rcHeader.DeflateRect( 2, 0, 3, 0 );
	if( rcHeader.Width() <= 0 || rcHeader.Height() <= 0 )
		return TRUE;

	// draw sort arrow
	int iSort = GetSortIndex( iCol );
	if( pColumn->m_dwFormat&TLF_CAPTION_SORT && Sorts.GetSize()>0 )
	{
		CRect rcSort;
		rcSort = rcHeader;
		rcSort.left = rcSort.right - TLL_WIDTH/2;

		POINT pt;
		pt.x = rcSort.left;
		pt.y = rcSort.top + ( rcSort.Height() - TLL_HEIGHT)/2;

		pDC->IntersectClipRect( rcHeader );
		pDC->IntersectClipRect( rcSort );

		if( iSort == 0 )
		{
			// first sort column
			ASSERT( pColumn->m_dwFormat&TLF_SORT_MASK );

			if( pColumn->m_dwFormat&TLF_SORT_ASC )
				m_imgSort.Draw( pDC, TLH_SORT_ASC, pt, ILD_TRANSPARENT );
			else
				m_imgSort.Draw( pDC, TLH_SORT_DESC, pt, ILD_TRANSPARENT );
		}
		else if( iSort > 0 )
		{
			// other sort column
			ASSERT( pColumn->m_dwFormat&TLF_SORT_MASK );

			if( pColumn->m_dwFormat&TLF_SORT_ASC )
				m_imgSort.Draw( pDC, TLH_SORT_ASC2, pt, ILD_TRANSPARENT );
			else
				m_imgSort.Draw( pDC, TLH_SORT_DESC2, pt, ILD_TRANSPARENT );
		}
		else
		{
			// can sort column
			ASSERT( !(pColumn->m_dwFormat&TLF_SORT_MASK) );

			m_imgSort.Draw( pDC, TLH_SORT_NO, pt, ILD_TRANSPARENT );				
		}

		::ExtSelectClipRgn( pDC->m_hDC, DEFAULT_HRGN, RGN_COPY );

		rcHeader.DeflateRect( 0, 0, TLL_WIDTH/2, 0 );
	}

	if( rcHeader.Width() <= 0 )
		return TRUE;

	// draw image
	if( pColumn->m_dwFormat&TLF_CAPTION_IMAGE && pColumn->m_iImage>=0 )
	{
		pDC->IntersectClipRect( rcHeader );

		CRect rcImage;
		if( pColumn->m_dwFormat&TLF_IMAGEALIGN_LEFT )
		{
			rcImage			= rcHeader;
			rcImage.right	= rcHeader.left + TLL_WIDTH;
			rcHeader.left	= rcHeader.left + TLL_WIDTH;
		}
		else if( pColumn->m_dwFormat&TLF_IMAGEALIGN_RIGHT )
		{
			rcImage = rcHeader;
			rcImage.left	= rcHeader.right - TLL_WIDTH;
			rcHeader.right	= rcHeader.right - TLL_WIDTH;
		}
		else if( pColumn->m_dwFormat&TLF_IMAGEALIGN_CENTER )
		{
			rcImage			= rcHeader;
			rcHeader.left	= rcHeader.right;
		}
		else
		{
			rcImage			= rcHeader;
			rcImage.right	= rcHeader.left + TLL_WIDTH;
			rcHeader.left	= rcHeader.left + TLL_WIDTH;
		}

		POINT pt;
		if( !(pColumn->m_dwFormat&TLF_IMAGEALIGN_CENTER) )
		{
			pt.x = rcImage.left;
			pt.y = rcImage.top + ( rcImage.Height() - TLL_HEIGHT )/2;
		}
		else
		{
			pt.x = rcImage.left + ( rcImage.Width() - TLL_WIDTH )/2;
			pt.y = rcImage.top + ( rcImage.Height() - TLL_HEIGHT )/2;
		}

		pDC->IntersectClipRect( rcImage );
		m_pImageList->Draw( pDC, pColumn->m_iImage, pt, ILD_TRANSPARENT );
		::ExtSelectClipRgn( pDC->m_hDC, DEFAULT_HRGN, RGN_COPY );
	}

	rcHeader.DeflateRect( 1, 0, 1, 0 );
	if( rcHeader.Width() <= 0 )
		return TRUE;

	// draw text
	if( pColumn->m_dwFormat&TLF_CAPTION_TEXT )
	{
		pDC->IntersectClipRect( rcHeader );

		int nOldMode = pDC->SetBkMode( TRANSPARENT );
		CFont* pOldFont = pDC->SelectObject( GetHeaderFont() );
		pDC->DrawText( pColumn->m_strCaption, rcHeader, pColumn->GetCaptionFormat() );
		pDC->SelectObject( pOldFont );
		pDC->SetBkMode( nOldMode );

		::ExtSelectClipRgn( pDC->m_hDC, DEFAULT_HRGN, RGN_COPY );
	}		
	return TRUE;
}

void CTreeListHeaderCtrl::PreSubclassWindow() 
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::PreSubclassWindow", __DATE__,__FILE__, __LINE__);
	// create from subclass
	CWnd::PreSubclassWindow();

	if( m_bSubclassFromCreate == FALSE )
	{
		if( !Create() )
		{
			AfxThrowMemoryException();
			return;
		}
	}

	return;
}

LRESULT CTreeListHeaderCtrl::OnLayout(WPARAM wParam, LPARAM lParam)
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::OnLayout", __DATE__,__FILE__, __LINE__);
	// releayout
	CRect* pRect= (CRect*)lParam;
	
	if( pRect->Height() >= GetHeaderHeight() )
	{
		pRect->bottom = pRect->top + GetHeaderHeight();
	}

	MoveWindow( pRect, TRUE );

	return TRUE;
}

void CTreeListHeaderCtrl::OnMouseMove(UINT nFlags, CPoint point) 
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::OnMouseMove", __DATE__,__FILE__, __LINE__);
	// mouse moving
	if( !m_bSizing && !m_bDraging )
	{
		int iFlag;
		int iCol;

		if( HitTestSize( point, &iFlag, &iCol ) )
		{
			if( iFlag & TLHT_ONHEADERSIZE )
			{
				m_bSplit = FALSE;
				SetCursor( m_hSize );
			}
			else
			{
				m_bSplit = TRUE;
				SetCursor( m_hSplit );
			}
		}
	}
	else if( m_bSizing )
	{
		DoSizing( point );
	}
	else if( m_bDraging )
	{
		DoChecking( point );
		DoDraging( point );
	}

	CWnd::OnMouseMove(nFlags, point);
}

void CTreeListHeaderCtrl::OnLButtonDown(UINT nFlags, CPoint point) 
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::OnLButtonDown", __DATE__,__FILE__, __LINE__);
	// left button down
	m_pTreeListCtrl->SetFocus();

	CWnd::OnLButtonDown(nFlags, point);

	ASSERT( m_bSizing == FALSE );

	if( !m_bSizing && !m_bDraging )
	{
		int iFlag;
		int iCol;
		
		if( HitTestSize( point, &iFlag, &iCol ) )
		{
			m_iCol = iCol;
			BeginSizing( point );
		}
		else if( HitTest( point, &iFlag, &iCol ) )
		{
			if( iFlag&TLHT_ONHEADER )
			{
				m_iCol = iCol;
				BeginDraging( point );
			}
		}
	}
}

void CTreeListHeaderCtrl::OnLButtonUp(UINT nFlags, CPoint point) 
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::OnLButtonUp", __DATE__,__FILE__, __LINE__);
	// left button up
	if( m_bSizing )
	{
		EndSizing( point );
	}

	if( m_bDraging )
	{
		EndDraging( point );
	}

	CWnd::OnLButtonUp(nFlags, point);
}

void CTreeListHeaderCtrl::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::OnLButtonDblClk", __DATE__,__FILE__, __LINE__);
	CWnd::OnLButtonDblClk(nFlags, point);

	if( !m_bSizing && !m_bDraging )
	{
		int iFlag;
		int iCol;
		
		if( HitTestSize( point, &iFlag, &iCol ) )
		{
			m_pTreeListCtrl->SetColumnWidth( iCol, TLSCW_AUTOSIZE_USEHEADER );
		}
	}
}

void CTreeListHeaderCtrl::OnCancelMode() 
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::OnCancelMode", __DATE__,__FILE__, __LINE__);
	// cancel internal event
	if( m_bSizing )
	{
		EndSizing();
	}

	if( m_bDraging )
	{
		EndDraging();
	}

	CWnd::OnCancelMode();
}

BOOL CTreeListHeaderCtrl::BeginSizing( CPoint point )
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::BeginSizing", __DATE__,__FILE__, __LINE__);
	// begin a sizing
	CPtrArray&			arColumns	= m_pTreeListCtrl->m_arColumns;
	CArray<int, int>&	arShows		= m_pTreeListCtrl->m_arShows;

	ASSERT( m_bSizing == FALSE );
	ASSERT( m_bDraging == FALSE );

	m_bSizing = TRUE;
	SetCapture();

	if( !m_bSplit )
		SetCursor( m_hSize );
	else
		SetCursor( m_hSplit );

	CTreeListColumnInfo* pColumnInfo;
	pColumnInfo = (CTreeListColumnInfo*)arColumns[m_iCol];

	int nTrackPos = GetParent()->GetScrollPos( SB_HORZ ) + point.x;
	pColumnInfo->BeginTrack( nTrackPos );

	return TRUE;
}

BOOL CTreeListHeaderCtrl::EndSizing( CPoint point )
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::EndSizing", __DATE__,__FILE__, __LINE__);
	// end a sizing
	CPtrArray&			arColumns	= m_pTreeListCtrl->m_arColumns;
	CArray<int, int>&	arShows		= m_pTreeListCtrl->m_arShows;

	m_bSizing = FALSE;

	CTreeListColumnInfo* pColumnInfo;
	pColumnInfo = (CTreeListColumnInfo*)arColumns[m_iCol];
		
	int nTrackPos = GetParent()->GetScrollPos( SB_HORZ ) + point.x;
	pColumnInfo->EndTrack( nTrackPos );

	m_pTreeListCtrl->SetAllScroll();
	m_pTreeListCtrl->RedrawWindow();

	ReleaseCapture();
	return TRUE;
}

BOOL CTreeListHeaderCtrl::DoSizing( CPoint point )
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::DoSizing", __DATE__,__FILE__, __LINE__);
	// do a sizing
	CPtrArray&			arColumns	= m_pTreeListCtrl->m_arColumns;
	CArray<int, int>&	arShows		= m_pTreeListCtrl->m_arShows;

	CTreeListColumnInfo* pColumnInfo;
	pColumnInfo = (CTreeListColumnInfo*)arColumns[m_iCol];
		
	int nTrackPos = GetParent()->GetScrollPos( SB_HORZ ) + point.x;
	pColumnInfo->DoTrack( nTrackPos );

	m_pTreeListCtrl->SetAllScroll();
	m_pTreeListCtrl->RedrawWindow();

	return TRUE;
}

BOOL CTreeListHeaderCtrl::BeginDraging( CPoint point )
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::BeginDraging", __DATE__,__FILE__, __LINE__);
	// begin a draging
	CPtrArray&			arColumns	= m_pTreeListCtrl->m_arColumns;
	CArray<int, int>&	arShows		= m_pTreeListCtrl->m_arShows;

	ASSERT( m_bSizing == FALSE );
	ASSERT( m_bDraging == FALSE );

	if( !(GetStyle()&TLHS_MASKDRAG) )
		return FALSE;

	m_bDraging = TRUE;
	SetCapture();

	CTreeListColumnInfo* pColumnInfo;
	int nPerfix = 0;
	for( int iShow=0; iShow<arShows.GetSize(); iShow++ )
	{
		int iCol = arShows[iShow];
		pColumnInfo = (CTreeListColumnInfo*)arColumns[iCol];
		
		if( iCol == m_iCol )
			break;

		nPerfix += pColumnInfo->m_nWidth;
	}

	CRect rcHeader;
	rcHeader.SetRect( 0, 0, pColumnInfo->m_nWidth, GetHeaderHeight() );
	rcHeader.OffsetRect( -m_pTreeListCtrl->GetScrollPos( SB_HORZ ), 0 );

	m_ptBegin = point;
	m_ptDelta.x = rcHeader.left - point.x ;
	m_ptDelta.y = rcHeader.top - point.y;

	Invalidate();
	UpdateWindow();

	return TRUE;
}

BOOL CTreeListHeaderCtrl::EndDraging( CPoint point )
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::EndDraging", __DATE__,__FILE__, __LINE__);
	ASSERT( m_bDraging == TRUE );
	CPtrArray&			 arColumns	= m_pTreeListCtrl->m_arColumns;
	CArray<int, int>&	 arShows	= m_pTreeListCtrl->m_arShows;

	m_bDraging = FALSE;
	ReleaseCapture();

	if( m_pDragWnd != NULL )
	{
		m_pDragWnd->Hide();
		m_pDragWnd->DestroyWindow();
		delete m_pDragWnd;
		m_pDragWnd = NULL;
	}

	if( m_pDropWnd != NULL )
	{
		m_pDropWnd->Hide();
		m_pDropWnd->DestroyWindow();
		delete m_pDropWnd;
		m_pDropWnd = NULL;
	}

	int nFlag;
	int nCol;
	if( HitTestGaps( point, &nFlag, &nCol ) )
	{
		if( nCol != m_iCol )
		{
			int iShow;
			int iDelShow = -1;
			for( iShow = 0; iShow<arShows.GetSize(); iShow++ )
			{
				if( m_iCol == arShows[iShow] )
				{
					iDelShow = iShow;
					break;
				}
			}

			if( iDelShow >= 0 )
			{
				arShows.RemoveAt( iDelShow );
			}

			int iInsShow = -1;
			for( iShow = 0; iShow<arShows.GetSize(); iShow++ )
			{
				if( nCol == arShows[iShow] )
				{
					iInsShow = iShow;
					break;
				}
			}

			if( nFlag&TLHT_ONHEADERRIGHT )
				iInsShow += 1;

			if( iInsShow >= 0 )
			{
				arShows.InsertAt( iInsShow, m_iCol );
			}
		}
	}

	Invalidate();

	m_pTreeListCtrl->Invalidate();

	return TRUE;
}

BOOL CTreeListHeaderCtrl::DoDraging( CPoint point )
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::DoDraging", __DATE__,__FILE__, __LINE__);
	CPtrArray&			arColumns	= m_pTreeListCtrl->m_arColumns;
	CTreeListColumnInfo* pColumnInfo;
	pColumnInfo = (CTreeListColumnInfo*)arColumns[m_iCol];

	if( m_pDragWnd == NULL )
	{
		int cx = GetSystemMetrics( SM_CXDRAG );
		int cy = GetSystemMetrics( SM_CYDRAG );

		ASSERT( GetStyle()*TLHS_MASKDRAG );

		CRect rcDrag;
		rcDrag.SetRect( -cx, -cy, cx, cy );
		rcDrag.OffsetRect( m_ptBegin );
		
		if( !rcDrag.PtInRect( point ) )
		{
			
			CRect rcDragWnd;
			GetHeaderRect( rcDragWnd, m_iCol );
			if( GetStyle()&TLHS_FULLDRAG )
			{
				rcDragWnd.OffsetRect( point );
				rcDragWnd.OffsetRect( m_ptDelta );
			}
			else
			{
				rcDragWnd.OffsetRect( point.x, 0 );
				rcDragWnd.OffsetRect( m_ptDelta.x, 0 );
			}
			ClientToScreen( rcDragWnd );

			m_pDragWnd = new CTLHDragWnd;
			m_pDragWnd->Create( rcDragWnd, this, m_iCol );
			m_pDragWnd->Show();
		}		
	}
	else
	{
		CRect rcDragWnd;
		GetHeaderRect( rcDragWnd, m_iCol );
		if( GetStyle()&TLHS_FULLDRAG )
		{
			rcDragWnd.OffsetRect( point );
			rcDragWnd.OffsetRect( m_ptDelta );
		}
		else
		{
			rcDragWnd.OffsetRect( point.x, 0 );
			rcDragWnd.OffsetRect( m_ptDelta.x, 0 );
		}
		ClientToScreen( rcDragWnd );

		m_pDragWnd->MoveWindow( rcDragWnd );
	}

	return TRUE;
}

BOOL CTreeListHeaderCtrl::DoChecking( CPoint point )
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::DoChecking", __DATE__,__FILE__, __LINE__);
	CPtrArray&			arColumns	= m_pTreeListCtrl->m_arColumns;
	CArray<int, int>&	arShows		= m_pTreeListCtrl->m_arShows;
	int nWinPos	= m_pTreeListCtrl->GetScrollPos( SB_HORZ );

	int nFlag;
	int nCol;
	int nPos;

	BOOL bShow;

	bShow = HitTestGaps( point, &nFlag, &nCol );

	if( bShow )
	{
		if( m_pDropWnd == NULL )
		{
			m_pDropWnd = new CTLHDropWnd;
			m_pDropWnd->Create( this );
		}

		nPos = 0;
		if( nFlag&TLHT_ONHEADERRIGHT )
		{
			for( int iShow = 0; iShow < arShows.GetSize(); iShow++ )
			{
				int iCol = arShows[iShow];

				CTreeListColumnInfo* pColumnInfo;
				pColumnInfo = (CTreeListColumnInfo*)arColumns[iCol];

				nPos = nPos + pColumnInfo->m_nWidth;

				if( iCol == nCol )
					break;
			}
		}

		nPos = nPos - nWinPos;

		CPoint pt;
		pt.x = nPos;
		pt.y = 0;

		ClientToScreen( &pt );
		m_pDropWnd->Show( pt );
	}
	else
	{
		if( m_pDropWnd != NULL )
		{
			m_pDropWnd->Hide();
		}
	}

	return TRUE;
	
	/*
		m_pDropWnd = new CTLHDropWnd;
		m_pDropWnd->Create( this );
		m_pDropWnd->Show();
	*/
}

BOOL CTreeListHeaderCtrl::EndSizing()
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::EndSizing", __DATE__,__FILE__, __LINE__);
	// end sizing
	CPoint	point;

	GetCursorPos( &point );
	ScreenToClient( &point );

	return EndSizing( point );
}

BOOL CTreeListHeaderCtrl::EndDraging()
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::EndDraging", __DATE__,__FILE__, __LINE__);
	// end draging
	CPoint point;

	GetCursorPos( &point );
	ScreenToClient( &point );

	return EndDraging( point );
}


BOOL CTreeListHeaderCtrl::HitTestSize( CPoint pt, int* pFlag, int* pCol )
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::HitTestSize", __DATE__,__FILE__, __LINE__);
	// hit test for sizing
	ASSERT( pFlag != NULL );
	ASSERT( pCol != NULL );

	CPtrArray&			 arColumns	= m_pTreeListCtrl->m_arColumns;
	CArray<int, int>&	 arShows	= m_pTreeListCtrl->m_arShows;
	int nWinPos = m_pTreeListCtrl->GetScrollPos( SB_HORZ );

	*pFlag	= 0;
	*pCol	= 0;

	CRect rcClient;
	rcClient.SetRect( 0, 0, GetHeaderWidth(), GetHeaderHeight() );
	rcClient.right += 2;	// fix right border for resize
	rcClient.OffsetRect( -nWinPos, 0 );

	// check if in client rect
	if( !rcClient.PtInRect(pt) )
		return FALSE;

	// check if on gaps
	int nPos = pt.x + nWinPos;
	int nHeaderPos = 0;
	int nSpaceMin  = 0;
	BOOL bOnSize   = FALSE;
	for( int iShow = 0; iShow<arShows.GetSize(); iShow++ )
	{
		CTreeListColumnInfo* pColumnInfo;
		int iCol;

		iCol = arShows[iShow];
		pColumnInfo = (CTreeListColumnInfo*)arColumns[iCol];

		// calculate header right border position;
		nHeaderPos = nHeaderPos + pColumnInfo->m_nWidth;

		// filter nosize column
		if( pColumnInfo->m_dwFormat&TLF_NOSIZE )
			continue;

		// is close right border enough from left
		if( nHeaderPos-3 <= nPos && nPos < nHeaderPos )
		{
			int nGap = nHeaderPos - nPos;
			if( !bOnSize )
			{
				// find first closed right border from left
				*pCol		= iCol;
				nSpaceMin	= nGap;
				bOnSize		= TRUE;
			}
			else
			{
				// find more closed right border from left
				if( nGap < nSpaceMin )
				{
					*pCol	  = iCol;
					nSpaceMin = nGap;
				}
			}
		}
		
		// is close right border enough from right
		if( nHeaderPos <= nPos && nPos <= nHeaderPos+2 )
		{
			int nGap = nPos - nHeaderPos;
			if( !bOnSize )
			{
				// find first closed right border form right
				*pCol		= iCol;
				nSpaceMin	= nGap;
				bOnSize		= TRUE;
			}
			else
			{
				// find more closed right broder from right
				if( nGap <= nSpaceMin )
				{
					*pCol	  = iCol;
					nSpaceMin = nGap;
				}
			}
		}
	} // end for

	if( bOnSize )
	{
		CTreeListColumnInfo* pSizeColumn = ( CTreeListColumnInfo* )arColumns[*pCol];
		if( pSizeColumn->m_nWidth != 0 )
			*pFlag = TLHT_ONHEADERSIZE;
		else
			*pFlag = TLHT_ONHEADERSPLIT;
	}

	return bOnSize;
}

BOOL CTreeListHeaderCtrl::HitTestGaps( CPoint pt, int* pFlag, int* pCol )
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::HitTestGaps", __DATE__,__FILE__, __LINE__);
	// hit test for show
	ASSERT( pFlag != NULL );
	ASSERT( pCol != NULL );

	CPtrArray&			arColumns = m_pTreeListCtrl->m_arColumns;
	CArray<int, int>&	arShows	  = m_pTreeListCtrl->m_arShows;
	int nWinPos = m_pTreeListCtrl->GetScrollPos( SB_HORZ );

	*pFlag	= 0;
	*pCol	= 0;

	CRect rcClient;
	rcClient.SetRect( 0, 0, GetHeaderWidth(), GetHeaderHeight() );
	rcClient.left -= 16;		// fix left border for reorder
	rcClient.right += 16;	// fix right border for reorder
	rcClient.OffsetRect( -nWinPos, 0 );

	// check if in client rect
	if( !rcClient.PtInRect( pt ) )
		return FALSE;

	// check on which column
	int nPos = pt.x + nWinPos;
	int nHeaderPos	= 0;
	int nSpaceMin	= 0;
	BOOL bOnGaps	= FALSE;
	CTreeListColumnInfo* pColumnInfo;
	int iCol;
	int nDelta;

	// check first left border;
	if( arShows.GetSize() > 0 )
	{
		iCol = arShows[0];
		nDelta = 16;
		if( nHeaderPos-nDelta <= nPos && nPos< nHeaderPos )
		{
			// find first closed first left border from left
			ASSERT( !bOnGaps );
			int nGap	= nHeaderPos - nPos;
			*pCol		= iCol;
			*pFlag		= TLHT_ONHEADERLEFT;
			nSpaceMin	= nGap;
			bOnGaps		= TRUE;
		}

		// is close right border enough from right
		CTreeListColumnInfo* pFirstColumnInfo;
		pFirstColumnInfo = (CTreeListColumnInfo*)arColumns[arShows[0]];
		nDelta = max( pFirstColumnInfo->m_nWidth/2, 16 );

		if( nHeaderPos <= nPos && nPos <= nHeaderPos+nDelta )
		{
			// find first closed first left border from right
			ASSERT( !bOnGaps );
			int nGap	= nPos - nHeaderPos;
			*pCol		= iCol;
			*pFlag		= TLHT_ONHEADERLEFT;
			nSpaceMin	= nGap;
			bOnGaps		= TRUE;
		}
	}

	// check all right border;
	for( int iShow = 0; iShow<arShows.GetSize(); iShow++ )
	{
		iCol = arShows[iShow];
		pColumnInfo = (CTreeListColumnInfo*)arColumns[iCol];

		// calculate header right border position;
		nHeaderPos = nHeaderPos + pColumnInfo->m_nWidth;

		// is close right border enough from left
		nDelta = max(pColumnInfo->m_nWidth / 2, 16 );
		if( nHeaderPos-nDelta <= nPos && nPos < nHeaderPos )
		{
			int nGap = nHeaderPos - nPos;
			if( !bOnGaps )
			{
				// find first closed right border from left
				*pCol		= iCol;
				*pFlag		= TLHT_ONHEADERRIGHT;
				nSpaceMin	= nGap;
				bOnGaps		= TRUE;
			}
			else
			{
				// find more closed right border from left
				if( nGap < nSpaceMin )
				{
					*pCol		= iCol;
					*pFlag		= TLHT_ONHEADERRIGHT;
					nSpaceMin	= nGap;
				}
			}
		}

		// is close right border enough from right
		if( iShow < arShows.GetUpperBound() )
		{
			CTreeListColumnInfo* pNextColumnInfo;
			pNextColumnInfo = (CTreeListColumnInfo*)arColumns[arShows[iShow+1]];
			nDelta = max( pNextColumnInfo->m_nWidth/2, 16 );
		}
		else
		{
			nDelta = 16;
		}

		if( nHeaderPos <= nPos && nPos <= nHeaderPos+nDelta )
		{
			int nGap = nPos - nHeaderPos;
			if( !bOnGaps )
			{
				// find first closed right border from right
				*pCol		= iCol;
				*pFlag		= TLHT_ONHEADERRIGHT;
				nSpaceMin	= nGap;
				bOnGaps		= TRUE;
			}
			else
			{
				// fine more closed right border from right
				if( nGap <= nSpaceMin )
				{
					*pCol		= iCol;
					*pFlag		= TLHT_ONHEADERRIGHT;
					nSpaceMin	= nGap;
				}
			}
		}
	} // end for
				
	return bOnGaps;
}

BOOL CTreeListHeaderCtrl::HitTest( CPoint pt, int* pFlag, int* pCol )
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::HitTest", __DATE__,__FILE__, __LINE__);
	// hit test for header
	ASSERT( pFlag != NULL );
	ASSERT( pCol != NULL );

	CPtrArray&			 arColumns	= m_pTreeListCtrl->m_arColumns;
	CArray<int, int>&	 arShows	= m_pTreeListCtrl->m_arShows;
	CTreeListColumnInfo* pColumnInfo= NULL;
	int nWinPos = m_pTreeListCtrl->GetScrollPos( SB_HORZ );

	*pFlag = 0;
	*pCol = 0;

	CRect rcClient;
	rcClient.SetRect( 0, 0, GetHeaderWidth(), GetHeaderHeight() );
	rcClient.OffsetRect( -nWinPos, 0 );

	// check if in client rect
	if( !rcClient.PtInRect(pt) )
		return NULL;

	// check if on header
	int nPerfix = 0;
	for( int iShow = 0; iShow<arShows.GetSize(); iShow++ )
	{
		int iCol = arShows[iShow];
		pColumnInfo	 = (CTreeListColumnInfo*)arColumns[iCol];

		CRect rcColumn;
		rcColumn.SetRect( 0, 0, pColumnInfo->m_nWidth, GetHeaderHeight() );
		rcColumn.OffsetRect( nPerfix, 0 );
		rcColumn.OffsetRect( -nWinPos, 0 );

		if( rcColumn.PtInRect( pt ) )
		{
			CRect rcHeader;
			
			// if on border
			rcHeader = rcColumn;
			rcHeader.DeflateRect( 2, 0, 3, 0 );
			if( rcHeader.Width()>0 )
			{
				if( !rcHeader.PtInRect( pt ) )
					*pFlag |= TLHT_ONHEADERGAPS;
			}

			// if on sort arrow
			if( pColumnInfo->m_dwFormat&TLF_CAPTION_SORT )
			{
				CRect rcSort;
				rcSort = rcHeader;
				rcSort.left = rcSort.right - TLL_WIDTH/2;
				
				if( rcSort.PtInRect( pt ) )
					*pFlag |= TLHT_ONHEADERSORT;

				rcHeader.DeflateRect( 0, 0, TLL_WIDTH/2, 0 );
			}

			// if on image
			if( rcHeader.Width()>0 && pColumnInfo->m_dwFormat&TLF_CAPTION_IMAGE )
			{
				CRect rcImage;
				if( pColumnInfo->m_dwFormat&TLF_IMAGEALIGN_LEFT )
				{
					rcImage			= rcHeader;
					rcImage.right	= rcHeader.left + TLL_WIDTH;
					rcHeader.left	= rcHeader.left + TLL_WIDTH;
				}
				else if( pColumnInfo->m_dwFormat&TLF_IMAGEALIGN_RIGHT )
				{
					rcImage			= rcHeader;
					rcImage.left	= rcHeader.right - TLL_WIDTH;
					rcImage.right	= rcHeader.right - TLL_WIDTH;
				}
				else if( pColumnInfo->m_dwFormat&TLF_IMAGEALIGN_CENTER )
				{
					rcImage			= rcHeader;
					rcHeader.left	= rcHeader.right;
				}
				else
				{
					rcImage			= rcHeader;
					rcImage.right	= rcHeader.left + TLL_WIDTH;
					rcHeader.left	= rcHeader.left + TLL_WIDTH;
				}

				if( rcImage.PtInRect( pt ) )
					*pFlag |= TLHT_ONHEADERIMAGE;
			}
			
			// if on text
			if( rcHeader.Width()>0 && pColumnInfo->m_dwFormat&TLF_CAPTION_TEXT )
			{
				if( rcHeader.PtInRect( pt ) )
					*pFlag |= TLHT_ONHEADERTEXT;
			}

			*pCol = iCol;

			return TRUE;
		}

		nPerfix += rcColumn.Width();
	}

	return FALSE;
}

int CTreeListHeaderCtrl::GetSortIndex( int iCol )
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::GetSortIndex", __DATE__,__FILE__, __LINE__);
	CArray<int, int>&	Sorts	= m_pTreeListCtrl->m_arSorts;

	int iRet = -1;

	for( int iSort = 0; iSort<Sorts.GetSize(); iSort++ )
	{
		if( iCol == Sorts[iSort] )
		{
			iRet = iSort;
			break;
		}
	}

	return iRet;
}

DWORD CTreeListHeaderCtrl::GetStyle()
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::GetStyle", __DATE__,__FILE__, __LINE__);
	return m_dwStyle;
}

DWORD CTreeListHeaderCtrl::SetStyle( DWORD dwStyle )
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::SetStyle", __DATE__,__FILE__, __LINE__);
	DWORD dwoldStyle = m_dwStyle;
	m_dwStyle = dwStyle;
	return dwoldStyle;
}

void CTreeListHeaderCtrl::OnRButtonDown(UINT nFlags, CPoint point) 
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::OnRButtonDown", __DATE__,__FILE__, __LINE__);
	m_pTreeListCtrl->SetFocus();
	
	CWnd::OnRButtonDown(nFlags, point);
}

void CTreeListHeaderCtrl::OnRButtonUp(UINT nFlags, CPoint point) 
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::OnRButtonUp", __DATE__,__FILE__, __LINE__);
	// TODO: Add your message handler code here and/or call default
	
	CWnd::OnRButtonUp(nFlags, point);
}

void CTreeListHeaderCtrl::OnRButtonDblClk(UINT nFlags, CPoint point) 
{
	IMPLEMENT_LOGGING("CTreeListHeaderCtrl::OnRButtonDblClk", __DATE__,__FILE__, __LINE__);
	CWnd::OnRButtonDblClk(nFlags, point);

	if( !m_bSizing && !m_bDraging )
	{
		int iFlag;
		int iCol;
		
		if( HitTestSize( point, &iFlag, &iCol ) )
		{
			m_pTreeListCtrl->SetColumnWidth( iCol, TLSCW_AUTOSIZE );
		}
	}
}
